/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "BrPts.h"
#include "YNShell.h"
#include "GUI_support.h"
#include "loader.h"
#include "memory.h"
#include "esp.h"
#include "global.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Viewport.h>  /* Athena Viewport Widget */ 
#include <X11/Xaw/MenuButton.h>/* Athena MenuButton Widget */
#include <X11/Xaw/SimpleMenu.h>/* Athena SimpleMenuWidget */
#include <X11/Xaw/SmeBSB.h>    /* Athena SmeBSB Widget */
#include <X11/Xaw/List.h>      /* Athena List Widget */

/* BEGIN LOCAL DEFINE */
#define DefaultBaseIndicator "0x"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern unsigned int PC;
extern Widget topLevel;
extern Widget view_istruzione;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateBrPts(void);
static void PopdownBP(Widget w,XtPointer client_data,XtPointer call_data);
static void ShowBPList(void);

static void AddBP(Widget w,XtPointer client_data,XtPointer call_data);
static void RegChoice(Widget w, XtPointer client_data, XtPointer call_data);
static void CheckChoice(Widget w, XtPointer client_data, XtPointer call_data);
static void DeleteBP(Widget w, XtPointer client_data, XtPointer call_data);
static void HardDeleteBP(void);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_BP,BPlabel;
static int CreatedBrPts=False;

static Widget PCtext;
static Widget CondReg,CondCheck,Condvalue;
static Widget listBPs;
static String *BPs;

static unsigned int DeleteListIndexBP;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateBrPts(void)
     /*
     ** Create the widgets for the editor
     */
{
  int i;
  char textname[ShortBufferLength],label[ShortBufferLength];
  Widget form_BP;
  Widget PClabel,Condlabel;
  Widget menureg;
  Widget menucheck;
  Widget *Regs,*Checks;
  Widget AddBreakpoint,ViewBP;
  Widget cancel;

  XawTextBlock add_block;

  XtTranslations Translations; /* to parse the translation table */

  Regs=(Widget *)malloc((MaxReg+2)*sizeof(Widget));
  Checks=(Widget *)malloc(3*sizeof(Widget));

  pshell_BP = XtVaCreatePopupShell(
	"xmasBreakpoints",
	transientShellWidgetClass,
	topLevel,NULL);
        form_BP = XtVaCreateManagedWidget(
           "form_BP",
	   formWidgetClass,
	   pshell_BP,NULL);
	BPlabel = XtVaCreateManagedWidget(
	   "BPlabel",
	   labelWidgetClass,
	   form_BP,NULL);
	PClabel =  XtVaCreateManagedWidget(
	   "PClabel",
	   labelWidgetClass,
	   form_BP,NULL);
	PCtext = XtVaCreateManagedWidget(
	   "PCtext",
	   asciiTextWidgetClass,
	   form_BP,
	   XtNeditType, XawtextEdit,NULL);
	Condlabel =  XtVaCreateManagedWidget(
	   "Condlabel",
	   labelWidgetClass,
	   form_BP,NULL);
	CondReg = XtVaCreateManagedWidget(
	     "CondReg",
	     menuButtonWidgetClass,
	     form_BP,
	     XtNmenuName,"menureg",NULL);
	     menureg = XtCreatePopupShell(
		  "menureg",
		  simpleMenuWidgetClass,
		  CondReg,NULL,0);
	     for (i=0; i < MaxReg+2; i++) {
	       sprintf(textname,"reg%d",i);
	       switch (i) {
	       case RegHi: sprintf(textname,"hi"); sprintf(label,"hi"); break;
	       case RegLo: sprintf(textname,"lo");sprintf(label,"lo");  break;
	       default: sprintf(textname,"reg%d",i) ;sprintf(label,"$%x",i); break;
	       }
	       Regs[i] = XtVaCreateManagedWidget(
		  textname,
		  smeBSBObjectClass,
		  menureg,
		  XtNlabel,label,NULL);
	       XtAddCallback(Regs[i],XtNcallback,RegChoice,(XtPointer)0);
	     }
	CondCheck = XtVaCreateManagedWidget(
	     "CondCheck",
	     menuButtonWidgetClass,
	     form_BP,
	     XtNmenuName,"menucheck",NULL);
	     menucheck = XtCreatePopupShell(
		  "menucheck",
		  simpleMenuWidgetClass,
		  CondCheck,NULL,0);
	     for (i=0; i < 3; i++) {
	       sprintf(textname,"test%d",i);
	       switch (i) {
	       case 0: sprintf(textname,"greater"); sprintf(label,">"); break;
	       case 1: sprintf(textname,"less"); sprintf(label,"<"); break;
	       case 2: sprintf(textname,"equal"); sprintf(label,"="); break;
	       }
	       Checks[i] = XtVaCreateManagedWidget(
		  textname,
		  smeBSBObjectClass,
		  menucheck,
		  XtNlabel,label,NULL);
	       XtAddCallback(Checks[i],XtNcallback,CheckChoice,(XtPointer)0);
	     }
	Condvalue = XtVaCreateManagedWidget(
	   "Condvalue",
	   asciiTextWidgetClass,
	   form_BP,
	   XtNeditType, XawtextEdit,NULL);
	AddBreakpoint = XtVaCreateManagedWidget(
	   "AddBreakpoint",
	   commandWidgetClass,
	   form_BP,NULL);
	ViewBP = XtVaCreateManagedWidget(
           "ViewBP",
           viewportWidgetClass,
	   form_BP,
	   NULL);
	   listBPs = XtVaCreateManagedWidget(
	       "listBPs",
	       listWidgetClass,
	       ViewBP,NULL);
	cancel = XtVaCreateManagedWidget(
	   "cancel",
	   commandWidgetClass,
	   form_BP,NULL);


   Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
   XtOverrideTranslations(PCtext,Translations);
   XtOverrideTranslations(Condvalue,Translations);

   XtAddCallback(AddBreakpoint,XtNcallback,AddBP,NULL);
   XtAddCallback(listBPs,XtNcallback,DeleteBP,NULL);
   XtAddCallback(cancel,XtNcallback,PopdownBP,NULL);

   /*
   ** Provide a default setting for the editor
   */
   add_block.firstPos = 0;
   add_block.format = XawFmt8Bit;
   add_block.length = (int)strlen(DefaultBaseIndicator);
   add_block.ptr = DefaultBaseIndicator;
   XawTextReplace(PCtext,add_block.firstPos,add_block.length,&add_block);
   XawTextSetInsertionPoint(PCtext,add_block.length);
   XawTextReplace(Condvalue,add_block.firstPos,add_block.length,&add_block);
   XawTextSetInsertionPoint(Condvalue,add_block.length);


   BPs=malloc(getMaxBreakLine()*sizeof(String));
   for (i=0; i<getMaxBreakLine(); i++)
    BPs[i]=malloc(MediumBufferLength);
   ShowBPList();

   CreatedBrPts=True; /* remeber that it is created */
}

/*
** Callback
*/
static void RegChoice(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the register choosed by the user
     */
{
  String label;

  XtVaGetValues(w,XtNlabel,&label,NULL);
  XtVaSetValues(CondReg,XtNlabel,label,NULL);
}

/*
** Callback
*/
static void CheckChoice(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the test choosed by the user
     */
{
  String label;

  XtVaGetValues(w,XtNlabel,&label,NULL);
  XtVaSetValues(CondCheck,XtNlabel,label,NULL);
}


/*
** Action
*/
void PopupBrPts(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup the editor, if needed create it
     */
{
  if (!CreatedBrPts)
    CreateBrPts();
  Calculate_relative_shell_position_and_set(pshell_BP);
  XtPopup(pshell_BP,XtGrabNone);
  UpdateBPlabel();
}

/*
** Callback
*/
static void PopdownBP(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the pshell_BP
     */
{
  XtPopdown(pshell_BP);
}

/*
** Callback
*/
static void AddBP(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Add a breakpoint
     */
{
  int i,readBP;
  int reg,valuetest=0;
  unsigned int numericPC;
  char testkind='\0';
  String line;

  XtVaGetValues(PCtext,XtNstring,&line,NULL);
  if (sscanf(line,"%x",&numericPC) > 0) {
    if (numericPC < getMaxAddress()) {
      XtVaGetValues(CondReg,XtNlabel,&line,NULL);
      switch (*line) {
      case '$': sscanf(line+1,"%x",&reg); break;
      case 'H':
      case 'h': reg=RegHi; break;
      case 'L':
      case 'l': reg=RegLo; break;
      default: reg=-1; break;
      }
      XtVaGetValues(CondCheck,XtNlabel,&line,NULL);
      switch (*line) {
      case '<':
      case '>':
      case '=': testkind=*line; break;
      default: testkind='?'; break;
      }
      XtVaGetValues(Condvalue,XtNstring,&line,NULL);
      if (sscanf(line,"%8x",&valuetest) != 1)
      valuetest=-1;
      i=0;
      while (i < getMaxBreakLine()) {
	getBreakLinePC(i,&readBP);
	if (readBP == -1) {
	  setBreakLine(i,numericPC,reg,valuetest,testkind);
	  break;
	}
	i++;
      }
      if (i < getMaxBreakLine()) {
	if (numericPC == PC)
	  IstructionViewer(view_istruzione,PC);
	ShowBPList();
      }
      else
	ErrorMessage(BrPtsNoMoreRoom,NULL,NULL,0);
    }
    else
      ErrorMessage(BrPtsPCOutOfBound,NULL,NULL,getMaxAddress()-1);
  }
  else
    ErrorMessage(BrPtsPCNull,NULL,NULL,0);
}

static void ShowBPList(void)
     /*
     ** Show the BP list with updated values
     */
{
  unsigned int i;
  int value, reg, cost;
  char testkind;
  char valueLabel[SmallBufferLength],regLabel[SmallBufferLength],costLabel[SmallBufferLength];

  for (i=0; i < getMaxBreakLine(); i++) {
    getBreakLine(i,&value,&reg,&cost,&testkind);
    if (value == -1)
      sprintf(valueLabel,"---");
    else
      sprintf(valueLabel,"%.3x",value);
    switch (reg) {
    case -1: sprintf(regLabel,"----"); break;
    case RegHi: sprintf(regLabel,"hi  "); break; /* "hi  " is awefull */
    case RegLo: sprintf(regLabel,"lo  "); break; /* "lo  " is awefull */
    default: sprintf(regLabel,"$%-3x",reg); break;
    }
    if (cost == -1)
      sprintf(costLabel,"----------");
    else
      sprintf(costLabel,"0x%.8x",cost);
    sprintf(BPs[i],"Program counter 0x%s check register %s %c %s",
	    valueLabel,regLabel,testkind,costLabel);
  }
  XawListChange(listBPs,BPs,getMaxBreakLine(),-1,TRUE);
}

void UpdateBP(void)
     /*
     ** Write all the availableBP widget with the relative value
     */
{
  if (CreatedBrPts == True)
    ShowBPList();
}

void UpdateBPlabel(void)
     /*
     ** Show the BP status
     */
{
  if (CreatedBrPts == True) {
    if (getBP())
      XtVaSetValues(BPlabel,XtNlabel,"BreakPoints status: Active",NULL);
    else
      XtVaSetValues(BPlabel,XtNlabel,"BreakPoints status: Disabled",NULL);
  }
}

/*
** Callback
*/
static void DeleteBP(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Choose which BP delete and call the YNshell to ask
     */
{
  XawListReturnStruct *list_of_BP;

  list_of_BP=XawListShowCurrent(listBPs);
  DeleteListIndexBP=list_of_BP->list_index;
  PopupYN(HardDeleteBP,"Are you sure to delete this breakpoint?",OnMousePosition,w);
}

static void HardDeleteBP(void)
     /*
     ** Do the hard work to delete the BP choosed
     */
{
  unsigned int BPpc;

  getBreakLinePC(DeleteListIndexBP,&BPpc);
  clearBreakLine(DeleteListIndexBP);
  compactBreakLine(DeleteListIndexBP); /* avoid hole list */
  ShowBPList();
  if (BPpc == PC) /* update the display if needed */
      IstructionViewer(view_istruzione,PC);
}
