/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "Calc.h"
#include "GUI_support.h"
#include "esp.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/keysymdef.h>
#include <X11/Shell.h>         /* Athena Shell Widget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command Widget */
#include <X11/Xaw/Repeater.h>  /* Athena Repeater Widget */
#include <X11/Xaw/Label.h>     /* Athena Label Widget */

/* BEGIN LOCAL DEFINE */
#define DecimalMode     "Dec"
#define HexadecimalMode "Hex"
#define BinaryMode      "Bin"
#define Dec 1
#define Hex 2
#define Bin 3

#define first_operand  0
#define second_operand 1
#define length_buf_op 64
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateCalc(void);
static void Cancel(Widget w,XtPointer client_data,XtPointer call_data);
static void Digits(Widget w,XtPointer client_data,XtPointer call_data);
static void DeleteDigit(Widget w,XtPointer client_data,XtPointer call_data);
static void ClearDigit(Widget w,XtPointer client_data,XtPointer call_data);
static void Base(Widget w,XtPointer client_data,XtPointer call_data);
static void Operator(Widget w,XtPointer client_data,XtPointer call_data);
static void Equal(Widget w,XtPointer client_data,XtPointer call_data);

static void Key(Widget w,XEvent *event,String *params,Cardinal *num_params);

static void Calcola(void);

static void clean_number(void);
static void update_number(void);
static void add_digit_to_number(char *digit);
static void delete_a_digit_from_number(void);
static int  get_value_from_number(int first_second);
static void write_to_display(int string_op,int value);
static void convertformat2(int base);

static void BinKeyboard(void);
static void HexKeyboard(void);
static void DecKeyboard(void);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget delete_digit,clear,equal;
static Widget divide,add,sub,multiply;
static Widget pshell_Calc,display;
static int CreatedCalc=False;
static char op[2][length_buf_op];
static int Baseop[2];
static Widget zero,one,two,three,four,five,six,seven,eight,nine;
static Widget a,b,c,d,e,f;
static int must_clear=False,base_mode=Hex;
static unsigned int first_second_op=first_operand;
static char operator;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

/*
** NB: must_clear is used to know when a digit start a new number, only after a
**     operator
*/

static void CreateCalc(void)
     /*
     ** Create the widgets for the Calc shell
     */
{
  static Widget form_Calc,cancel,form_digits;
  static Widget dhob;
  static Widget and,or,exor;

  XtAccelerators Accelerators; /* to parse the accelerators table */

  XtActionsRec AllActions[] = { /* action to add */
    {"Key",Key}
  };

  pshell_Calc = XtVaCreatePopupShell(
	   "xmasCalc",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   form_Calc = XtVaCreateManagedWidget(
              "form_Calc",
	      formWidgetClass,
	      pshell_Calc,NULL);
	   display = XtVaCreateManagedWidget(
              "display",
	      labelWidgetClass,
	      form_Calc,
	      XtNjustify,XtJustifyRight,
	      XtNwidth,250,
	      NULL);
	   dhob = XtVaCreateManagedWidget(
              "dhob",
	      repeaterWidgetClass,
	      form_Calc,
	      NULL);
	   clear = XtVaCreateManagedWidget(
              "clear",
	      commandWidgetClass,
	      form_Calc,
	      NULL);
	   delete_digit = XtVaCreateManagedWidget(
              "delete_digit",
	      commandWidgetClass,
	      form_Calc,
	      NULL);
	   form_digits = XtVaCreateManagedWidget(
              "form_digits",
	      formWidgetClass,
	      form_Calc,
	      NULL);
	      c = XtVaCreateManagedWidget(
	         "c",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      d = XtVaCreateManagedWidget(
	         "d",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      e = XtVaCreateManagedWidget(
	         "e",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      f = XtVaCreateManagedWidget(
	         "f",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      eight = XtVaCreateManagedWidget(
	         "eight",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      nine = XtVaCreateManagedWidget(
	         "nine",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      a = XtVaCreateManagedWidget(
	         "a",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      b = XtVaCreateManagedWidget(
	         "b",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      four = XtVaCreateManagedWidget(
	         "four",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      five = XtVaCreateManagedWidget(
	         "five",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      six = XtVaCreateManagedWidget(
	         "six",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      seven = XtVaCreateManagedWidget(
	         "seven",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      zero = XtVaCreateManagedWidget(
	         "zero",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      one = XtVaCreateManagedWidget(
	         "one",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	      two = XtVaCreateManagedWidget(
	         "two",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       three = XtVaCreateManagedWidget(
	         "three",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       divide = XtVaCreateManagedWidget(
	         "divide",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       multiply = XtVaCreateManagedWidget(
	         "multiply",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       sub = XtVaCreateManagedWidget(
	         "sub",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       add = XtVaCreateManagedWidget(
	         "add",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       equal = XtVaCreateManagedWidget(
	         "equal",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       and = XtVaCreateManagedWidget(
                 "and",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       or = XtVaCreateManagedWidget(
		 "or",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	       exor = XtVaCreateManagedWidget(
		 "exor",
		 commandWidgetClass,
		 form_digits,
		 NULL);
	   cancel = XtVaCreateManagedWidget(
	      "cancel",
	      commandWidgetClass,
	      form_Calc,
	      NULL);

   /* add actions */
   XtAppAddActions(app_context, AllActions, XtNumber(AllActions));
   /*
   ** Calc Callback
   */
   XtAddCallback(cancel,XtNcallback,Cancel,(XtPointer)0);
   XtAddCallback(zero,XtNcallback,Digits,(XtPointer)"0");
   XtAddCallback(one,XtNcallback,Digits,(XtPointer)"1");
   XtAddCallback(two,XtNcallback,Digits,(XtPointer)"2");
   XtAddCallback(three,XtNcallback,Digits,(XtPointer)"3");
   XtAddCallback(four,XtNcallback,Digits,(XtPointer)"4");
   XtAddCallback(five,XtNcallback,Digits,(XtPointer)"5");
   XtAddCallback(six,XtNcallback,Digits,(XtPointer)"6");
   XtAddCallback(seven,XtNcallback,Digits,(XtPointer)"7");
   XtAddCallback(eight,XtNcallback,Digits,(XtPointer)"8");
   XtAddCallback(nine,XtNcallback,Digits,(XtPointer)"9");
   XtAddCallback(a,XtNcallback,Digits,(XtPointer)"a");
   XtAddCallback(b,XtNcallback,Digits,(XtPointer)"b");
   XtAddCallback(c,XtNcallback,Digits,(XtPointer)"c");
   XtAddCallback(d,XtNcallback,Digits,(XtPointer)"d");
   XtAddCallback(e,XtNcallback,Digits,(XtPointer)"e");
   XtAddCallback(f,XtNcallback,Digits,(XtPointer)"f");

   XtAddCallback(divide,XtNcallback,Operator,(XtPointer)"/");
   XtAddCallback(multiply,XtNcallback,Operator,(XtPointer)"*");
   XtAddCallback(sub,XtNcallback,Operator,(XtPointer)"-");
   XtAddCallback(add,XtNcallback,Operator,(XtPointer)"+");
   XtAddCallback(and,XtNcallback,Operator,(XtPointer)"&");
   XtAddCallback(or,XtNcallback,Operator,(XtPointer)"|");
   XtAddCallback(exor,XtNcallback,Operator,(XtPointer)"^");

   XtAddCallback(equal,XtNcallback,Equal,(XtPointer)0);

   XtAddCallback(dhob,XtNcallback,Base,(XtPointer)0);
   XtAddCallback(delete_digit,XtNcallback,DeleteDigit,(XtPointer)0);
   XtAddCallback(clear,XtNcallback,ClearDigit,(XtPointer)0);

   /* to install accelerators */   
   Accelerators = XtParseAcceleratorTable
    ("#override\n\
        <KeyPress>: Key()");
    XtVaSetValues(cancel,XtNaccelerators,Accelerators,NULL);
    XtInstallAccelerators(form_Calc,cancel);

   clean_number();
   CreatedCalc=True;
}

/*
** Action
*/
static void Key(Widget w, XEvent *event, String *params,Cardinal *num_params)
     /*
     ** Dispatch the keyboard event
     */
{
  XKeyReleasedEvent *key = (XKeyReleasedEvent *) event;
  KeySym *keysym=NULL;
  XComposeStatus *status_in_out=NULL;
  char carattere[16]={'\0'};

  XLookupString(key,carattere,sizeof(carattere),keysym,status_in_out);
  switch (*carattere) {
  case '\b': /* <- key */
  case 127: /*Canc/Del key*/XtCallCallbacks(delete_digit,XtNcallback,0); break;
  case 'C': XtCallCallbacks(clear,XtNcallback,0); break;
  case '/': XtCallCallbacks(divide,XtNcallback,0); break;
  case '*': XtCallCallbacks(multiply,XtNcallback,0); break;
  case '-': XtCallCallbacks(sub,XtNcallback,0); break;
  case '+': XtCallCallbacks(add,XtNcallback,0); break;
  case 13: /* Enter key */
  case '=': XtCallCallbacks(equal,XtNcallback,0); break;
  case '0': XtCallCallbacks(zero,XtNcallback,0); break;
  case '1': XtCallCallbacks(one,XtNcallback,0); break;
  case '2': XtCallCallbacks(two,XtNcallback,0); break;
  case '3': XtCallCallbacks(three,XtNcallback,0); break;
  case '4': XtCallCallbacks(four,XtNcallback,0); break;
  case '5': XtCallCallbacks(five,XtNcallback,0); break;
  case '6': XtCallCallbacks(six,XtNcallback,0); break;
  case '7': XtCallCallbacks(seven,XtNcallback,0); break;
  case '8': XtCallCallbacks(eight,XtNcallback,0); break;
  case '9': XtCallCallbacks(nine,XtNcallback,0); break;
  case 'a': XtCallCallbacks(a,XtNcallback,0); break;
  case 'b': XtCallCallbacks(b,XtNcallback,0); break;
  case 'c': XtCallCallbacks(c,XtNcallback,0); break;
  case 'd': XtCallCallbacks(d,XtNcallback,0); break;
  case 'e': XtCallCallbacks(e,XtNcallback,0); break;
  case 'f': XtCallCallbacks(f,XtNcallback,0); break;
  }
}

/*
** Action
*/
void PopupCalc(Widget w, XEvent *event, String *params,Cardinal *num_params)
     /*
     ** Popup the Calc shell
     */
{
  if(!CreatedCalc)
    CreateCalc();
  Calculate_relative_shell_position_and_set(pshell_Calc);
  XtPopup(pshell_Calc,XtGrabNone); /* get the focus */
  update_number();
}

/*
** Callback
*/
static void Cancel(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the Calc shell
     */
{
  XtPopdown(pshell_Calc);
}

/*
** Callback
*/
static void Digits(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Add the digit to the number to display
     */
{
  char *string_digit = (char *) client_data;

  if (must_clear == True) {
    clean_number();
    must_clear=False;
  }
  add_digit_to_number(string_digit);
  update_number();
}

/*
** Callback
*/
static void DeleteDigit(Widget w,XtPointer client_data,XtPointer call_data)
     /*
     ** Delete a digit from the number
     */
{
  delete_a_digit_from_number();
  update_number();
}

/*
** Callback
*/
static void ClearDigit(Widget w,XtPointer client_data,XtPointer call_data)
     /*
     ** Clean the display
     */
{
  clean_number();
  update_number();
  operator='\0';
}

/*
** Callback
*/
static void Base(Widget w,XtPointer client_data,XtPointer call_data)
     /*
     ** Switch the base from decimal and hexdecimal or viceversa
     */
{
  String label;

  XtVaGetValues(w,XtNlabel,&label,NULL);
  switch(*label) {
  case 'D':
  case 'd': convertformat2(Hex);
    HexKeyboard();
    XtVaSetValues(w,XtNlabel,HexadecimalMode,NULL);
    base_mode=Hex;
    break;
  case 'H':
  case 'h': convertformat2(Bin);
    BinKeyboard();
    XtVaSetValues(w,XtNlabel,BinaryMode,NULL);
    base_mode=Bin;
    break;
  case 'B':
  case 'b': convertformat2(Dec);
    DecKeyboard();
    XtVaSetValues(w,XtNlabel,DecimalMode,NULL);
    base_mode=Dec;
    break;
  default: HexKeyboard();
    XtVaSetValues(w,XtNlabel,HexadecimalMode,NULL);
    base_mode=Hex;
    break;
  }
}

static void BinKeyboard(void)
     /*
     ** Select the key for the Bin mode
     */
{
  XtSetSensitive(a,False);
  XtSetSensitive(b,False);
  XtSetSensitive(c,False);
  XtSetSensitive(d,False);
  XtSetSensitive(e,False);
  XtSetSensitive(f,False);
  XtSetSensitive(nine,False);
  XtSetSensitive(eight,False);
  XtSetSensitive(seven,False);
  XtSetSensitive(six,False);
  XtSetSensitive(five,False);
  XtSetSensitive(four,False);
  XtSetSensitive(three,False);
  XtSetSensitive(two,False);
}
static void HexKeyboard(void)
     /*
     ** Select the key for the Hex mode
     */
{
  XtSetSensitive(a,True);
  XtSetSensitive(b,True);
  XtSetSensitive(c,True);
  XtSetSensitive(d,True);
  XtSetSensitive(e,True);
  XtSetSensitive(f,True);
  XtSetSensitive(nine,True);
  XtSetSensitive(eight,True);
  XtSetSensitive(seven,True);
  XtSetSensitive(six,True);
  XtSetSensitive(five,True);
  XtSetSensitive(four,True);
  XtSetSensitive(three,True);
  XtSetSensitive(two,True);
}

static void DecKeyboard(void)
     /*
     ** Select the key for the Dec mode
     */
{
  XtSetSensitive(a,False);
  XtSetSensitive(b,False);
  XtSetSensitive(c,False);
  XtSetSensitive(d,False);
  XtSetSensitive(e,False);
  XtSetSensitive(f,False);
  XtSetSensitive(nine,True);
  XtSetSensitive(eight,True);
  XtSetSensitive(seven,True);
  XtSetSensitive(six,True);
  XtSetSensitive(five,True);
  XtSetSensitive(four,True);
  XtSetSensitive(three,True);
  XtSetSensitive(two,True);
}

/*
** Callback
*/
static void Operator(Widget w,XtPointer client_data,XtPointer call_data)
     /*
     ** Capture the operator pressed and calculate a reltive total if the
     ** espression is composed
     ** NB: 1 + 2 + 3 = 6
     */
{
  char *operation = (char *) client_data;

  Baseop[first_second_op]=base_mode;
  if (operator == '\0') {
    operator=*operation;
    first_second_op=second_operand;
  }
  else {
    Calcola();
    operator=*operation;
    first_second_op=second_operand;
    clean_number();
  }   
  must_clear=True;  
}

/*
** Callback
*/
static void Equal(Widget w,XtPointer client_data,XtPointer call_data)
     /*
     ** Calculate the result of the operation, when the user click
     ** on equal command
     */
{
  /* this set the base of the second operand */
  Baseop[first_second_op]=base_mode;
  Calcola();
  operator='\0';
}

static void Calcola(void)
     /*
     ** Calculate the elementary total and store it in first op
     */
{
  int op1,op2,numeric_value=0;
  unsigned int valid_operator=True;

  op1=get_value_from_number(first_operand);
  op2=get_value_from_number(second_operand);
  switch (operator) {
  case '/': numeric_value = op1 / op2; break;
  case '*': numeric_value = op1 * op2; break;
  case '+': numeric_value = op1 + op2; break;
  case '-': numeric_value = op1 - op2; break;
  case '&': numeric_value = op1 & op2; break;
  case '|': numeric_value = op1 | op2; break;
  case '^': numeric_value = op1 ^ op2; break;
  default: valid_operator=False;
    break;
  }
  if (valid_operator == True) {
    first_second_op=first_operand;
    write_to_display(first_operand,numeric_value);
    update_number();
    must_clear=True;
  }
}

static void clean_number(void)
     /*
     ** Clean the number to display
     */
{
  memset(op[first_second_op],'\0',length_buf_op);
}

static void update_number(void)
     /*
     ** Update the label of display
     */
{
  XtVaSetValues(display,XtNlabel,op[first_second_op],NULL);
}

static void add_digit_to_number(char *digit)
     /*
     ** Add the last digit
     */
{
   if (!((strlen(op[first_second_op]) == 0) && (*digit == '0')))
    /*
    ** add a zero only if isn't the first digit, don't allow number like
    ** 012 00f with zero before any other digit
    */
     if (strlen(op[first_second_op]) < 20)  /* dump the 21th digits */
       switch (base_mode) {
       case Dec: if (!((*digit >= '0') && (*digit <= '9')))
	 break;
       case Hex: strcat(op[first_second_op],digit);
	 break;
       case Bin: strcat(op[first_second_op],digit);
	 break;
       } 
}

static void delete_a_digit_from_number(void)
     /*
     ** Delete the last digit
     */
{
  int length;

  length=strlen(op[first_second_op]);
  if ((length > 0) && (length <= length_buf_op))
    memset(&op[first_second_op][length-1],'\0',(unsigned int)length_buf_op-length);
   else
     clean_number();
}

static int get_value_from_number(int first_second)
     /*
     ** Return the value with attention at the base
     */
{
  int ret;

  switch (Baseop[first_second]) {
  case Dec: sscanf(op[first_second],"%20d",&ret); break;
  case Hex: sscanf(op[first_second],"%8x",&ret); break;
  case Bin: ret=bit2int(op[first_second]); break;
  }
  return(ret);
}

static void write_to_display(int string_op,int value)
     /*
     ** Convert from numeric to string fromat
     */
{
  switch (base_mode) {
  case Hex: sprintf(op[string_op],"%8x",value); break;
  case Dec: sprintf(op[string_op],"%20d",value); break;
  case Bin: stampabitinstring((unsigned int)value,op[string_op]); break;
  }
}

static void convertformat2(int base)
     /*
     ** Convert the base of the display between Hex and Dec
      */
{
  String number_in_display;
  int numeric_display=0;
  char convert[length_buf_op]="";

  XtVaGetValues(display,XtNlabel,&number_in_display,NULL);
  switch (base) {
  case Hex:
    if (sscanf(number_in_display,"%d",&numeric_display) > 0 )
      /* convert only if there are valid input characters */
      sprintf(convert,"%8x",numeric_display);
  break;
  case Dec:
    numeric_display=bit2int(number_in_display);
    sprintf(convert,"%10d",numeric_display);
  break;
  case Bin:
    if (sscanf(number_in_display,"%8x",&numeric_display) > 0 )
      stampabitinstring((unsigned int)numeric_display,convert);
    break;
  default: ErrorMessage(CalcConvertError,NULL,NULL,0); break;
  }
  Baseop[first_second_op]=base;
  /* update with the new correct format string */
  strcpy(op[first_second_op],convert);
  XtVaSetValues(display,XtNlabel,convert,NULL);
}
