/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "ChangeEntry.h"
#include "GUI_support.h"
#include "policy.h"
#include "cache.h"
#include "memory.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Repeater.h>  /* Athena Repeater Widget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/List.h>       /* Athena List Widget */
#include <X11/Shell.h>         /* Athena ShellWidget */

/* BEGIN LOCAL DEFINE */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern Widget list_cache; /* to get the widget struct */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void ChangeValidorDirty(Widget w, XtPointer client_data, XtPointer call_data);
static void DoneChangeEntry(Widget w, XtPointer client_data, XtPointer call_data);
static void PopdownChangeEntry(Widget w, XtPointer client_data, XtPointer call_data);
static void CreateChangeEntry(void);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_changeentry,ldclabel,tagtext,datatext;
static Widget validcommand,dirtycommand,keepcommand,demandtext;
static int CreatedChangeEntry=False;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateChangeEntry(void)
{
  static Widget form_changeentry;
  static Widget taglabel,datalabel;
  static Widget validlabel,dirtylabel,keeplabel;
  static Widget demandlabel,done_changeentry,cancel_changeentry;

  XtTranslations Translations; /* to parse the translation table */

  pshell_changeentry = XtVaCreatePopupShell(
	"xmasChangeCacheEntry",
	transientShellWidgetClass,
	topLevel,NULL);
        form_changeentry = XtVaCreateManagedWidget(
           "form_changeentry",
	   formWidgetClass,
	   pshell_changeentry,NULL);
	ldclabel = XtVaCreateManagedWidget(
	     "ldclabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	taglabel = XtVaCreateManagedWidget(
	     "taglabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	tagtext = XtVaCreateManagedWidget(
	     "tagtext",
	     asciiTextWidgetClass,
	     form_changeentry,
	     XtNeditType, XawtextEdit,NULL);
	datalabel = XtVaCreateManagedWidget(
	     "datalabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	datatext = XtVaCreateManagedWidget(
	   "datatext",
	   asciiTextWidgetClass,
	   form_changeentry,
	   XtNeditType, XawtextEdit,
	   NULL);
	validlabel = XtVaCreateManagedWidget(
	     "validlabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	validcommand = XtVaCreateManagedWidget(
	   "validrepeater",
	   repeaterWidgetClass,
	   form_changeentry,
	   NULL);
	dirtylabel = XtVaCreateManagedWidget(
	     "dirtylabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	dirtycommand = XtVaCreateManagedWidget(
	   "dirtyrepeater",
	   repeaterWidgetClass,
	   form_changeentry,
	   NULL);
	keeplabel = XtVaCreateManagedWidget(
	     "keeplabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	keepcommand = XtVaCreateManagedWidget(
	   "keeprepeater",
	   repeaterWidgetClass,
	   form_changeentry,
	   NULL);
	demandlabel = XtVaCreateManagedWidget(
	     "demandlabel",
	     labelWidgetClass,
	     form_changeentry,NULL);
	demandtext = XtVaCreateManagedWidget(
	   "demandtext",
	   asciiTextWidgetClass,
	   form_changeentry,
	   XtNeditType, XawtextEdit,
	   NULL);
	done_changeentry = XtVaCreateManagedWidget(
	   "done_changeentry",
	   commandWidgetClass,
	   form_changeentry,NULL);
	cancel_changeentry = XtVaCreateManagedWidget(
	   "cancel_changeentry",
	   commandWidgetClass,
	   form_changeentry,NULL);

   /* Change Entry callbacks */
   XtAddCallback(cancel_changeentry,XtNcallback,PopdownChangeEntry,(XtPointer)0);
   XtAddCallback(done_changeentry,XtNcallback,DoneChangeEntry,topLevel);
   XtAddCallback(validcommand,XtNcallback,ChangeValidorDirty,validcommand);
   XtAddCallback(dirtycommand,XtNcallback,ChangeValidorDirty,dirtycommand);
   XtAddCallback(keepcommand,XtNcallback,ChangeValidorDirty,keepcommand);

   Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
   XtOverrideTranslations(tagtext, Translations);
   XtOverrideTranslations(datatext, Translations);
   XtOverrideTranslations(demandtext, Translations);
   CreatedChangeEntry=True;
}

/*
** Callback
*/
static void PopdownChangeEntry(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Darken the fabolous shell for change an entry of the cache
     */
{
  XtPopdown(pshell_changeentry);
}

/*
** Callback
*/
static void DoneChangeEntry(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Darken the fabolous shell for change an entry of the cache
     */
{
  XawListReturnStruct *list_struct;
  unsigned int Ldc;
  String strvalue;
  int datavalue;
  unsigned int tagvalue,validbit,dirtybit,keepbit,demandvalue;

  XtPopdown(pshell_changeentry);
  list_struct=XawListShowCurrent(list_cache);
  Ldc=(unsigned int)list_struct->list_index;

  XtVaGetValues(validcommand,XtNlabel,&strvalue,NULL);
  sscanf(strvalue,"%8x",(int *)&validbit);
  if (validbit != 0) {
    XtVaGetValues(tagtext,XtNstring,&strvalue,NULL);
    sscanf(strvalue,"%8x",(int *)&tagvalue);
    if (getpolpos(tagvalue,False) == Ldc) {
      settag(Ldc,tagvalue);
      XtVaGetValues(datatext,XtNstring,&strvalue,NULL);
      sscanf(strvalue,"%8x",(int *)&datavalue);
      setdata(Ldc,datavalue);
      /*XtVaGetValues(validcommand,XtNlabel,&strvalue,NULL);
	sscanf(strvalue,"%8x",(int *)&validbit);*/
      setvalid(Ldc,validbit);
      XtVaGetValues(dirtycommand,XtNlabel,&strvalue,NULL);
      sscanf(strvalue,"%8x",(int *)&dirtybit);
      setdirty(Ldc,dirtybit);
      XtVaGetValues(keepcommand,XtNlabel,&strvalue,NULL);
      sscanf(strvalue,"%8x",(int *)&keepbit);
      setKeep(Ldc,keepbit);
      XtVaGetValues(demandtext,XtNstring,&strvalue,NULL);
      sscanf(strvalue,"%8x",(int *)&demandvalue);
      setdemand(Ldc,demandvalue);
      reshow_cache();
      if ((getModoScrittura() != WriteBack) && (validbit != 1) &&
	  (dirtybit != 1)) {
	memwrite(tag(Ldc),datavalue);
	reshow_memory();
      }
    }
    else
      ErrorMessage(CETagWrong,NULL,NULL,(int)tagvalue);
  }
  else
    ErrorMessage(CEValidBit,NULL,NULL,0);
}


/*
** Callback
*/
static void ChangeValidorDirty(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** change the label 0/1 of the widget valid or dirty
     */
{
  Widget command = (Widget) client_data;
  String ptrlabel;
  unsigned int valuelabel;

  XtVaGetValues(command,XtNlabel,&ptrlabel,NULL);
  sscanf(ptrlabel,"%1d",(int *)&valuelabel);
  switch (valuelabel) {
  case 1: XtVaSetValues(command,XtNlabel,"0",NULL); break;
  case 0: XtVaSetValues(command,XtNlabel,"1",NULL); break;
  }
}

/*
** Action
*/
void ChangeEntry(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup a shell for change an entry in the cache
     ** action associated to Ctrl Btn1 in cache list
     */
{
  String string,etichetta;
  XawListReturnStruct *list_struct;
  unsigned int Ldc;

  etichetta=(String)malloc(sizeof(char)*64);
  string=(String)malloc(sizeof(char)*64);
  if (CreatedChangeEntry == False)
    CreateChangeEntry();
  list_struct=XawListShowCurrent(list_cache);
  Ldc=(unsigned int)list_struct->list_index;
  sprintf(string,"Change cache entry: #0x%-3x",Ldc);
  XtVaSetValues(ldclabel,XtNlabel,string,NULL);
  sprintf(etichetta,"0x%.8x",tag(Ldc));
  XtVaSetValues(tagtext,XtNstring,etichetta,NULL);
  XawTextSetInsertionPoint(tagtext,(int)strlen(etichetta));
  sprintf(etichetta,"0x%.8x",data(Ldc));
  XtVaSetValues(datatext,XtNstring,etichetta,NULL);
  XawTextSetInsertionPoint(datatext,(int)strlen(etichetta));
  sprintf(string,"%d",(int)valid(Ldc));
  XtVaSetValues(validcommand,XtNlabel,string,NULL);
  sprintf(string,"%d",(int)dirty(Ldc));
  XtVaSetValues(dirtycommand,XtNlabel,string,NULL);
  sprintf(string,"%u",getKeep(Ldc));
  XtVaSetValues(keepcommand,XtNlabel,string,NULL);
  sprintf(etichetta,"0x%.8x",(int)demand(Ldc));
  XtVaSetValues(demandtext,XtNstring,etichetta,NULL);
  XawTextSetInsertionPoint(demandtext,(int)strlen(etichetta));
  Calculate_relative_shell_position_and_set(pshell_changeentry);
  XtPopup(pshell_changeentry,XtGrabNone);
  free(string);
  free(etichetta);
}
