/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "Choose.h"
#include "GUI_support.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>         /* Athena Shell Widget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command Widget */
#include <X11/Xaw/Repeater.h>  /* Athena Repeater Widget */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */

/* BEGIN LOCAL DEFINE */
#define LFCRconversion "CRLF"
#define LFonly         "LF"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateChoose(void);
static void PopdownChoose(Widget w,XtPointer client_data,XtPointer call_data);
static void ChangePrinter(Widget w,XtPointer client_data,XtPointer call_data);
static void Done(Widget w,XtPointer client_data,XtPointer call_data);
static void LFCR(Widget w,XtPointer client_data,XtPointer call_data);
static void ChooseActionYes(Widget w,XEvent *event,String *params,
			    Cardinal *num_params);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_Choose,chooser,text_printer,lf_cr;
static Widget Choose_done,Choose_cancel;
static int CreatedChoose=False;
/* for execute the done function get from PopupChoose */
static void (*exec_function)(char *, char *,unsigned int);
static unsigned int AutomaticLF_LFCR=True;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void CreateChoose(void)
     /*
     ** Create the widgets for the YNShell
     */
{
  static Widget form_Choose;

  XtTranslations Translations; /* to parse the translation table */

  XtActionsRec AllActions[] = { /* action to add */
    {"ChooseActionYes",ChooseActionYes},
  };

  pshell_Choose = XtVaCreatePopupShell(
	   "xmasPrint",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   form_Choose = XtVaCreateManagedWidget(
              "form_Choose",
	      formWidgetClass,
	      pshell_Choose,NULL);
	   chooser=XtVaCreateManagedWidget(
	      "chooser",
	      repeaterWidgetClass,
	      form_Choose,NULL);
	   text_printer=XtVaCreateManagedWidget(
              "text_printer",
	      asciiTextWidgetClass,
	      form_Choose,
	      XtNeditType, XawtextEdit,
	      XtNfromHoriz,chooser,NULL);
	   lf_cr = XtVaCreateManagedWidget(
	      "lf_cr",
	      repeaterWidgetClass,
	      form_Choose,
	      XtNfromVert,chooser,
	      XtNwidth,50,NULL);
	   Choose_done = XtVaCreateManagedWidget(
	      "Choose_done",
	      commandWidgetClass,
	      form_Choose,
	      XtNfromVert,lf_cr,NULL);
	   Choose_cancel = XtVaCreateManagedWidget(
	      "Choose_cancel",
	      commandWidgetClass,
	      form_Choose,
	      XtNfromVert,lf_cr,
	      XtNfromHoriz,Choose_done,NULL);

   XtAppAddActions(app_context, AllActions, XtNumber(AllActions));

   /*
   ** ChooseShell Callback
   */
   XtAddCallback(chooser,XtNcallback,ChangePrinter,(XtPointer)0);
   XtAddCallback(Choose_done,XtNcallback,Done,(XtPointer)0);
   XtAddCallback(Choose_cancel,XtNcallback,PopdownChoose,(XtPointer)0);
   XtAddCallback(lf_cr,XtNcallback,LFCR,(XtPointer)0);

   Translations = XtParseTranslationTable
     ("<KeyPress>Return: ChooseActionYes()\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
   XtOverrideTranslations(text_printer, Translations);

   CreatedChoose=True;
}

/*
** Callback
*/
static void ChangePrinter(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Switch text <--> postscript
     */
{
  String string;
  char gotPrinterType[256];

  XtVaGetValues(w,XtNlabel,&string,NULL);
  strcpy(gotPrinterType,string);
  switch (*string) {
  case 'T': XtVaSetValues(w,XtNlabel,PS_PRINTER,NULL);
    XtVaSetValues(lf_cr,XtNsensitive,False,NULL); break;
  case 'P': XtVaSetValues(w,XtNlabel,TEXT_PRINTER,NULL);
    XtVaSetValues(lf_cr,XtNsensitive,True,NULL); break;
  default: XtVaSetValues(w,XtNlabel,TEXT_PRINTER,NULL);
    ErrorMessage(ErrorOnPrinterType,gotPrinterType,NULL,0);
  }
}

void PopupChoose(Widget relative,void (*funzione)(char *, char *, unsigned int))
     /*
     ** Popup the shell, set the label into Choose_label and get the function
     ** to execute in response to the press of done widget
     */
{
  if (funzione == NULL)
    ErrorMessage(ChooseFunctionError,NULL,NULL,0);
  else {
    if(!CreatedChoose)
      CreateChoose();
    Shell_position_generic(relative,pshell_Choose);
    XtPopup(pshell_Choose,XtGrabNone);/* XtGrabNonexclusive);  get the focus */
    exec_function=funzione; /* function to execute for done widget */
  }
}

static void PopdownChoose(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the ChooseShell
     */
{
  XtPopdown(pshell_Choose);
}

/*
** Callback
*/
static void Done(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the ChooseShell and execute the function associated
     */
{
  String type,printer;

  XtVaGetValues(chooser,XtNlabel,&type,NULL);
  XtVaGetValues(text_printer,XtNstring,&printer,NULL);
  if (strlen(printer) > 0) {
    exec_function(type,printer,AutomaticLF_LFCR); /* execute the function for the Done job */
    XtCallCallbacks(Choose_cancel,XtNcallback,0);
  }
  else
    ErrorMessage(ErrorOnPrinterName,printer,NULL,0);
}

static void ChooseActionYes(w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
     /*
     ** shortcut for Yes widget with keypressed y
     */
{
  XtCallCallbacks(Choose_done,XtNcallback,0);
}

/*
** Callback
*/
static void LFCR(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the ChooseShell
     */
{
  String label;

  XtVaGetValues(lf_cr,XtNlabel,&label,NULL);
  if (*label=='C') {
    XtVaSetValues(lf_cr,XtNlabel,LFonly,NULL);
    AutomaticLF_LFCR=False;
  }
  else {
    XtVaSetValues(lf_cr,XtNlabel,LFCRconversion,NULL);
    AutomaticLF_LFCR=True;
  }
}
