/*
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>

#include "images/xlogo11" /* for the mark */

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "config.h"
#include "Email.h"
#include "ErrorMessage.h"
#include "OpenSave.h"
#include "Print.h"
#include "Choose.h"
#include "YNShell.h"
#include "Edit.h"
#include "GUI_support.h"
#include "loader.h"
#include "policy.h"
#include "memory.h"
#include "general_support.h"
#include "esp.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
/*
** The elementary program
*/
#define BasicProgram "\nhalt"
#define BasicProgramCounter "0x"
/*
** Editor Auto Save Time Interval
*/
#define TimeEditorAutoSave 1000*60*3 /* three minutes */
#define NoNameAvailable "No Name Available Now"

#define DirectoryNotAvailable "No directory initialized"

#define MessageforEditYN "The file exists, overwrite ?"

#define NUM_MENU_FILE    6
#define NUM_MENU_BUFFERS 4
#define DIR_SIZE 512
#define PC_SIZE   16
#define NAME_SIZE 256

#define TXTmode  0
#define HTMLmode 1
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern unsigned int PC;
extern Widget view_istruzione;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateEdit(void);
static void PopdownEdit(Widget w, XtPointer client_data, XtPointer call_data);
static void Savecallback(Widget w, XtPointer client_data, XtPointer call_data);
static void SavecallbackHTML(Widget w, XtPointer client_data, XtPointer call_data);
static void SavecallbackEMAIL(Widget w, XtPointer client_data, XtPointer call_data);
static void EDoneEdit(Widget w, XtPointer client_data, XtPointer call_data);
static void Checker(Widget w, XtPointer client_data, XtPointer call_data);
static int Syntax_Check(void);
static int generic_text_reset(Widget text,char *ptr);
static void Open(Widget w, XtPointer client_data, XtPointer call_data);
static void SaveAs(void);
static int Saver(char *complete_filename, unsigned int mode);
static void Save(Widget w, XtPointer client_data, XtPointer call_data);
static void CleanEditor(Widget w, XtPointer client_data, XtPointer call_data);
static void ChoosePrint(Widget w, XtPointer client_data, XtPointer call_data);

static void SaveAsHTML(String filename);
static void SaveEdit(String filename);
static void LoadEdit(String filename);
static char *new_extract_row(char *input, char *output);
static char *extract_significative_row(char *input, char *output);
static void AutoSave(void);

static int newbufferInfo(String name, String path, String startPC);

/*
** BUFFER PROTOTYPES
*/
static void update_buffer(Widget w, XtPointer client_data,XtPointer call_data);
static void initialize_buffer(void);
static void update_name_buffer(String filename);

static void SetNameBuffer(unsigned int index, String filename);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
struct buffer
{
  String text;
  char cwd[LongBufferLength];
  char pc[SmallBufferLength];
};

static Widget pshell_edit,textedit,textPC;
static int CreatedEdit=False;
static XtIntervalId EditorAutoSave;
static String globalfilename=NULL;

static Widget entryMenuBuffers[NUM_MENU_BUFFERS];
static struct buffer entry_buffer[NUM_MENU_BUFFERS];
static unsigned int ActualBuffer=0;
static Pixmap mark;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void EAStimer(XtPointer w, XtIntervalId *id)
     /*
     ** Do the Editor Auto Save
     ** NB: WRITE EVERY TimeEditorAutoSave time, don't test the change
     */
{
  EditorAutoSave=XtAppAddTimeOut(XtWidgetToApplicationContext(w),
				 TimeEditorAutoSave,EAStimer,w);
  if (CreatedEdit == True)
    AutoSave();
}

int EditorActiveEAS(void)
     /*
     ** Active the Editor Auto Save timer
     */
{
  EAStimer(topLevel,NULL);
  return(True);
}

int EditorDisableEAS(void)
     /*
     ** Disable the Editor Auto Save timer
     */
{
  XtRemoveTimeOut(EditorAutoSave);
  return(True);
}

static void CreateEdit(void)
     /*
     ** Create the widgets for the editor section
     */
{
  int i;
  char name[ShortBufferLength];
  static Widget form_edit,doneedit,canceledit,labelPC;
  static Widget checker,clean_editor;
  static Widget print_command;
  static Widget buttonbuffers,menubuffers;
  static Widget buttonfile,menufile,entryMenuFile[NUM_MENU_FILE];
  XawTextBlock add_block;

  XtTranslations Translations; /* to parse the translation table */

  pshell_edit = XtVaCreatePopupShell(
	"xmasEditor",
	transientShellWidgetClass,
	topLevel,NULL);
        form_edit = XtVaCreateManagedWidget(
           "form_edit",
	   formWidgetClass,
	   pshell_edit,NULL);
	buttonfile = XtVaCreateManagedWidget(
	   "buttonfile",
	   menuButtonWidgetClass,
	   form_edit,
	   XtNmenuName,"menufile",
	   NULL);
	   menufile = XtVaCreatePopupShell(
	      "menufile",
	      simpleMenuWidgetClass,
	      buttonfile,
	      NULL);
	   for (i=0; i < NUM_MENU_FILE; i++) {
		sprintf(name,"file%d",i+1);
	        entryMenuFile[i] = XtVaCreateManagedWidget(name,
				       smeBSBObjectClass,menufile,
				       XtNwidth,50,NULL);
	      }
	buttonbuffers = XtVaCreateManagedWidget(
	   "buttonbuffers",
	   menuButtonWidgetClass,
	   form_edit,
	   XtNfromHoriz,buttonfile,
	   XtNmenuName,"menubuffers",
	   NULL);
	   menubuffers = XtVaCreatePopupShell(
	      "menubuffers",
	      simpleMenuWidgetClass,
	      buttonbuffers,
	      XtNwidth,256, /* set the lenght of the viewable buffer name */
	      NULL);
	      for (i=0; i < NUM_MENU_BUFFERS; i++) {
		sprintf(name,"buffer%d",i+1);
	        entryMenuBuffers[i] = XtVaCreateManagedWidget(name,
				       smeBSBObjectClass,menubuffers,
				       XtNwidth,50,NULL);
		XtAddCallback(entryMenuBuffers[i],XtNcallback,update_buffer,(
			      XtPointer)i);
	      }
        print_command=XtVaCreateManagedWidget(
	   "print_command",
	   commandWidgetClass,
	   form_edit,
	   XtNfromHoriz,buttonbuffers,NULL);
	textedit = XtVaCreateManagedWidget(
	   "textedit",
	   asciiTextWidgetClass,
	   form_edit,
	   XtNeditType, XawtextEdit,
	   XtNwidth,300,
	   XtNheight,400,
	   XtNfromVert,buttonfile,
	   XtNscrollVertical,XawtextScrollAlways,NULL);
	labelPC = XtVaCreateManagedWidget(
	   "labelPC",
	   labelWidgetClass,
	   form_edit,
	   XtNfromVert,textedit,NULL);
	textPC = XtVaCreateManagedWidget(
	   "textPC",
	   asciiTextWidgetClass,
	   form_edit,
	   XtNeditType, XawtextEdit,
	   XtNfromVert,textedit,
	   XtNfromHoriz,labelPC,NULL);
	doneedit = XtVaCreateManagedWidget(
	   "doneedit",
	   commandWidgetClass,
	   form_edit,
	   XtNfromVert,textedit,
	   XtNfromHoriz,textPC,NULL);
	checker = XtVaCreateManagedWidget(
	   "checker",
	   commandWidgetClass,
	   form_edit,
	   XtNfromVert,labelPC,NULL);
	clean_editor = XtVaCreateManagedWidget(
	   "clean_editor",
	   commandWidgetClass,
	   form_edit,
	   XtNfromHoriz,checker,
	   XtNfromVert,labelPC,NULL);
	canceledit = XtVaCreateManagedWidget(
	   "canceledit",
	   commandWidgetClass,
	   form_edit,
	   XtNfromVert,checker,NULL);

   /*
   ** General callback
   */
   XtAddCallback(entryMenuFile[0],XtNcallback,Open,(XtPointer)0);
   XtAddCallback(entryMenuFile[1],XtNcallback,Save,(XtPointer)0);
   XtAddCallback(entryMenuFile[2],XtNcallback,Savecallback,(XtPointer)0);
   XtAddCallback(entryMenuFile[3],XtNcallback,SavecallbackHTML,(XtPointer)0);
   XtAddCallback(entryMenuFile[4],XtNcallback,SavecallbackEMAIL,(XtPointer)0);
   XtAddCallback(entryMenuFile[5],XtNcallback,PopdownEdit,(XtPointer)0);

   XtAddCallback(canceledit,XtNcallback,PopdownEdit,(XtPointer)0);
   XtAddCallback(doneedit,XtNcallback,EDoneEdit,(XtPointer)0);
   XtAddCallback(checker,XtNcallback,Checker,(XtPointer)0);
   XtAddCallback(clean_editor,XtNcallback,CleanEditor,(XtPointer)0);
   XtAddCallback(print_command,XtNcallback,ChoosePrint,(XtPointer)0);

   Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
   XtOverrideTranslations(textPC, Translations);

   /*
   ** Provide a default setting for the editor
   */
   add_block.firstPos = 0;
   add_block.format = XawFmt8Bit;

   /*
   ** prepare the pc text widget
   */
   add_block.length = (int)strlen(BasicProgramCounter);
   add_block.ptr = BasicProgramCounter;
   XawTextReplace(textPC,add_block.firstPos,add_block.length,&add_block);
   XawTextSetInsertionPoint(textPC,add_block.length);

   /*
   ** prepare the program text widget
   */
   add_block.length = (int)strlen(BasicProgram);
   add_block.ptr = BasicProgram;
   XawTextReplace(textedit,add_block.firstPos,add_block.length,
			&add_block);
   /*
   ** read the bitmap to mark the actual buffer
   */
   mark=XCreateBitmapFromData(XtDisplay(topLevel),
			      RootWindowOfScreen(XtScreen(topLevel)),
			      xlogo11_bits, xlogo11_width,xlogo11_height);
   initialize_buffer();
   LoadEditStatus();
   CreatedEdit=True; /* remeber that it is created */
}

/*
** Callback
*/
static void ChoosePrint(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Call the pshell for print the progrm into editor
     */
{
  PopupChoose(w,Print);
}

/*
** Callback
*/
static void CleanEditor(Widget w,XtPointer client_data,	XtPointer call_data)
     /*
     ** Clean up the editor and put the insert point at the begin
     ** Clean up the text entry for the PC and put the insert position right
     */
{
  XawTextSetInsertionPoint(textedit,0); /* 0 => initial position */
  generic_text_reset(textedit,BasicProgram);
  generic_text_reset(textPC,BasicProgramCounter);
  XawTextSetInsertionPoint(textPC,(int)strlen(BasicProgramCounter));
  memset(entry_buffer[ActualBuffer].cwd,'\0',DIR_SIZE);
  XtFree(entry_buffer[ActualBuffer].text);
  entry_buffer[ActualBuffer].text=XtNewString(NoNameAvailable);
  XtVaSetValues(entryMenuBuffers[ActualBuffer],XtNlabel,NoNameAvailable,NULL);
}

/*
** Callback
*/
static void Checker(Widget w,XtPointer client_data,
		    XtPointer call_data)
     /*
     ** Check the program of the editor => call Syntax_Check()
     */
{
  if (Syntax_Check() == True)
    ErrorMessage(EditCheckOK,NULL,NULL,0/*(int)cont-1*/);
}

static int Syntax_Check(void)
     /*
     ** Check the program into the editor and return the correctness
     ** True: the program is correct \
     **                              |--> correctness
     ** False: the program is wrong  /
     ** NB: if the editor is empty answer that the program is ok (light BUG)
     */
{
  String strp;
  unsigned int istr,cont=1;
  int error=False;
  char row[ShortBufferLength];

  XtVaGetValues(textedit,XtNstring,&strp,NULL);
  while (((strp=extract_significative_row(strp,row)) != NULL) &&
	 (error == False)) {
    if((error=String2Istr(row,&istr)) != 0) /* decode istructions */
      break;
  /*
  ** clean up the row for avoid that some old characters
  ** appear into new strings
  */
    memset(row,'\0',sizeof(row));
    cont++;
  }
  switch (error) {
  case False: return(True); /* no error found */
  case EditIstrError: ErrorMessage(EditCheckError,NULL,NULL,(int)cont);
    break;
  default: ErrorMessage(error,NULL,NULL,(int)cont);
  }
  return(False); /* the program is wrong */
}

/*
** Callback
*/
static void SavecallbackEMAIL(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Popup the shell to send buffer as email
     */
{
  String stringPC;
  char *messageBody;

  XtVaGetValues(textPC,XtNstring,&stringPC,NULL);
  messageBody=malloc(strlen(entry_buffer[ActualBuffer].text)+strlen(stringPC)+16);
  sprintf(messageBody,".PC %s\n.text\n%s",stringPC,entry_buffer[ActualBuffer].text);
  PopupEmail(messageBody);
  free(messageBody);
}

/*
** Callback
*/
static void SavecallbackHTML(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Opoup the shell to choose yhe name and save in HTML mode
     */
{
  PopupOpenSave(w,"Save File:",SaveAsHTML);
}

static void SaveAsHTML(String filename)
     /*
     ** Save the buffer in HTML mode
     ** NB: do not any kind of check!
     */
{
  String program;

  XtVaGetValues(textedit,XtNstring,&program,NULL);
  Saver(filename,HTMLmode);
}

/*
** Callback
*/
static void Savecallback(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Call SaveEdit for save the editor into a file
     */
{
  PopupOpenSave(w,"Save File:",SaveEdit);
}

static void SaveEdit(String filename)
     /*
     ** Save the editor program into a file
     */
{
  String program;

  globalfilename=filename;
  XtVaGetValues(textedit,XtNstring,&program,NULL);
  if (strlen(program) == 0) /* if there is some characters */
    ErrorMessage(EditNoProgram,filename,NULL,0);
  else {
    if (access(filename,F_OK) == 0) /* test for existence of the  file */
      PopupYN(SaveAs,MessageforEditYN,AbsoluteApplicationMiddle,NULL);
    else
      SaveAs();
  }
}

/*
** Callback
*/
static void PopdownEdit(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the editor
     */
{
  XtPopdown(pshell_edit);
}

/*
** Action
*/
void PopupEdit(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup the editor, if needed create it
     */
{
  if (!CreatedEdit)
    CreateEdit();
  Shell_position_on_right(pshell_edit);
  XtPopup(pshell_edit,XtGrabNone);
}

static int generic_text_reset(Widget text,char *ptr)
     /*
     ** Erase the widget text and place the string referred by ptr
     */
{
  XawTextBlock add_block;
  String oldtext;

  XtVaGetValues(text,XtNstring,&oldtext,NULL);
  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=strlen(ptr);
  add_block.ptr=ptr;
  XawTextDisableRedisplay(text);
  XawTextReplace(text,0,(XawTextPosition)strlen(oldtext),&add_block);
  XawTextEnableRedisplay(text);
  return(True);
}

int EditorGetBufferFromFile(String filename, String name, String path, unsigned int startPC)
{
  FILE *fp;
  char riga[MediumBufferLength],alfanumericPC[SmallBufferLength];
  char *punt;
  XawTextPosition EditEnd=0;
  XawTextBlock add_block;

  if (!CreatedEdit)
    CreateEdit();
  add_block.firstPos = 0;
  add_block.format = XawFmt8Bit;
  if ((fp=fopen(filename,"r")) == NULL) {
    ErrorMessage(EditFileOpenError,filename,NULL,0);
    return(False);
  }
  generic_text_reset(textedit,"\0"); /* erase the text into editor */
  XawTextDisableRedisplay(textedit);
  /*
  ** write the program in the editor
  */
  while (fgets(riga,(int)sizeof(riga),fp) != NULL)
    if (strcmp(riga,".text\n") == 0)
	break;
  while (fgets(riga,(int)sizeof(riga),fp) != NULL) {
    if (riga[0]=='B')
      punt=riga+3; 
    else
      punt=riga;
    add_block.length = (int)strlen(punt);
    add_block.ptr = punt;
    XawTextReplace(textedit,EditEnd,EditEnd,&add_block);
    EditEnd+=strlen(punt)+1;
  }
  XawTextEnableRedisplay(textedit);
  fclose(fp);
  /*
  ** update the buffer status
  */
  sprintf(alfanumericPC,"%x",startPC);
  newbufferInfo(name,path,alfanumericPC);
  return(True);
}

/*
** Callback
*/
static void EDoneEdit(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Popdown the editor and update if needed
     */
{
  String istructions,pclocal,get_istr_from_widget;
  char row[ShortBufferLength],*istrs;
  unsigned int i=0,ProgramCounter;
  int *TrMem,cont=0,error=False,rit,numericpc=-1;

  ProgramCounter=getMaxAddress();
  XtVaGetValues(textedit,XtNstring,&get_istr_from_widget,NULL);
  /*
  ** without it generate a bug with istructions and Syntax_Check()
  */
  istrs=(char *)malloc(sizeof(char)*(strlen(get_istr_from_widget)+1));
  strcpy(istrs,get_istr_from_widget);
  istructions=istrs;
  XtVaGetValues(textPC,XtNstring,&pclocal,NULL);
  sscanf(pclocal,"%8x",&numericpc);
  if (strlen(istructions) == 0) /* the program is empty */
    error=EditNoProgram;
  else if (strlen(pclocal) <= 2) /* invalid PC */
    error=EditPCUnset;
  else if ((numericpc >= getMaxAddress()) && (numericpc >= 0))
    error=EditPCoutMemory;
  else {
    sscanf(pclocal+2,"%x",&ProgramCounter);
    if (Syntax_Check() == False) /* the program is wrong */
    error=True;
  }
  TrMem=(int *)malloc(sizeof(int)*(getMaxAddress()-ProgramCounter));
  while (((istructions=extract_significative_row(istructions,row)) != NULL) &&
	 (error == False)) {
    if (cont>=(getMaxAddress()-ProgramCounter)) { 
      error=EditMuchIstrs; /* more istr than available memory from PC */
      break;
    }
    if((rit=String2Istr(row,&TrMem[cont])) != 0) { /* decode istructions */
      error=rit;
      break;
    }
    cont++;
  }
  switch (error) {
  case False: 
    for (i=0; i<cont; i++) /* write the program into cache */
      polwritewithoutstat(ProgramCounter+i,TrMem[i]);
    reshow_cache();
    reshow_memory();
    if ((PC>=ProgramCounter) && (PC<ProgramCounter+cont))
      IstructionViewer(view_istruzione,PC);
    break;
  case True:
    break;
  default: ErrorMessage(error,NULL,NULL,cont+1);
  }
  free(TrMem);
  free(istrs);
}

/*
** Callback
*/
static void Open(Widget w, XtPointer client_data, XtPointer call_data)
{
  PopupOpenSave(w,"Open File:",LoadEdit);
}

static void LoadEdit(String filename)
     /*
     ** Load in the editor the program selected in OpenSave
     */
{
  char cwd[LongBufferLength];

  getcwd(cwd,sizeof(cwd));
  EditorGetBufferFromFile(filename,filename,cwd,0);
}

static char *new_extract_row(char *input,char *output)
     /*
     ** extract a row from the input string and write into output
     ** return NULL if the string is empty or a comment
     */
{
  char *new_line;

  if ((*input) == '\0')
    /*
    ** stand for the end of the string
    */
    return(NULL);
  if ((*input) == '#')
      /*
      ** the line is a comment
      */
    return(NULL);
  if ((new_line=strchr(input,'\n'))!=NULL) {
    /*
    ** \n is the end of line
    */
    strncpy(output,input,(unsigned)(new_line-input));
    output[(unsigned)(new_line-input)]='\0';
    /*
    ** to the next line
    */
    return(new_line+1);
  }
  else {
    /*
    ** \0 stand for the end of string 
    */
    strcpy(output,input);
    /*
    ** go to the end of string
    */
    return(input+strlen(input));
  }
}

static char *extract_significative_row(char *input, char *output)
     /*
     ** allow to skip sequences of \n
     */
{
  char *ret;

  ret=input; /* allow the next pass of the while */
  do {
    input=ret; /* for the next pass */
    if ((ret=new_extract_row(input,output)) == NULL)
      break;
  }
  while ((ret-input) == 1); /* 1 stand for the near char is a \n */
  return(ret);
}

static void SaveAs(void)
     /*
     ** Allow of save the program into the editor with a user name
     */
{
  Saver(globalfilename,TXTmode);
  update_name_buffer(globalfilename);
}

static void AutoSave(void)
     /*
     ** Save the program in the editor, in response to AutoSave
     */
{
  String filename;

  XtVaGetValues(entryMenuBuffers[ActualBuffer],XtNlabel,&filename,NULL);
  globalfilename=filename;
  SaveAs();
}

static int Saver(char *complete_filename, unsigned int mode)
     /*
     ** Do the hard work of save the program into a file
     */
{
  FILE *fp;
  String program,pclocal;
  unsigned int ProgramCounter;
  char Date[ShortBufferLength],Clock[ShortBufferLength];
  char directory[LongBufferLength];
  unsigned int prgLength=0;

  XtVaGetValues(textPC,XtNstring,&pclocal,NULL);
  XtVaGetValues(textedit,XtNstring,&program,NULL);
  if ((fp=fopen(complete_filename,"w")) == NULL) {
    ErrorMessage(EditFileWrong,complete_filename,NULL,0);
    return(False);
  }
  else {
    switch (mode) {
    case HTMLmode: DateTime(Date,Clock);getFileDirectory(directory);
      prgLength=strlen(program);
      fprintf(fp,
"<HTML>\n\
<HEAD>\n\
   <TITLE>%s</TITLE>\n\
   <META NAME=\"GENERATOR\" CONTENT=\"%s %s\">\n\
   <META NAME=\"AUTHOR\" CONTENT=\"%s\">\n\
   <META NAME=\"AUTHOR\" CONTENT=\"%s\">\n\
</HEAD>\n\
<BODY BGCOLOR=white>\n\
<P><A HREF=%s>Xmas</A> file name: <STRONG>%s</STRONG>\n\
<BR>Directory: %s\n\
<BR>Source size: %d byte(s)\n\
<BR>Date: %s\n\
<BR>Time: %s\n\
<BR>Source code:\n\
<PRE>\n"
,complete_filename,ProgName,VersionString,AUTHORemail,secondAUTHORemail,WEB,complete_filename,directory,prgLength,Date,Clock);
      break;
    }
    if (strlen(pclocal) > 2) { /* valid Program Counter */
      sscanf(pclocal+2,"%x",&ProgramCounter);
      fprintf(fp,".Pc %x\n",ProgramCounter);
    }
    fprintf(fp,".text\n%s",program); /* write the program */
    switch (mode) {
    case HTMLmode: fprintf(fp,"\n</PRE>\n</BODY>\n</HTML>");
			   break;
    }
    fclose(fp);
    return(True);
  }
}

void Save(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Save the program with the name of the buffer
     */
{
  char complete_filename[DIR_SIZE*2];
  String name;
  char *last_space;

  if (strlen(entry_buffer[ActualBuffer].cwd) == 0)
    PopupOpenSave(w,"Save File:",SaveEdit);
  else {
    XtVaGetValues(entryMenuBuffers[ActualBuffer],XtNlabel,&name,NULL);
    last_space=&name[strlen(name)-1];
    while (*last_space == ' ') /* go back to the last valid carachter */
      last_space--;
    *(last_space+1)='\0'; /* put the delimiter of the string */
    strcpy(complete_filename,entry_buffer[ActualBuffer].cwd);
    strcat(complete_filename,"/");
    strcat(complete_filename,name);
    if (Saver(complete_filename,TXTmode) == True)
      PopupY("Buffer saved!",OnMousePosition,w);
  }
}

void getProgram(String *program)
     /*
     ** Return a pointer to the program
     */
{
  XtVaGetValues(textedit,XtNstring,program,NULL);
}

void getFileName(String filename)
     /*
     ** Return a pointer to the file name of the program
     */
{
  String tmpString;

  XtVaGetValues(entryMenuBuffers[ActualBuffer],XtNlabel,&tmpString,NULL);
  strcpy(filename,tmpString);
}

void getFileDirectory(String directory)
     /*
     ** Return the directory for the buffer
     */
{
  if (*entry_buffer[ActualBuffer].cwd != '\0') {
    /* the direcotry is set */
    strcpy(directory,entry_buffer[ActualBuffer].cwd);
    strcat(directory,"/");
  }
  else /* when the buffer is whitout directory, return an empty string */
    strcpy(directory,DirectoryNotAvailable);
}

void getStartPC(String StartPC)
{
  String tmpStartPC;

  XtVaGetValues(textPC,XtNstring,&tmpStartPC,NULL);
  strcpy(StartPC,tmpStartPC);
}

/*
** PLEASE PAY ATTENTION
** CODE FOR BUFFERS FEATURE UNDER THIS LINE
*/

static void initialize_buffer(void)
     /*
     ** Initialize the structure for the menu entry
     */
{
  int i;

  for (i=0; i < NUM_MENU_BUFFERS; i++) {
    entry_buffer[i].text=malloc(sizeof(BasicProgram));
    strcpy(entry_buffer[i].text,BasicProgram);
    memset(entry_buffer[i].cwd,'\0',DIR_SIZE);
    memset(entry_buffer[i].pc,'\0',PC_SIZE);
  }
}

/*
** Callback
*/
static void update_buffer(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Update the text of the editor for the relative menu entry
     */
{
  int choosed = (int) client_data;
  String tmpstring;

  XtVaGetValues(textedit,XtNstring,&tmpstring,NULL);
  XtFree(entry_buffer[ActualBuffer].text);
  entry_buffer[ActualBuffer].text=XtNewString(tmpstring);
  XtVaGetValues(textPC,XtNstring,&tmpstring,NULL);
  strncpy(entry_buffer[ActualBuffer].pc,tmpstring,PC_SIZE-1);
  entry_buffer[ActualBuffer].pc[PC_SIZE-1]='\0';

  generic_text_reset(textedit,entry_buffer[choosed].text);
  generic_text_reset(textPC,entry_buffer[choosed].pc);
  XawTextSetInsertionPoint(textPC,(XawTextPosition)strlen(entry_buffer[choosed].pc));

  XtVaSetValues(entryMenuBuffers[ActualBuffer],XtNleftBitmap,NULL,NULL);
  XtVaSetValues(entryMenuBuffers[choosed],XtNleftBitmap,mark,NULL);
  ActualBuffer=choosed; /* choosed will be the next ActualBuffer */
}

static void update_name_buffer(String filename)
     /*
     ** Update the string of the menu entry and the cwd of the menu structure
     */
{
  SetNameBuffer(ActualBuffer,filename);
  getcwd(entry_buffer[ActualBuffer].cwd,
	 sizeof(entry_buffer[ActualBuffer].cwd));
}

static void SetNameBuffer(unsigned int index, String filename)
     /*
     ** Write the string filename into buffer label name
     */
{
  XtVaSetValues(entryMenuBuffers[index],XtNlabel,filename,NULL);
}

static int newbufferInfo(String name, String path, String startPC)
     /*
     ** Update all information of ActualBuffer
     */
{
  char tmpPath[DIR_SIZE];
  char tmpStartPC[PC_SIZE];
  char tmpName[NAME_SIZE];

  if (name == NULL)
    strcpy(tmpName,NoNameAvailable);
  else {
    if (strlen(name) >= NAME_SIZE)
      return(EditBufferNameTooLong);
    else
      strncpy(tmpName,name,NAME_SIZE);
  }
  if (path == NULL)
    memset(tmpPath,'\0',sizeof(tmpPath));
  else {
    if (strlen(path) >= DIR_SIZE)
      return(EditBufferPathTooLong);
    else
      strncpy(tmpPath,path,DIR_SIZE);
  }
  if (startPC == NULL)
    memset(tmpStartPC,'\0',sizeof(tmpStartPC));
  else {
    if (strlen(startPC) >= PC_SIZE)
      return(EditBufferPCTooLong);
    else
      strncpy(tmpStartPC,startPC,PC_SIZE);
  }
  SetNameBuffer(ActualBuffer,tmpName);
  strncpy(entry_buffer[ActualBuffer].cwd,tmpPath,DIR_SIZE);
  strncpy(entry_buffer[ActualBuffer].pc,tmpStartPC,PC_SIZE);
  return(True);
}

/*
** END CODE FOR BUFFERS FEATURE
*/

/*
** PLEASE PAY ATTENTION
** CODE FOR SAVING EDITOR STATUS FEATURE UNDER THIS LINE
*/

int SaveEditStatus(String XmasDir)
     /*
     ** Save the edit status:
     **     save the buffer with the relative /path/name
     **     save the selected buffer to edit
     **     save the Start PC
     ** NB: don't check the write result becuse the LoadEditStatus function
     **     check every data its read
     */
{
  FILE *fp;
  char filename[LongBufferLength];
  unsigned int i;
  String name,program,sPC;

  if (!CreatedEdit)
    return(False);
  for (i=0; i<NUM_MENU_BUFFERS; i++) {
    sprintf(filename,"%s/buffer%d",XmasDir,i);
    if ((fp=fopen(filename,"w")) != NULL) {
      XtVaGetValues(entryMenuBuffers[i],XtNlabel,&name,NULL);
      XtVaGetValues(textedit,XtNstring,&program,NULL);
      fprintf(fp,"%s\n",name);
      fprintf(fp,"%s\n",entry_buffer[i].cwd);
      /* print always a number */
      fprintf(fp,"0x%x\n",atoi(entry_buffer[i].pc));
      if (i == ActualBuffer)
	fprintf(fp,"%s",program); /* the program may be not saved */
      else
	fprintf(fp,"%s",entry_buffer[i].text);
      fclose(fp);
    }
  } 
  sprintf(filename,"%s/editor",XmasDir);
  if ((fp=fopen(filename,"w")) != NULL) {
    XtVaGetValues(textPC,XtNstring,&sPC,NULL);
    fprintf(fp,"%d",ActualBuffer);
    fclose(fp);
  }
  return(True);
}

int LoadEditStatus(void)
     /*
     ** Load the edit status:
     **     load the buffer with the relative /path/name
     **     load the selected buffer to edit
     **     load the Start PC
     */
{
  char XmasDir[XmasDirLength],filename[LongBufferLength];
  char buffername[ShortBufferLength],bufferline[LongBufferLength];
  FILE *fp;
  XawTextBlock add_block;
  int text_length;
  unsigned int i;
  String program;

  i=XmasDirStatus(XmasDir,sizeof(XmasDir));
  for (i=0; i<NUM_MENU_BUFFERS; i++) {
    sprintf(filename,"%s/buffer%d",XmasDir,i);/*make the name of buffer files*/
    if ((fp=fopen(filename,"r")) != NULL) {
      if (fgets(buffername,sizeof(buffername),fp) != NULL) {
	buffername[strlen(buffername)-1]='\0';
	SetNameBuffer(i,buffername);
      } /* == NULL stand for an error from the file => don't make anything */
      if (fgets(entry_buffer[i].cwd,sizeof(entry_buffer[i].cwd),fp) != NULL)
	entry_buffer[i].cwd[strlen(entry_buffer[i].cwd)-1]='\0';
      if (fgets(entry_buffer[i].pc,sizeof(entry_buffer[i].pc),fp) != NULL)
	entry_buffer[i].pc[strlen(entry_buffer[i].pc)-1]='\0';
      generic_text_reset(textedit,"");
      text_length=strlen("");
      add_block.firstPos=0;
      add_block.format=XawFmt8Bit;
      XawTextDisableRedisplay(textedit);
      /* if the first time the return is NULL then do nothing */
      while (fgets(bufferline,sizeof(bufferline),fp) != NULL) {
	add_block.length=strlen(bufferline);
	add_block.ptr=bufferline;
	XawTextReplace(textedit,text_length,text_length,&add_block);
	text_length+=add_block.length;
	XtVaGetValues(textedit,XtNstring,&program,NULL);
	entry_buffer[i].text=XtNewString(program);
      }
      XawTextEnableRedisplay(textedit);
      fclose(fp);
    }
  }
  sprintf(filename,"%s/editor",XmasDir);
  if ((fp=fopen(filename,"r")) != NULL) {
    if ((i=fscanf(fp,"%d",&ActualBuffer)) != -1) { /* check the read */
      if (ActualBuffer >= NUM_MENU_BUFFERS)
	ActualBuffer=0;
      generic_text_reset(textPC,entry_buffer[ActualBuffer].pc);
      XawTextSetInsertionPoint(textPC,(XawTextPosition)strlen(entry_buffer[ActualBuffer].pc));
      XtVaSetValues(textedit,XtNstring,entry_buffer[ActualBuffer].text,NULL);
      XtVaSetValues(entryMenuBuffers[ActualBuffer],XtNleftBitmap,mark,NULL);
    }
    fclose(fp);
  }
  return(True);
}

int GetNumBuffer(void)
     /*
     ** Return the total number of buffers
     */
{
  return(NUM_MENU_BUFFERS);
}

int GetBufferFilename(int num,char *filename)
     /*
     ** Return the buffer filename
     */
{
  String name;
  
  if ((num>=0) && (num<NUM_MENU_BUFFERS)) {
    XtVaGetValues(entryMenuBuffers[num],XtNlabel,&name,NULL);
    strcpy(filename,name);
    return(True);    
  }
  return(False);
}

int getBufferProgram(int num,String *program)
     /*
     ** Return the buffer program
     */
{
  if ((num>=0) && (num<NUM_MENU_BUFFERS)) {
    *program=entry_buffer[num].text;
    return(True);
  }
  return(False);
}
