/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h> /* why show gcc a warning for popen/pclose? */
#include <stdlib.h>
/*
** Awful code!
** only to avoid boring warning during compile time!
*/
FILE *popen(const char *command, const char *type);
int pclose(FILE *stream);

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Label.h>     /* Athena Label Widget */
#include <X11/Xaw/Repeater.h>  /* Athena Repeater Widget */

/*
** include of application modules
*/
#include "Email.h"
#include "opthandler.h"
#include "GUI_support.h"
#include "ErrorMessage.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define AddressWidth 300
#define SubjectWidth AddressWidth
#define BodyWidth    AddressWidth
#define BodyHeight   200

#define defaultMAILER "mail -n"
#define defaultMAILERlength 32 /*chars for mail command and -s options, etc. */
#define NoSubject "NoSubject!"
#define OptionSpace 32
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateEmail(void);
static void PopdownEmail(Widget w, XtPointer client_data, XtPointer call_data);
static void Accept(Widget w, XtPointer client_data, XtPointer call_data);
static void SubjectCallback(Widget w, XtPointer client_data, XtPointer call_data);

static int generic_text_reset(Widget text,char *ptr);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static int CreatedEmail=False;
static Widget pshellEmail;
static Widget address,subject,body;
static int SubjectFlag=True;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateEmail(void)
     /*
     ** Create the widget structure for the Sizer shell
     */
{
  static Widget formEmail,emailLabel,accept,close;
  static Widget subjectRep,bodyLabel;
  XtTranslations Translations; /* to parse the translation table */
 
  pshellEmail = XtVaCreatePopupShell(
	   "xmasEmail",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   formEmail = XtVaCreateManagedWidget(
              "form_Email",
	      formWidgetClass,
	      pshellEmail,NULL);
	   emailLabel = XtVaCreateManagedWidget(
                   "emailLabel",
		   labelWidgetClass,
		   formEmail,
		   NULL);
	   address = XtVaCreateManagedWidget(
                   "address",
		   asciiTextWidgetClass,
		   formEmail,
		   XtNeditType, XawtextEdit,
		   XtNwidth,AddressWidth,
		   XtNfromHoriz,emailLabel,
		   NULL);
	   subjectRep = XtVaCreateManagedWidget(
                   "subjectRep",
		   repeaterWidgetClass,
		   formEmail,
		   XtNfromVert,emailLabel,
		   NULL);
	   subject = XtVaCreateManagedWidget(
                   "subject",
		   asciiTextWidgetClass,
		   formEmail,
		   XtNeditType, XawtextEdit,
		   XtNwidth,SubjectWidth,
		   XtNfromHoriz,subjectRep,
		   XtNfromVert,emailLabel,
		   NULL);
	   bodyLabel = XtVaCreateManagedWidget(
		   "bodyLabel",
		   labelWidgetClass,
		   formEmail,
		   XtNfromVert,subject,
		   NULL);
	   body = XtVaCreateManagedWidget(
                   "subject",
		   asciiTextWidgetClass,
		   formEmail,
		   XtNeditType, XawtextEdit,
		   XtNscrollVertical,XawtextScrollAlways,
		   XtNwidth,BodyWidth,
		   XtNheight,BodyHeight,
		   XtNfromVert,subject,
		   XtNfromHoriz,emailLabel,
		   NULL);
	   accept = XtVaCreateManagedWidget(
                   "accept",
		   commandWidgetClass,
		   formEmail,
		   XtNvertDistance,30,
		   XtNfromVert,body,
		   NULL);
	   close = XtVaCreateManagedWidget(
                   "close",
		   commandWidgetClass,
		   formEmail,
		   XtNvertDistance,30,
		   XtNfromHoriz,accept,
		   XtNfromVert,body,
		   NULL);

  XtAddCallback(close,XtNcallback,PopdownEmail,(XtPointer)0);
  XtAddCallback(accept,XtNcallback,Accept,(XtPointer)0);
  XtAddCallback(subjectRep,XtNcallback,SubjectCallback,(XtPointer)0);

  Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
  XtOverrideTranslations(address, Translations);
  XtOverrideTranslations(subject, Translations);

  CreatedEmail=True;
}

void PopupEmail(char *message)
     /*
     ** Pop up the Email shell and initialize the status
     */
{
  if(!CreatedEmail)
    CreateEmail();
  if (message != NULL)
    generic_text_reset(body,message);
  Calculate_relative_shell_position_and_set(pshellEmail);
  XtPopup(pshellEmail,XtGrabNone); /* get the focus */
}

/*
** Callback
*/
static void PopdownEmail(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the Email shell
     */
{
  XtPopdown(pshellEmail);
}

/*
** Callback
*/
static void Accept(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Send the mail
     */
{
  String emailaddr,emailsubj,emailbody;
  char *mail=NULL;
  FILE *fp;
  char RetValue[MediumBufferLength];

  XtVaGetValues(address,XtNstring,&emailaddr,NULL);
  XtVaGetValues(subject,XtNstring,&emailsubj,NULL);
  XtVaGetValues(body,XtNstring,&emailbody,NULL);
  if (strlen(emailaddr) > 0) {
    if (getOptionFromName(EmailPreferencesFile,EmailProgOpt,RetValue,sizeof(RetValue)) == True) {
      mail=malloc(strlen(RetValue));
      strncpy(mail,RetValue+1,strlen(RetValue));
    }
    else {
      mail=malloc(defaultMAILERlength);
      strncpy(mail,defaultMAILER,strlen(defaultMAILER));
    }
      /*if (strlen(emailaddr) > 0) {*/
    mail=realloc(mail,strlen(mail)+strlen(emailaddr)+strlen(emailsubj)+OptionSpace);
    strcat(mail," ");
    strcat(mail,emailaddr);
    if (SubjectFlag == True) {
      strcat(mail," -s ");
      if (strlen(emailsubj) > 0) {
	strcat(mail,"\"");
	strcat(mail,emailsubj);
	strcat(mail,"\"");
      }
      else
	strcat(mail,NoSubject);
    }
    if ((fp=(FILE *)popen(mail,"w")) != NULL) { /* why must I do the cast? */
      fprintf(fp,"%s",emailbody);
      if (pclose(fp) == -1)
	ErrorMessage(PcloseFailure,NULL,NULL,0);
    }
    else
	ErrorMessage(CouldntSend,mail,NULL,0);
    free(mail);
    PopdownEmail(NULL,NULL,NULL);
  }
  else
    ErrorMessage(NullAddress,NULL,NULL,0);
}

static void SubjectCallback(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Enable/disable the subject asciiText
     */
{
  if (SubjectFlag == True) {
    XtSetSensitive(subject,False);
    SubjectFlag=False;
  }
  else {
    XtSetSensitive(subject,True);
    SubjectFlag=True;
    }
}

static int generic_text_reset(Widget text, char *ptr)
     /*
     ** Erase the widget text and place the string referred by ptr
     */
{
  XawTextBlock add_block;
  String oldtext;

  XtVaGetValues(text,XtNstring,&oldtext,NULL);
  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=strlen(ptr);
  add_block.ptr=ptr;
  XawTextDisableRedisplay(text);
  XawTextReplace(text,0,(XawTextPosition)strlen(oldtext),&add_block);
  XawTextEnableRedisplay(text);
  return(True);
}
