/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "config.h"
#include "Sizer.h"
#include "Email.h"
#include "pathexec.h"
#include "general_support.h"
#include "Preferences.h"
#include "Search.h"
#include "OpenSave.h"
#include "Choose.h"
#include "Print.h"
#include "MemEditor.h"
#include "menuHELP.h"
#include "YNShell.h"
#include "BrPts.h"
#include "Edit.h"
#include "ChangeEntry.h"
#include "errormessage.h"
#include "GUI.h"
#include "GUI_support.h"
#include "loader.h"
#include "policy.h"
#include "cache.h"
#include "memory.h"
#include "global.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */

/* BEGIN LOCAL DEFINE */
#define AsciiTextWidth 400
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateErrorMessage(void);
static int WriteMessage(String string_message, Widget text);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static int CreatedErrorMessage=False;
static Widget pshell_message,text_message;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateErrorMessage(void)
     /*
     ** Create the widget structure for the mail shell
     */
{
  Widget form_message,done_message;
  XtTranslations Translations; /* to parse the translation table */

  pshell_message = XtVaCreatePopupShell(
	"xmasMessage",
	transientShellWidgetClass,
	topLevel,NULL);
  form_message =  XtVaCreateManagedWidget(
           "form_message",
	   formWidgetClass,
	   pshell_message,NULL);
  text_message = XtVaCreateManagedWidget(
	   "text_message",
	   asciiTextWidgetClass,
	   form_message,
	   XtNdisplayCaret,False,
	   XtNwrap,XawtextWrapNever,
	   XtNscrollVertical,XawtextScrollWhenNeeded,
	   XtNscrollHorizontal,XawtextScrollWhenNeeded,
	   XtNwidth,430,
	   XtNheight,150,NULL);
  done_message = XtVaCreateManagedWidget(
           "done_message",
	   commandWidgetClass,
	   form_message,
	   XtNfromVert,text_message,
	   NULL);
  
  XtAddCallback(done_message,XtNcallback,PopdownMessage,0);

  Translations = XtParseTranslationTable
     ("Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)\n\
       <KeyPress>Return: ReturnActionTextMessage()");
   XtOverrideTranslations(text_message,Translations);
   Translations = XtParseTranslationTable
     ("<KeyPress>Return: ReturnActionTextMessage()");
   XtOverrideTranslations(form_message,Translations);
   XtOverrideTranslations(done_message,Translations);

   CreatedErrorMessage=True;
}

int ErrorMessageComplete(int returned, String string, String string1,
			 int number, char *file, int line, char *function)
     /*
     ** Show the message relative to returned value with the support of
     ** string and number
     */
{
  static char mess[MaxCarAvailableForMessage];
  char hexvalue[SmallBufferLength];

  if (!CreatedErrorMessage)
    CreateErrorMessage();
  switch (returned) {
  case SetRegWithoutValue: strcpy(mess,ErrorSetRegWithoutValue);
    break;
  case SetLdmWithoutValue: strcpy(mess,ErrorSetLdmWithoutValue);
    break;
  case ExecLWWrongAddres:
  case ExecSWWrongAddres: strcpy(mess,ErrorSWLWAddress);
    break;
  case LoaderIncompleteNWayError: strcpy(mess,ErrorIncompleteNWay);
    break;
  case LoaderDataError: strcpy(mess,ErrorData);
    break;
  case LoaderPCError: strcpy(mess,ErrorPC);
    break;
  case SetNWayError: strcpy(mess,ErrorNWay);
    break;
  case SetMemorySizeError: strcpy(mess,ErrorMemorySize);
    break;
  case SetCacheSizeError: strcpy(mess,ErrorCacheSize);
    break;
  case LoaderFileIncomplete: sprintf(mess,"LOAD ERROR\n%s%s\n%s",
				     ErrorFileIncomplete,string,
				     ErrorFileIncompleteExample);
    break;
  case LoaderFileError: sprintf(mess,"LOAD ERROR\n\"%s\"\n%s",string,
				ErrorFileOpen);
    break;
  case LoaderUnknownError: strcpy(mess,ErrorUnknown);
    break;
  case LoaderWhithoutRS: sprintf(mess,"%s at line %d",ErrorLoaderWhithoutRS,
				 number);
    break;
  case LoaderWhithoutRT: sprintf(mess,"%s at line %d",ErrorLoaderWhithoutRT,
				 number);
    break;
  case LoaderWhithoutRD: sprintf(mess,"%s at line %d",ErrorLoaderWhithoutRD,
				 number);
    break;
  case LoaderWhithoutIMM: sprintf(mess,"%s at line %d",ErrorLoaderWhithoutIMM,
				  number);
    break;
  case SetRegisterError:
  case ExecWriteonROR: strcpy(mess,ErrorWriteonROR);
    break;
  case LoaderExceededMaxReg:
  case ExecExceededMaxReg: sprintf(mess,"%s %d",ErrorExceededMaxReg,number);
    break;
  case LoaderExceededImmField: sprintf(mess,"%s %d",ErrorExceededImmField,number);
    break;
  case EditIstrError:
  case LoaderIstrError: sprintf(mess,"LOAD ERROR\n%s %s\n%s %s",ErrorIstrError,
				string1,ErrorIstrErrorLoader,string);
    break;
  case LoaderExceededMinShiftAmount: sprintf(mess,"LOAD ERROR\n%s %d",ErrorExceededMinSa,number);
    break;
  case LoaderExceededMaxShiftAmount: sprintf(mess,"LOAD ERROR\n%s %d",ErrorExceededMaxSa,number);
    break;
  case ExecSAOutOfRange: strcpy(mess,ErrorExecSAOutOfRange);
    break;
  case ExecDivByZero: strcpy(mess,ErrorExecDivByZero);
    break;
  case ExecSmallZeroFieldError: sprintf(mess,"%s %d",ErrorExecSmallZeroFieldError,number);
    break;
  case ExecZeroFieldError: sprintf(mess,"%s %d",ErrorExecZeroFieldError,number);
    break;
  case ExecZeroDivMultError: sprintf(mess,"%s %d",ErrorExecZeroDivMultError,number);
    break;
  case ExecIstrError: sprintf(mess,"EXECUTOR ERROR\n%s%s%x",ErrorIstrError,
			      ErrorIstrErrorExecutor,number);
    break;
  case ExecIstrLimit: sprintf(mess,"%s %d",ErrorIstrLimit,getIstrLimit());
    break;
  case ExecOutofTime: sprintf(mess,"%s %d",ErrorOutofTime,getOutofTimeLimit());
    break;
  case ExecBreakPoint: sprintf(mess,"%s 0x%x",SignalBreakPoint,number);
    break;
  case ExecHalt: strcpy(mess,SignalHalt);
    break;
  case LoaderOutPC: sprintf(mess,"LOADER:\n%s",ErrorOutPC);
    break;
  case ExecOutPC: sprintf(mess,"EXECUTOR:\n%s",ErrorOutPC);
    break;
  case DialogCacheInfo: sprintf(mess,"Entry #%x status:\n%s",number,string);
    break;
  case PoliticaUnknown:
  case ScritturaUnknown:
  case ReplaceUnknown: sprintf(mess,"%s %s\n%s",UnknownKind,string,
			       ContactAuthor);
    break;
  case Help: sprintf(mess,"%s",HelpMessage);
    break;
  case Author: sprintf(mess,"%s\n%s\nCompiled date: %s\nCompiled time: %s",
		       AuthorMessage,VersionMessage,__DATE__,__TIME__);
    break;
  case CETagWrong: sprintf(hexvalue,"0x%.8x",number);
    sprintf(mess,"%s %s",ErrorCETag,hexvalue);
    break;
  case CEValidBit: strcpy(mess,ErrorCEValidBit);
    break;
  case OUTPC: sprintf(mess,"PC=%s\n%s",string,ErrorOUTPC);
    break;
  case EditMuchIstrs: strcpy(mess,ErrorEditMuchIstrs);
    break;
  case EditCheckWrong:
  case EditIstrUnKnown: sprintf(mess,"%s %d\n%s",ErrorEditIstrUnKnown,number,
				ErrorHelptoHuman);
    break;
  case EditFileWrong: sprintf(mess,"%s\n%s",ErrorEditFileWrong,string);
    break;
  case EditNoProgram: strcpy(mess,ErrorEditNoProgram);
    break;
  case EditCheckOK: strcpy(mess,NOERRORCHECK);
    break;
  case EditPCUnset: strcpy(mess,ErrorEditPCUnset);
    break;
  case EditFileOpenError: sprintf(mess,"%s\n%s",ErrorEditFileOpenError,string);
    break;
  case EditPCoutMemory: sprintf(mess,"%s 0x%x",ErrorEditPCoutMemory,getMaxAddress()-1);
    break;
  case EditCheckError: sprintf(mess,"%s %d",ErrorEditCheckError,number);
    break;
    /*
    ** Breakpoint shell
    */
  case BrPtsPCNull: strcpy(mess,ErrorBrPtsPCNull);
    break;
  case BrPtsPCOutOfBound: sprintf(mess,"%s 0x%x",ErrorBrPtsPCOutOfBound,number);
    break;
  case BrPtsNoMoreRoom: strcpy(mess,ErrorBrPtsNoMoreRoom);
    break;
    /*
    ** YNShell
    */
  case YNFunctionError: sprintf(mess,"%s\ncalled in file %s line %d",ErrorYNFunctionError,file,line);
    break;
  case YNLabelError: sprintf(mess,"%s\ncalled in file %s line %d",ErrorYNLabelError,file,line);
    break;
  case MemEditorViewer: strcpy(mess,ErrorMemEditorViewer);
    break;
  case IncorrectStartValue: strcpy(mess,ErrorIncorrectStartValue);
    break;
  case IncorrectEndValue: strcpy(mess,ErrorIncorrectEndValue);
    break;
  case StartOutMemory: strcpy(mess,ErrorStartOutMemory);
    break;
  case EndOutMemory: strcpy(mess,ErrorEndOutMemory);
    break;
  case EndLessThanStart: strcpy(mess,ErrorEndLessThanStart);
    break;
  case MemEditorTmpFile: sprintf(mess,"%s\n%s",ErrorMemEditorTmpFile,string);
    break;
    /*
    ** Print module
    */
  case PrintTmpFileError: sprintf(mess,"%s\n%s",ErrorPrintTmpFileError,string);
    break;
  case PrintCommandText: sprintf(mess,"%s\n%s",ErrorPrintCommandTextError,
				 string);
    break;
  case PrintCommandLaTeX: sprintf(mess,"%s\n%s",ErrorPrintCommandLaTeX,string);
    break;
  case PrintCommandRM: sprintf(mess,"%s\n%s",ErrorPrintCommandRM,string);
    break;
  case PrintCommandUnlink: sprintf(mess,"%s\n%s",ErrorPrintCommandUnlink,
				   string);
    break;
  case PrintUnabletoGetTmpDir: sprintf(mess,"%s %s",ErrorPrintUnabletoGetTmpDir,string);
    break;
  case PrintUnalbleGetCommand: sprintf(mess,"%s %s",ErrorPrintUnalbleGetCommand,string1);
    break;
    /*
    ** Choose module
    */
  case ErrorOnPrinterType: strcpy(mess,ErrorErrorOnPrinterType);
    break;
  case ChooseFunctionError: sprintf(mess,"%s\ncalled in file %s line %d",ErrorChooseFunctionError,file,line);
    break;
  case ErrorOnPrinterName: strcpy(mess,ErrorErrorOnPrinterName);
    break;
    /*
    **
    */
  case DateTimeCore: sprintf(mess,"%s",string);
    break;
  case OpenSaveDeleteError: strcpy(mess,ErrorOpenSaveDeleteError);
    break;
  case OpenSaveUnableScan: sprintf(mess,"%s\n%s",ErrorOpenSaveUnableScan,string);
    break;
  case SearchInputInvalid: strcpy(mess,SearchInputInvalidError);
    break;
    /*
    ** Preferences module
    */
  case PreferencesCannotSave: sprintf(mess,"%s\n%s",ErrorPreferencesCannotSave,string);
    break;
  case PreferencesCannotEnterDirectory: strcpy(mess,ErrorPreferencesCannotEnterDirectory);
    break;
  case PreferencesCannotLoadFile: sprintf(mess,"%s %s",ErrorPreferencesCannotLoadFile,string);
    break;
    /*
    ** Path Execution module
    */
  case PathExecFileWrong: sprintf(mess,"%s\n%s",ErrorEditFileWrong,string);
    break;
    /*
    ** Email messages
    */
  case CouldntSend: sprintf(mess,"%s\n%s",ErrorCouldntSend,string);
    break;
  case PcloseFailure: strcpy(mess,ErrorPcloseFailure);
    break;
  case NullAddress: sprintf(mess,"%s\n%s or %s",ErrorNullAddress,AUTHORemail,secondAUTHORemail);
    break;
    /*
    ** Sizer messages
    */
  case SizerManualValueOutofRange:
    strcpy(mess,ErrorSizerManualValueOutofRange);
    break;

    /*
    ** Miscellanious messages
    */
  case DedicatedTo: strcpy(mess,Dedica);
    break;
  case Copyright: strcpy(mess,CopyrightMessage);
    break;
  case Birthday: strcpy(mess,MessBirthday);
    break;
  case UpdateFromOldVersion: sprintf(mess,"%s %s to\n%s",UpdateFromOldVersionError,string1,string);
    break;
  case UsingOldVersionOnRecentVersion: sprintf(mess,"%s\nRunning version %s on files of version %s",UsingOldVersionOnRecentVersionError,string,string1);
    break;
  case UnderConstruction: sprintf(mess,"%s\n%s",UnderConstructionMessage,
				 HelpMessage);
    break;
  case WelcomeCode: strcpy(mess,WelcomeMessage);
    break;
  default: sprintf(mess,"NO PANIC!!!\n%s\n\
ret: %d\n\
str: %s\n\
str1: %s\n\
number: %d\n\
in file: %s\n\
in function: %s\n\
at line: %d",
ErrorMessageFunction,returned,string,string1,number,file,function,line);
    break;
  }
  WriteMessage(mess,text_message);
  Calculate_relative_shell_position_and_set(pshell_message);
  XtPopup(pshell_message,XtGrabNone);
  return(True);
}

/*
** Callback
*/
void PopdownMessage(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Darken the fabolous shell for general message
     */
{
  XtPopdown(pshell_message);
}

/*
** Action
*/
void ReturnActionTextMessage(Widget w,XEvent *event,String *params,
				    Cardinal *num_params)
     /*
     ** action associated to Enter Key in text_message
     */
{
  PopdownMessage(NULL,NULL,NULL); /* call a callback function */
}

static int WriteMessage(String string_message, Widget text)
     /*
     ** Write the string mess in text_message replacing the old string
     */
{
  static int ActualMessageLength=0; /* length of message in text_message */
  XawTextBlock add_block;

  add_block.firstPos = 0;
  add_block.length = strlen(string_message);
  add_block.ptr = string_message;
  add_block.format = XawFmt8Bit;
  XawTextDisableRedisplay(text);
  XtVaSetValues(text, XtNeditType, XawtextEdit, NULL);
  XawTextSetInsertionPoint(text, 0);
  XawTextReplace(text, 0, ActualMessageLength, &add_block);
  XtVaSetValues(text, XtNeditType, XawtextRead, NULL);
  XawTextEnableRedisplay(text);
  ActualMessageLength=add_block.length;
  return(True);
}
