/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/utsname.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "images/xlogo11" /* for the mark */

/*
** include of application modules and images
*/
#include "images/icon"
#include "images/runner"
#include "images/general"
#include "images/mode"
#include "images/smile_computer"
#include "config.h"
#include "opthandler.h"
#include "Email.h"
#include "Sizer.h"
#include "Preferences.h"
#include "ErrorMessage.h"
#include "pathexec.h"
#include "Search.h"
#include "Calc.h"
#include "OpenSave.h"
#include "general_support.h"
#include "GUI_support.h"
#include "Choose.h"
#include "MemEditor.h"
#include "YNShell.h"
#include "menu.h"
#include "menuHELP.h" /* for DedicatedTo action */
#include "menuISTRCOUNTER.h"
#include "BrPts.h"
#include "Edit.h"
#include "ChangeEntry.h"
#include "GUI.h"
#include "loader.h"
#include "policy.h"
#include "cache.h"
#include "memory.h"
#include "esp.h"
#include "global.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Xaw/Label.h>     /* Athena Label Widget */
#include <X11/Xaw/Paned.h>     /* Athena Paned Widget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/List.h>      /* Athena List Widget */
#include <X11/Xaw/Dialog.h>    /* Athena Dialog Widget */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Viewport.h>  /* Athena Viewport Widget */ 
#include <X11/Xaw/Repeater.h>  /* Athena Repeater Widget */
#include <X11/Xaw/Box.h>       /* Athena Box Widget */
#include <X11/Xaw/MenuButton.h>/* Athena MenuButton Widget */
#include <X11/Xaw/SimpleMenu.h>/* Athena SimpleMenuWidget */
#include <X11/Xaw/SmeBSB.h>    /* Athena SmeBSB Widget */
#include <X11/Xaw/SmeLine.h>   /* Athena SmeLineWidget */
#include <X11/Xaw/Command.h>   /* Athena CommandWidget */
#include <X11/Shell.h>         /* Athena ShellWidget */

/* BEGIN LOCAL DEFINE */
/*
** to understand the kind of register got by PopupDialogReg
*/
#define R_first_char 'R'
#define Hi "Hi"
#define Hi_first_char 'H'
#define Lo "Lo"
#define Lo_first_char 'L'

/*
** Label showed by general dialog
*/
#define StringBitCache "Bit: "
#define StringBitMemory StringBitCache
#define StringForPC    "PC: "

/*
** Intervals Time
*/
#define Second 1000             /* bitmap */
#define ThirtySeconds Second*30 /* Birthday */
#define NumFrames 12
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
String listitem_reg[NumItemReg],*listitem_cache,*listitem_memory;
Widget list_cache,list_reg,bitcache;
Widget list_memory,total;
unsigned int item_reg=NumItemReg,item_cache;
Widget pshell_message;
XtAppContext app_context;
Widget topLevel;
unsigned int PC=0;
Widget view_istruzione;
int AlwaysMessageOnHalt=True;
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
/*
** Callbacks
*/
static void InvalidCache(Widget w, XtPointer client_data, XtPointer call_data);
static void ResetRegS(Widget w, XtPointer client_data, XtPointer call_data);
static void ResetStatistics(Widget w, XtPointer client_data, XtPointer call_data);
static void Esecutore(Widget w, XtPointer client_data, XtPointer call_data);
static void Quit(Widget w, XtPointer client_data, XtPointer call_data);
static void ChangePolitica(Widget w, XtPointer client_data, XtPointer call_data);
static void ChangeScrittura(Widget w, XtPointer client_data, XtPointer call_data);
static void ChangeReplace(Widget w, XtPointer client_data, XtPointer call_data);
static void CacheAlphaNum(Widget w, XtPointer client_data, XtPointer call_data);
static void MemoryAlphaNum(Widget w, XtPointer client_data, XtPointer call_data);
static void PopupDialog_cache(Widget w, XtPointer client_data, XtPointer call_data);
static void DialogDoneLoad(Widget w, XtPointer client_data, XtPointer call_data);
static void InvalidMemory(Widget w, XtPointer client_data, XtPointer call_data);
static void DialogCancel(Widget w, XtPointer client_data, XtPointer call_data);
void PopupDialog(Widget w, XtPointer client_data, XtPointer call_data);
static void PopupDialogReg(Widget w, XtPointer client_data, XtPointer call_data);
static void PopupDialogSetLdm(Widget w, XtPointer client_data, XtPointer call_data);
static void activeBP(Widget w, XtPointer client_data, XtPointer call_data);
static void activeTL(Widget w, XtPointer client_data, XtPointer call_data);
static void activeIT(Widget w, XtPointer client_data, XtPointer call_data);
static void activePE(Widget w, XtPointer client_data, XtPointer call_data);
static void activeSbS(Widget w, XtPointer client_data, XtPointer call_data);
static void activeFreeRun(Widget w, XtPointer client_data, XtPointer call_data);
static void LoadCallback(Widget w, XtPointer client_data, XtPointer call_data);
static void RaiseEmail(Widget w, XtPointer client_data, XtPointer call_data);

void PopupDialogMemSize(Widget w, XtPointer client_data, XtPointer call_data);
static void ChangeMemSize(int value);
void PopupDialogCacheSize(Widget w,XtPointer client_data, XtPointer call_data);
static void ChangeCacheSize(int value);

/*
** Actions
*/
static void ReturnAction(Widget w, XEvent *event, String *params,
			 Cardinal *num_params);
static void ReturnPolitica(Widget w, XEvent *event, String *params,
			   Cardinal *num_params);
static void ReshowCache_Memory(Widget w, XEvent *event, String *params,
			       Cardinal *num_params);
static void Decrement(Widget w, XEvent *event, String *params,
		      Cardinal *num_params);
static void Increment(Widget w, XEvent *event, String *params,
		      Cardinal *num_params);
static void JumpPC(Widget w, XEvent *event, String *params,
		   Cardinal *num_params);
static void QuitAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params);
static void LoadAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params);
static void Dedicato(Widget w, XEvent *event, String *params,
		     Cardinal *num_params);
static void Redraw_bitmap(Widget w, XEvent *event, String *params,
			  Cardinal *num_params);
static void CoreAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params);
/*
** Miscellenia functions
*/
static void SelectWhatUpdate(void);
void Load(String filename);
static void BitmapTimer(XtPointer w, XtIntervalId *id);
void quit(void);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
/*
** label for general dialog
*/
static char StringforPC[]=    "PC:      ";
static char StringforSetReg[]="Reg=value";
static char StringforSetLdm[]="Ldm=value";

static Widget pshell;
static Widget runfree,runstepbystep;
static Widget dialog;
static Widget scrittura,politica,replace_mode;
static Widget bitmemory;
static Widget pc;

/* FALLBACK RESOURCES */
static String fallback_rc[] = {
#include "xmas-app-defaults.h"
  NULL
};

static Pixmap mark;
static XtIntervalId bitmap_timer;
static XtIntervalId timer;

static struct {
  Pixmap frames[NumFrames];
  unsigned int numframes;
  Widget showBitmap;
  unsigned int width,height;
} bitmap;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void BitmapTimer(XtPointer w, XtIntervalId *id)
     /*
     ** Every timer expire call Redraw_bitmap to update the bitmap
     */
{
  bitmap_timer=XtAppAddTimeOut(XtWidgetToApplicationContext(w),Second/6,
			       BitmapTimer,w);
  Redraw_bitmap(bitmap.showBitmap,NULL,NULL,NULL);
}

static void BirthdayTimer(XtPointer w, XtIntervalId *id)
     /*
     ** Call ErrorMessage for show a message about the birthday
     */
{
  ErrorMessage(Birthday,NULL,NULL,0);
}

/*
** Callback
*/
static void activeBP(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the BreakPoint mode and select the mark for the menu item
     */
{
  switch (toggleBP()) {
  case False: XtVaSetValues(w,XtNleftBitmap,NULL,NULL); break;
  default: XtVaSetValues(w,XtNleftBitmap,mark,NULL); break;
  }
  UpdateBPlabel();
  IstructionViewer(view_istruzione,PC);
}

/*
** Callback
*/
static void activeTL(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the TimeLimit mode and select the mark for the menu item
     */
{
  switch (toggleTL()) {
  case False: XtVaSetValues(w,XtNleftBitmap,NULL,NULL); break;
  default: XtVaSetValues(w,XtNleftBitmap,mark,NULL); break;
  }
}

/*
** Callback
*/
static void activeIT(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the TimeLimit mode and select the mark for the menu item
     */
{
  switch (toggleIT()) {
  case False: XtVaSetValues(w,XtNleftBitmap,NULL,NULL); break;
  default: XtVaSetValues(w,XtNleftBitmap,mark,NULL); break;
  }
}

/*
** Callback
*/
static void activePE(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the Path Execution mode and select the mark for yhe menu item
     */
{
  switch (IsActiveExecutionPath()) {
  case True: XtVaSetValues(w,XtNleftBitmap,NULL,NULL); break;
  case False: XtVaSetValues(w,XtNleftBitmap,mark,NULL); break;
  }
  togglePE();
}

/*
** Callback
*/
void InvalidMemory(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Cleanup memory and cache, show the new state of the memories
     ** update the istruction viewer too
     */
{
  invalidmemory();/* from memory.c */
  reshow_memory();
  invalidcache(); /* from cache.c */
  reshow_cache();
  IstructionViewer(view_istruzione,PC);
  PEMessageChangeStatus("Memory invalided",NULL);
  /*
  ** IstructionViewer non puo' incontrare istruzioni errate !!!
  ** la cache e' vuota !!!
  */
}

/*
** Callback
*/
static void InvalidCache(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Cleanup the cache and show the new state
     ** show the memory too
     */
{
  invalidcache(); /* from cache.c */
  reshow_cache();
  reshow_memory();
  PEMessageChangeStatus("Cache invalided",NULL);
}

/*
** Callback
*/
static void ResetRegS(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Clenup registers and show the new state
     */
{
  resetregistri(); /* from esecutore in loader.c */
  reshow_reg();
}

/*
** Callback
*/
static void ResetStatistics(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Cleanup statistics and show the new state
     */
{
  Widget widget_total = (Widget) client_data;

  reset_statistic(); /* from cache.c */
  show_total(widget_total);
  PEMessageChangeStatus("Reset statistics",NULL);
}

/*
** Callback
*/
void PopupDialogSetLdm(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Pop up the dialog for set the new value of a Ldm
     */
{
  XawListReturnStruct *list_struct = (XawListReturnStruct *) call_data;
  String etichetta;
  Cardinal Ldc;
  Widget value_widget;

  etichetta=(String)malloc(200);
  Ldc=list_struct->list_index;    /* index of the element into the list */
  sprintf(etichetta,"%x=",Ldc);
  XtVaSetValues(dialog,XtNvalue,etichetta,NULL);
  value_widget=XtNameToWidget(dialog,"value");
  XawTextSetInsertionPoint(value_widget,(int)strlen(etichetta));
  Calculate_relative_shell_position_and_set(pshell);
  XtVaSetValues(dialog,XtNlabel,StringforSetLdm,NULL);
  XtPopup(pshell,XtGrabNonexclusive);
}

/*
** Callback
*/
void PopupDialogReg(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Pop up the dialog for set a register
     */
{
  XawListReturnStruct *list_struct = (XawListReturnStruct *) call_data;
  String etichetta;
  Cardinal Reg;
  Widget value_widget;

  etichetta=(String)malloc(200);
  Reg=list_struct->list_index;    /* index of the element into the list */
  if (Reg<MaxReg)                 /* decide which string to write into value */
    sprintf(etichetta,"R%x=",Reg);
  else if (Reg==MaxReg)
    sprintf(etichetta,"Hi=");
  else
    sprintf(etichetta,"Lo=");
  XtVaSetValues(dialog,XtNvalue,etichetta,NULL);
  value_widget=XtNameToWidget(dialog,"value");
  XawTextSetInsertionPoint(value_widget,(int)strlen(etichetta));
  Calculate_relative_shell_position_and_set(pshell);
  XtVaSetValues(dialog,XtNlabel,StringforSetReg,NULL);
  XtPopup(pshell, XtGrabNonexclusive);
}

/*
** Callback
*/
void Esecutore(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Decide which executor to run
     ** Update: cache, registers, statistics, pc, PC, istruction_viewer
     */
{
  int returned;

  if ((returned=execrun(&PC)) != ExecExactExecution)
    switch (returned) {
    case ExecHalt: if (AlwaysMessageOnHalt == True)
      ErrorMessage(returned,NULL,NULL,(int)PC);
    break;
    case ExecExactExecution: break;
    case ExecBreakPoint:
      ErrorMessage(returned,NULL,NULL,(int)PC); break;
    default: ErrorMessage(returned,NULL,NULL,(int)PC); break;
    }
  reshow_memory();
  reshow_cache();
  reshow_reg();
  show_total(total);
  reshow_label(pc,StringForPC,(int)PC);
  IstructionViewer(view_istruzione,PC);
  if (IsShowedExecutionPath() == True)
    UpdateShowText();
}

/*
** Callback
*/
void Quit(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Quit button callback function
     */
{
  PopupYN(quit,"Are you sure to quit now?",AbsoluteApplicationMiddle,NULL);
}

void quit(void)
{
  SaveConfiguration();
  /*printf("It was nice knowing you.\n");*/
  exit(EXIT_SUCCESS);
}

/*
** Callback
*/
void activeSbS(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the StepbyStep mode and select the mark for the menu item
     */
{
  setSbS();
  XtVaSetValues(runstepbystep,XtNleftBitmap,mark,NULL);
  XtVaSetValues(runfree,XtNleftBitmap,NULL,NULL);
}

/*
** Callback
*/
void activeFreeRun(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Active the FreeRun mode and select the mark for the menu item
     */
{
  setFR();
  XtVaSetValues(runfree,XtNleftBitmap,mark,NULL);
  XtVaSetValues(runstepbystep,XtNleftBitmap,NULL,NULL);
}

/*
** Callback
*/
void ChangePolitica(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Change the politicy
     ** Flush the cache if it is needed
     */
{
  Widget politica_widget = (Widget) client_data;
  String label;

  XtVaGetValues(politica_widget,XtNlabel,&label,NULL);
  if (getModoScrittura() == WriteBack) 
    /*
    ** the memory is flushed because the policy is changed
    ** the write is back must be reinitialized!
    */
    flushcache();
  switch(*label) {
  case 'D':
    setTipoPolitica(Fully);
    XtSetSensitive(replace_mode,True);
    ActiveCacheNormalMode();
    break;
  case 'F': setTipoPolitica(Nway);
    if (getWay() > esp(2,getBitMemoryCache())) {
	ErrorMessage(SetNWayError,NULL,NULL,(int)getWay());
	setWay(2);
    }
    ActiveCacheNormalMode();
    break;
  case 'N':
    setTipoPolitica(KDirect);
    XtSetSensitive(replace_mode,False);
    ActiveCacheKeepMode();
    break;
  case 'K':
    setTipoPolitica(Direct);
    XtSetSensitive(replace_mode,False);
    ActiveCacheNormalMode();
    break;
  default:
    ErrorMessage(PoliticaUnknown,"Policy",NULL,0);
    break;
  }
  PEMessageChangeStatus("Cache policy switched to ",getStringPolicy());
  show_politica(politica_widget);
}

/*
** Callback
*/
void ChangeScrittura(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Change the write mode of the cache
     */
{
  Widget scrittura_widget = (Widget) client_data;

  switch(getModoScrittura()) {
  case WriteThrough: setModoScrittura(WriteBack); break;
  case WriteBack: flushcache(); setModoScrittura(WriteThrough);
    reshow_memory();
    break;
  default: ErrorMessage(ScritturaUnknown,"Write Mode",NULL,0);
    break;
  }
  PEMessageChangeStatus("Cache write mode changed to ",getStringWritemode());
  show_scrittura(scrittura_widget);
}

/*
** Callback
*/
static void ChangeReplace(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Change the replace mode of a full cache
     */
{
  Widget replace_widget = (Widget) client_data;

  switch(getReplaceMode()) {
  case RANDOMode: setReplaceMode(LuMode); break;
  case FIFOMode: setReplaceMode(RANDOMode); break;
  case LruMode: setReplaceMode(FIFOMode); break;
  case LuMode: setReplaceMode(LruMode); break;
  default: ErrorMessage(ReplaceUnknown,"Replace Mode",NULL,0);
    break;
  }
  PEMessageChangeStatus("Cache replacement switched to ",getStringReplacement());
  invalidcache();
  reshow_cache();
  reshow_memory();
  show_replace_mode(replace_widget);
}

/*
** Callback
*/
static void CacheAlphaNum(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** De/activate the alfanumeric display cache mode
     */
{
  switch(getCacheAlphaNumericStatus()) {
  case False: setCacheAlphaNumericStatus(True);
    XtVaSetValues(w,XtNleftBitmap,mark,NULL);
    break;
  default: setCacheAlphaNumericStatus(False);
    XtVaSetValues(w,XtNleftBitmap,NULL,NULL);
  }
  reshow_cache();
}

/*
** Callback
*/
static void MemoryAlphaNum(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** De/activate the alfanumeric display memory mode
     */
{
  switch(getMemoryAlphaNumericStatus()) {
  case False: setMemoryAlphaNumericStatus(True);
    XtVaSetValues(w,XtNleftBitmap,mark,NULL);
    break;
  default: setMemoryAlphaNumericStatus(False);
    XtVaSetValues(w,XtNleftBitmap,NULL,NULL);
  }
  reshow_memory();
}

/*
** Callback
*/
void PopupDialog_cache(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the dialog for show the state of an entry in the cache
     */
{
  XawListReturnStruct *list_struct = (XawListReturnStruct *) call_data;
  char etichetta[MediumBufferLength];
  char strtag[SmallBufferLength],strdata[SmallBufferLength];
  char strdemand[SmallBufferLength];
  Cardinal Ldc;

  Ldc=list_struct->list_index;    /* index of the element into the list */
  sprintf(strtag,"0x%.8x",(int)tag(Ldc));
  sprintf(strdata,"0x%.8x",data(Ldc));
  sprintf(strdemand,"0x%.8x",(int)demand(Ldc));
  sprintf(etichetta,"Tag: %s\nData: %s\nValid: %x\nDirty: %x\nKeep: %x\nDemand: %s",strtag,strdata,valid(Ldc),dirty(Ldc),getKeep(Ldc),strdemand);
  ErrorMessage(DialogCacheInfo,etichetta,NULL,(int)Ldc);
}

/*
** Callback
*/
void PopupDialogMemSize(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the dialog for memory size
     */
{
  PopupSizerHex(1,0xb,getBitMemory(),ChangeMemSize);
}

static void ChangeMemSize(int value)
     /*
     ** Check the thumb value of scrollbar for memory bit
     */
{
  unsigned int i,oldBM;
  char MemBitstring[SmallBufferLength];

  sprintf(MemBitstring,"0x%x",value);
  oldBM=getBitMemory();
  if (setBitMemory((unsigned int)value) && (value>=getBitMemoryCache())) {
    if (reset_memory_size(getBitMemory()) != False) {
      for (i=0; i<esp(2,oldBM); i++)
	free(listitem_memory[i]);
      free(listitem_memory);
      value=esp(2,getBitMemory());
      listitem_memory=(String *)malloc(sizeof(String)*value);
      for (i=0; i<value; i++)
	listitem_memory[i]=(String)malloc(ShortBufferLength);
      if (PC >= getMaxAddress()) { /* if the memory il less than PC */
	PC=getMaxAddress()-1;
	reshow_label(pc,StringForPC,(int)PC);
      }
      invalidcache();
      reset_breakline();
      reshow_cache();     /* show cache list*/
      reset_statistic();  /* from cache.c */
      show_total(total);  /* update the statistics */
      reshow_memory();
      reshow_label(bitmemory,StringBitMemory,(int)getBitMemory());
      PEMessageChangeStatus("Memory size switched to ",MemBitstring);
      PEMessageChangeStatus("Cache invalided",NULL);
    }
    else
      ErrorMessage(SetMemorySizenotenough,MemBitstring,NULL,0);
  }
  else
    ErrorMessage(SetMemorySizeError,MemBitstring,NULL,0);
}

/*
** Callback
*/
void PopupDialogCacheSize(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the dialog for cache size
     */
{
  PopupSizerHex(1,0xb,getBitMemoryCache(),ChangeCacheSize);
}

static void ChangeCacheSize(int value)
     /*
     ** Check the thumb value of scrollbar for memory bit
     */
{
  unsigned int i;
  unsigned int testNewSize=False;
  char size[SmallBufferLength];

  switch (getTipoPolitica()) {
  case KDirect:
  case Direct:
  case Fully:
    testNewSize=(value <= getBitMemory() && setBitCache((unsigned int)value));
    break;
  case Nway:
    /*
    ** getWay MUST BE CHECKED BEFORE setBitCache
    ** otherwise may be that the cache size is updated without the getWay bound
    ** this is done because the if test start from left to right
    */
    testNewSize=((getWay() <= esp(2,(unsigned int)value)) &&
      value <= getBitMemory() && setBitCache((unsigned int)value));
    break;
  }
  if (testNewSize != False) {
    /* the new size of cache is correct */
    invalidcache();                    /* cleanup the cache */
    reset_cache_size();                /* set the new cache for NWay*/
    inizializza_politica();            /* set the new policy */
    for (i=0; i<item_cache; i++)
      free(listitem_cache[i]);         /* clenup all the strings */
    free(listitem_cache);              /* cleanup the array of pointer */
    item_cache=esp(2,getBitMemoryCache()); /* calculate #entry */
    /* new array for the list */
    listitem_cache=(String *)malloc(sizeof(String)*(item_cache));
    for (i=0; i<item_cache; i++)      /* allocate the space for the strings */
      listitem_cache[i]=(String)malloc(ShortBufferLength);
    reshow_cache();             /* show cache list */
    reshow_label(bitcache,StringBitCache,(int)getBitMemoryCache());
    reset_statistic();          /* from cache.c */
    show_total(total);          /* update the statistics */
    sprintf(size,"0x%x",value);
    
    PEMessageChangeStatus("Cache size switched to ",size);
  }
  else {
    char BitString[SmallBufferLength];

    sprintf(BitString,"0x%x",value);
    ErrorMessage(SetCacheSizeError,BitString,NULL,0);
  }
}

void PopupDialog(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Show the dialog for pc
     */
{
  char *string = (char *) client_data;

  Calculate_relative_shell_position_and_set(pshell);
  XtVaSetValues(dialog,XtNlabel,string,NULL);
  XtPopup(pshell,XtGrabNonexclusive);
}

static void SelectWhatUpdate(void)
     /*
     ** Select the reply to do
     */
{
  String string,label;
  int numero_registro,value,returned,withoutvalue=False;

  XtPopdown(pshell);
  string = XawDialogGetValueString(dialog);
  XtVaGetValues(dialog,XtNlabel,&label,NULL);
  if (*StringforPC == *label) {
    if (TestPC(&PC,string) == False) /* wrong value of PC */
      /*XBell(XtDisplay(pshell),100);*/
      ErrorMessage(OUTPC,string,NULL,0);
    else 
      reshow_label(pc,StringForPC,(int)PC);
  }
  if (*StringforSetReg == *label) {
    if (*string == R_first_char) {
      if((returned=sscanf(string,"R%x=%8x",&numero_registro,&value)) != 2)
	withoutvalue=True;
    }
    if (*string == Lo_first_char) {
      numero_registro=MaxReg+1;
      if(sscanf(string,"Lo=%8x",&value) !=1)
	withoutvalue=True;
    }
    if (*string == Hi_first_char) {
      numero_registro=MaxReg;
      if(sscanf(string,"Hi=%8x",&value) !=1)
	withoutvalue=True;
    }
    if (withoutvalue==True) {
      XBell(XtDisplay(pshell),100);
      ErrorMessage(SetRegWithoutValue,NULL,NULL,0);
    }
    else {
      if((returned=setregistro(numero_registro,value)) == True)
	reshow_reg();
      else
	ErrorMessage(returned,NULL,NULL,0);
    }
  }
  if (*StringforSetLdm == *label) {
    if ((returned=SetLdm(string,getMaxAddress())) == True) {
      reshow_cache();
      reshow_memory();
      show_total(total);
    }
    else
      ErrorMessage(returned,NULL,NULL,0);
  }
  IstructionViewer(view_istruzione,PC);
  XtVaSetValues(dialog,XtNvalue,"",NULL);  /* clean up dialog value */
}

/*
** Callback
*/
void DialogDoneLoad(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Select the "action" to do in reply to the ok command of the dialog
     */
{
  SelectWhatUpdate();
}

/*
** Callback
*/
void DialogCancel(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Darken the dialog without do anything
     */
{
  XtPopdown(pshell);
  XtVaSetValues(dialog,XtNvalue,"",NULL); /* clean up value */
}

/*
** Action
*/
static void Redraw_bitmap(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Show the correct frame in answer to Expose event or timer expire
     */
{
  Pixmap pixmap;
  static int frame=0;

  frame = frame % 12;
  pixmap = bitmap.frames[frame];
  if (DefaultDepthOfScreen(XtScreen(w)) == 1)
    XCopyArea(XtDisplay(w), pixmap, XtWindow(w),
              DefaultGCOfScreen(XtScreen(w)), 0, 0, 
              bitmap.width,bitmap.height, 0, 0);
    else
      XCopyPlane(XtDisplay(w), pixmap, XtWindow(w),
                 DefaultGCOfScreen(XtScreen(w)), 0, 0, 
                 bitmap.width,bitmap.height,0, 0, 1);
  if (event == NULL) /* isn' t expose event */
    frame++;
}

/*
** Action
*/
static void ReturnAction(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** action associated to Enter Key
     */
{
  SelectWhatUpdate();
}

/*
** Action
*/
static void Dedicato(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
  ErrorMessage(DedicatedTo,NULL,NULL,0);
}

/*
** Action
*/
/*
** called only when Btn1Up, because the repeater don't call itself
*/
static void ReturnPolitica(Widget w, XEvent *event, String *params,
			   Cardinal *num_params)
     /*
     ** action associated to click third button
     ** change the way of cache
     ** invalid the cache and show it
     */
{
  unsigned int newWay=0; /* the future Nway value */

  if (getModoScrittura() == WriteBack)
    flushcache();
  if (getTipoPolitica() == Nway) {
    switch (getWay()) {
    case 2: newWay=4;
      if (newWay <= esp(2,getBitMemoryCache()))
	break; /* cache big than NewWay */
      else
	ErrorMessage(SetNWayError,NULL,NULL,(int)getWay());
    case 4: newWay=8;
      if (newWay <= esp(2,getBitMemoryCache()))
	break; /* cache big than NewWay */
      else
	ErrorMessage(SetNWayError,NULL,NULL,(int)getWay());
    case 8: newWay=2; break;
    default: newWay=2; break;
    }
    setWay(newWay);
    show_politica(politica);
  }
}

/*
** Action
*/
static void ReshowCache_Memory(Widget w, XEvent *event, String *params,
			       Cardinal *num_params)
     /*
     ** clean up the  cache, update the relative widget
     ** update the istruction viewer
     ** update the memory widget
     */
{
  invalidcache();
  reshow_cache();
  if (getModoScrittura() == WriteBack) /* reshow the memory only when needed */
    reshow_memory();
}

/*
** Action
*/
static void Decrement(Widget w, XEvent *event, String *params,
		      Cardinal *num_params)
     /*
     ** Subtract one to the Program Counter, in reply to Btn3
     */
{
  if (PC > 0) {
    PC--;
    reshow_label(pc,StringForPC,(int)PC);
    IstructionViewer(view_istruzione,PC);
  }
}

/*
** Action
*/
static void Increment(Widget w, XEvent *event, String *params,
		      Cardinal *num_params)
     /*
     ** Add one to the Program Counter, in reply to Btn2
     */
{
  if (PC < getMaxAddress()-1) {
    PC++;
    reshow_label(pc,StringForPC,(int)PC);
    IstructionViewer(view_istruzione,PC);
  }
}

/*
** Action
*/
static void JumpPC(Widget w, XEvent *event, String *params,
		      Cardinal *num_params)
{
  XawListReturnStruct *list_struct;

  list_struct=XawListShowCurrent(w);
  PC=(unsigned int)list_struct->list_index;
  reshow_label(pc,StringForPC,(int)PC);
  IstructionViewer(view_istruzione,PC);
}

/*
** Action
*/
static void QuitAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params)
     /*
     ** Quit action to terminate
     */
{
  Quit(NULL,NULL,NULL); /* call a callback function */
}

/*
** Callback
*/
static void LoadCallback(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Load callback to popup the OpenSave
     */
{
  LoadAction(w,NULL,NULL,NULL);
}

/*
** Action
*/
static void LoadAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params)
     /*
     ** Load action to popup the OpenSave
     */
{
  PopupOpenSave(w,"Open File:",Load);
}

/*
** Callback
*/
static void RaiseEmail(Widget w, XtPointer client_data, XtPointer call_data)
{
  PopupEmail(NULL);
}

/*
** Action
*/
static void CoreAction(Widget w, XEvent *event, String *params,
		       Cardinal *num_params)
     /*
     ** Load action to popup the OpenSave
     */
{
  struct utsname buf;
  char Date[SmallBufferLength],Orario[SmallBufferLength];
  char global[LongBufferLength];

  DateTime(Date,Orario);
  uname(&buf); /* get some useful information */
  sprintf(global,"Host: %s\n\
OS: %s %s %s\n\
Date: %s\n\
Time: %s",buf.nodename,buf.sysname,buf.release,buf.machine,Date,Orario);
  ErrorMessage(DateTimeCore,global,NULL,0);
}

void Load(String filename)
     /*
     ** load a file into simulator
     */
{
  char istr_string[ShortBufferLength];
  int line_counter,returned;

  if((returned=loader(filename,&PC,istr_string,&line_counter)) == LoaderExactCode) {
    /* update the cache,scrittura,politica,replace_mode,total,pc */
    reshow_cache();
    reshow_memory();
    show_scrittura(scrittura);
    show_politica(politica);
    show_replace_mode(replace_mode);
    show_total(total);
    reshow_label(pc,StringForPC,(int)PC);
  }
  else
    ErrorMessage(returned,filename,istr_string,line_counter);
}

int main(int argc,char **argv,char **envp)
{
  Widget buttonfile,menufile,fileload,filemail,filequit;
  Widget buttonmoderun,menumoderun,linerun;
  Widget buttoncache,menucache,cacheinvalid,cacheresetstat,cachebit;
  Widget cachealphanum;

  Widget paned,dialog_done,dialog_cancel,label_dialog;
  Widget run,timelimit,breakpoint,pe,istrlimit;
  Widget form_command,form_labelcache,form_labelreg;
  Widget reset_reg;
  Widget view_cache,view_regs;
  Widget form_view_istruzione;
  Widget form_labelmemory,view_memory;
  Widget buttonmemory,menumemory,memoryinvalid,memorybit,memorysearch;
  Widget memoryalphanum;

  unsigned int i,MemoryMaxAddress;
  Widget value_widget;
  Pixmap runner,general,mode;
  Pixmap icon,dialog_load;

  XtTranslations Translations; /* to parse the translations table */
  XtAccelerators Accelerators; /* to parse the accelerators table */

  XtActionsRec AllActions[] = { /* action to add */
    {"ReturnActionTextMessage",ReturnActionTextMessage},
    {"LoadAction",LoadAction},
    {"QuitAction",QuitAction},
    {"Decrement",Decrement},
    {"Increment",Increment},
    {"Dedicato",Dedicato},
    {"ReturnPolitica",ReturnPolitica},
    {"ReshowCache_Memory",ReshowCache_Memory},
    {"ReturnAction",ReturnAction},
    {"ChangeEntry",ChangeEntry},
    {"Redraw_bitmap",Redraw_bitmap},
    {"PopupEdit",PopupEdit},
    {"PopupBrPts",PopupBrPts},
    {"PopupMemEditor",PopupMemEditor},
    {"PopupCalc",PopupCalc},
    {"CoreAction",CoreAction},
    {"UpdateMenuCounter",UpdateMenuCounter},
    {"PopupExecutionPath",PopupExecutionPath},
    {"PopupPreferencesAction",PopupPreferencesAction},
    {"JumpPC",JumpPC},
  };

  inizializza_memoria();
  inizializza_cache();
  inizializza_politica();
  inizializza_loader();
  InitializeExecutionPath();
  MemoryMaxAddress=getMaxAddress(); /* startup speed up */
  listitem_memory=(String *)malloc(sizeof(String)*MemoryMaxAddress);
  for (i=0; i<MemoryMaxAddress; i++)
    listitem_memory[i]=(String)malloc(ShortBufferLength);
  /* create an array of pointer to string */
  item_cache=esp(2,getBitMemoryCache()); /* calcola #entry */
  listitem_cache=(String *)malloc(sizeof(String)*item_cache);
  for (i=0; i<item_cache; i++)
    /* allocate memory for the string */
    listitem_cache[i]=(String)malloc(ShortBufferLength);
  for (i=0; i<item_reg; i++) /* create registers */
    listitem_reg[i]=(String)malloc(ShortBufferLength);
  XtSetLanguageProc(NULL, (XtLanguageProc)NULL, NULL);
  
  topLevel = XtVaAppInitialize(
	     &app_context,
	     "XMas",
             NULL, 0,
             &argc, argv,
	     fallback_rc,NULL); /* fallback resources */
  pshell = XtVaCreatePopupShell(
	"xmasDialog",
	transientShellWidgetClass,
	topLevel,NULL);
        dialog = XtVaCreateManagedWidget(
	   "dialog",
	   dialogWidgetClass,
	   pshell,NULL);
        dialog_done = XtVaCreateManagedWidget(
	   "dialog_done",
	   commandWidgetClass,
	   dialog,NULL);
	dialog_cancel = XtVaCreateManagedWidget(
	   "dialog_cancel",
	   commandWidgetClass,
	   dialog,NULL);
   paned = XtVaCreateManagedWidget(
	"paned",
	panedWidgetClass,
	topLevel,
	/*XtNheight,500,*/
	/*XtNwidth,700,*/
	NULL);
   form_command = XtVaCreateManagedWidget(
	"form_command",     /* widget name */
	formWidgetClass,    /* widget class */
	paned,NULL);
        buttonfile = XtVaCreateManagedWidget(
	     "buttonfile",
	     menuButtonWidgetClass,
	     form_command,
	     XtNmenuName,"menufile",NULL);
	     menufile = XtCreatePopupShell(
		  "menufile",
		  simpleMenuWidgetClass,
		  buttonfile,NULL,0);
	     fileload = XtVaCreateManagedWidget(
		  "fileload",
		  smeBSBObjectClass,
		  menufile,NULL);
	     filemail = XtVaCreateManagedWidget(
		  "filemail",
		  smeBSBObjectClass,
		  menufile,NULL);
	     filequit = XtVaCreateManagedWidget(
		  "filequit",
		  smeBSBObjectClass,
		  menufile,NULL);
	run = XtVaCreateManagedWidget(
	     "run",
             repeaterWidgetClass,
             form_command,NULL);
	buttonmoderun = XtVaCreateManagedWidget(
	     "buttonmoderun",
	     menuButtonWidgetClass,
	     form_command,
	     XtNmenuName,"menumoderun",NULL);
	     menumoderun = XtCreatePopupShell(
		  "menumoderun",
		  simpleMenuWidgetClass,
		  buttonmoderun,NULL,0);
	     runfree = XtVaCreateManagedWidget(
		  "runfree",
		  smeBSBObjectClass,
		  menumoderun,NULL);
	     runstepbystep = XtVaCreateManagedWidget(
		  "runstepbystep",
		  smeBSBObjectClass,
		  menumoderun,NULL);
	     linerun = XtVaCreateManagedWidget(
		  "linerun",
		  smeLineObjectClass,
		  menumoderun,NULL);
	     breakpoint = XtVaCreateManagedWidget(
		  "breakpoint",
		  smeBSBObjectClass,
		  menumoderun,NULL);
	     timelimit = XtVaCreateManagedWidget(
		  "timelimit",
		  smeBSBObjectClass,
		  menumoderun,NULL);
	     pe = XtVaCreateManagedWidget(
		  "pe",
		  smeBSBObjectClass,
		  menumoderun,NULL);
	     istrlimit = XtVaCreateManagedWidget(
		  "istrlimit",
		  smeBSBObjectClass,
		  menumoderun,NULL);
       InitializemenuOPTION(form_command);
       InitializemenuWINDOW(form_command);
       InitializemenuTUNING(form_command);
       InitializemenuHELP(form_command);
       bitmap.width=32;
       bitmap.height=32;
       bitmap.showBitmap = XtVaCreateManagedWidget(
		  "showBitmap",
		  coreWidgetClass,
		  form_command,
		  XtNwidth,bitmap.width,
		  XtNheight,bitmap.height,NULL);		  
   form_labelreg = XtVaCreateManagedWidget(
	"form_labelreg",
	formWidgetClass,
	paned,NULL);
        pc = XtVaCreateManagedWidget(
            "pc",
            commandWidgetClass,
            form_labelreg,NULL);
        reset_reg = XtVaCreateManagedWidget(
            "reset_reg",
            commandWidgetClass,
            form_labelreg,NULL);
	InitializemenuISTRCOUNTER(form_labelreg);
   form_view_istruzione = XtVaCreateManagedWidget(
	"form_view_istruzione",
	formWidgetClass,
	paned,NULL);
        view_istruzione = XtVaCreateManagedWidget(
	     "view_istruzione",
             labelWidgetClass,
             form_view_istruzione,NULL);
   view_regs = XtVaCreateManagedWidget(
           "view_regs",
           viewportWidgetClass,
           paned,NULL);
        list_reg = XtVaCreateManagedWidget(
	       "list_reg",
	       listWidgetClass,
	       view_regs,NULL);
   form_labelcache = XtVaCreateManagedWidget(
	"form_labelcache",
	formWidgetClass,
	paned,NULL);
        buttoncache = XtVaCreateManagedWidget(
	     "buttoncache",
	     menuButtonWidgetClass,
	     form_labelcache,
	     XtNmenuName,"menucache",NULL);
	     menucache = XtCreatePopupShell(
		  "menucache",
		  simpleMenuWidgetClass,
		  buttoncache,NULL,0);
	     cacheinvalid = XtVaCreateManagedWidget(
		  "cacheinvalid",
		  smeBSBObjectClass,
		  menucache,NULL);
	     cacheresetstat = XtVaCreateManagedWidget(
		  "cacheresetstat",
		  smeBSBObjectClass,
		  menucache,NULL);
	     cachebit = XtVaCreateManagedWidget(
		  "cachebit",
		  smeBSBObjectClass,
		  menucache,NULL);
	     cachealphanum = XtVaCreateManagedWidget(
		  "cachealphanum",
		  smeBSBObjectClass,
		  menucache,NULL);
	 politica = XtVaCreateManagedWidget(
              "politica",
              repeaterWidgetClass,
              form_labelcache,NULL);
	 total = XtVaCreateManagedWidget(
               "total",
               labelWidgetClass,
               form_labelcache,NULL);
	 scrittura = XtVaCreateManagedWidget(
              "scrittura",
              repeaterWidgetClass,
              form_labelcache,NULL);
	 replace_mode = XtVaCreateManagedWidget(
	      "replace_mode",
              repeaterWidgetClass,
              form_labelcache,NULL);
	 bitcache = XtVaCreateManagedWidget(
	      "bitcache",
              labelWidgetClass,
              form_labelcache,NULL);
   view_cache = XtVaCreateManagedWidget(
           "view_cache",
           viewportWidgetClass,
           paned,NULL);
       list_cache = XtVaCreateManagedWidget(
	     "list_cache",
             listWidgetClass,
	     view_cache,
	     XtNheight,100,
	     NULL);
   form_labelmemory = XtVaCreateManagedWidget(
	"form_labelmemory",
	formWidgetClass,
	paned,NULL);
        buttonmemory = XtVaCreateManagedWidget(
            "buttonmemory",
            menuButtonWidgetClass,
            form_labelmemory,
	    XtNmenuName,"menumemory",NULL);
	    menumemory = XtCreatePopupShell(
		  "menumemory",
		  simpleMenuWidgetClass,
		  buttonmemory,NULL,0);
	    memoryinvalid = XtVaCreateManagedWidget(
		  "memoryinvalid",
		  smeBSBObjectClass,
		  menumemory,NULL);
	    memorybit = XtVaCreateManagedWidget(
		  "memorybit",
		  smeBSBObjectClass,
		  menumemory,NULL);
	    memorysearch = XtVaCreateManagedWidget(
                  "memorysearch",
		  smeBSBObjectClass,
		  menumemory,NULL);
	    memoryalphanum = XtVaCreateManagedWidget(
                  "memoryalphanum",
		  smeBSBObjectClass,
		  menumemory,NULL);
	bitmemory = XtVaCreateManagedWidget(
	      "bitmemory",
              labelWidgetClass,
              form_labelmemory,NULL);
   view_memory = XtVaCreateManagedWidget(
           "view_memory",
           viewportWidgetClass,
           paned,NULL);
           list_memory = XtVaCreateManagedWidget(
		"list_memory",
		listWidgetClass,
		view_memory,NULL);

   XawListChange(list_reg,listitem_reg,(int) item_reg,-1,TRUE);
   /*
   ** To set the viewable space for the cache
   ** half size than memory = 32 items
   */
   XawListChange(list_cache,listitem_memory,(int) 32,-1,FALSE);
   /*
   ** To set the viewable space for the memory
   ** size = 64 items
   */
   XawListChange(list_memory,listitem_memory,(int) 64,-1,TRUE);

   /* Menu File callbacks */
   XtAddCallback(fileload,XtNcallback,LoadCallback,(XtPointer)0);
   XtAddCallback(filemail,XtNcallback,RaiseEmail,0);
   XtAddCallback(filequit,XtNcallback,Quit,0);

   /* Menu Run mode callbacks */
   XtAddCallback(runfree,XtNcallback,activeFreeRun,(XtPointer)0);
   XtAddCallback(runstepbystep,XtNcallback,activeSbS,(XtPointer)0);
   XtAddCallback(breakpoint,XtNcallback,activeBP,(XtPointer)0);
   XtAddCallback(timelimit,XtNcallback,activeTL,(XtPointer)0);
   XtAddCallback(pe,XtNcallback,activePE,(XtPointer)0);
   XtAddCallback(istrlimit,XtNcallback,activeIT,(XtPointer)0);

   /* Menu Cache callbacks */
   XtAddCallback(cacheinvalid,XtNcallback,InvalidCache,(XtPointer)0);
   XtAddCallback(cacheresetstat,XtNcallback,ResetStatistics,total);
   XtAddCallback(cachebit,XtNcallback,PopupDialogCacheSize,(XtPointer)0);
   XtAddCallback(cachealphanum,XtNcallback,CacheAlphaNum,(XtPointer)0);

   /* Menu Memory callbacks */
   XtAddCallback(memoryinvalid,XtNcallback,InvalidMemory,(XtPointer)0);
   XtAddCallback(memorybit,XtNcallback,PopupDialogMemSize,(XtPointer)0);
   XtAddCallback(memorysearch,XtNcallback,PopupSearch,(XtPointer)0);
   XtAddCallback(memoryalphanum,XtNcallback,MemoryAlphaNum,(XtPointer)0);

   XtAddCallback(list_memory,XtNcallback,PopupDialogSetLdm,0);

   XtAddCallback(list_cache,XtNcallback,PopupDialog_cache,0);
   XtAddCallback(dialog_done,XtNcallback,DialogDoneLoad,0);
   XtAddCallback(dialog_cancel,XtNcallback,DialogCancel,0);
   XtAddCallback(run,XtNcallback,Esecutore,0);
   XtAddCallback(pc,XtNcallback,PopupDialog,StringforPC);
   XtAddCallback(reset_reg,XtNcallback,ResetRegS,0);
   XtAddCallback(list_reg,XtNcallback,PopupDialogReg,0);
   XtAddCallback(politica,XtNcallback,ChangePolitica,politica);
   XtAddCallback(scrittura,XtNcallback,ChangeScrittura,scrittura);
   XtAddCallback(replace_mode,XtNcallback,ChangeReplace,replace_mode);

   /* to install accelerators */
   Accelerators = XtParseAcceleratorTable /* accelerators for buttonfile */
     ("#override\n\
       Ctrl<KeyPress>q: QuitAction()\n\
       Ctrl<KeyPress>o: LoadAction(topLevel)");
   XtVaSetValues(buttonfile,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(paned,buttonfile);
   Accelerators = XtParseAcceleratorTable /* accelerators for run */
     ("#override\n\
       Ctrl<KeyPress>r: set() notify() unset()");
   XtVaSetValues(run,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(paned,run);
   Accelerators = XtParseAcceleratorTable /* accelerators for paned */
     ("#override\n\
       Ctrl<KeyPress>e: PopupEdit()\n\
       Ctrl<KeyPress>b: PopupBrPts()\n\
       Ctrl<KeyPress>m: PopupMemEditor()\n\
       Ctrl<KeyPress>c: PopupCalc()\n\
       Ctrl<KeyPress>p: PopupExecutionPath()\n\
       Ctrl<KeyPress>f: PopupPreferencesAction(general.prf)\n\
       Ctrl Shift Alt <KeyPress>d: Dedicato()");
   XtVaSetValues(paned,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(paned,paned);


   /* to install translations */
   Translations = XtParseTranslationTable /* translation for value */
     ("<Key>Return: ReturnAction()\n\
      Ctrl<KeyPress>r: no-op(RingBell)\n\
      Ctrl<KeyPress>s: no-op(RingBell)\n\
      Meta<KeyPress>i: no-op(RingBell)");
   value_widget=XtNameToWidget(dialog,"value");
   XtOverrideTranslations(value_widget, Translations);
   Translations = XtParseTranslationTable
     ("<Btn3Up>: ReturnPolitica() ReshowCache_Memory()\n\
      <Btn1Up>: stop() unset() ReshowCache_Memory()");
   XtOverrideTranslations(politica, Translations);
   Translations = XtParseTranslationTable
     ("<Btn3Up>: Decrement()\n\
      <Btn2Up>: Increment()");
   XtOverrideTranslations(pc, Translations);
   Translations = XtParseTranslationTable
     ("Ctrl <Btn1Up>: ChangeEntry()");
   XtOverrideTranslations(list_cache, Translations);
   
   Translations = XtParseTranslationTable
     ("Shift <Btn1Up>: JumpPC()");
   XtOverrideTranslations(list_cache, Translations);
   XtOverrideTranslations(list_memory, Translations);

   Translations = XtParseTranslationTable
     ("<Expose>: Redraw_bitmap()");
   XtOverrideTranslations(bitmap.showBitmap, Translations);
   Translations = XtParseTranslationTable
     ("<Btn1Down>: CoreAction()");
   XtOverrideTranslations(bitmap.showBitmap, Translations);

   /* add actions */
   XtAppAddActions(app_context, AllActions, XtNumber(AllActions));

   icon=XCreateBitmapFromData(XtDisplay(topLevel),
			      RootWindowOfScreen(XtScreen(topLevel)),
			      icon_bits, icon_width, icon_height);
   mark=XCreateBitmapFromData(XtDisplay(topLevel),
			      RootWindowOfScreen(XtScreen(topLevel)),
			      xlogo11_bits, xlogo11_width,xlogo11_height);
   runner=XCreateBitmapFromData(XtDisplay(topLevel),
				RootWindowOfScreen(XtScreen(topLevel)),
				runner_bits,runner_width,
				runner_height);
   general=XCreateBitmapFromData(XtDisplay(topLevel),
				 RootWindowOfScreen(XtScreen(topLevel)),
				general_bits,general_width,general_height);
   mode=XCreateBitmapFromData(XtDisplay(topLevel),
			      RootWindowOfScreen(XtScreen(topLevel)),
			      mode_bits,mode_width,mode_height);
   dialog_load=XCreateBitmapFromData(XtDisplay(topLevel),
			      RootWindowOfScreen(XtScreen(topLevel)),
			      smile_computer_bits,smile_computer_width,
			      smile_computer_height);

   XtVaSetValues(topLevel, XtNiconPixmap, icon,NULL);
   XtVaSetValues(run,XtNleftBitmap,runner,NULL);
   XtVaSetValues(buttonfile,XtNleftBitmap,general,NULL);
   XtVaSetValues(buttonmoderun,XtNleftBitmap,mode,NULL);
   label_dialog=XtNameToWidget(dialog,"label");
   XtVaSetValues(label_dialog,XtNleftBitmap,dialog_load,NULL);

   /* initial mark position */
   XtVaSetValues(runstepbystep,XtNleftBitmap,mark,NULL);
   XtVaSetValues(timelimit,XtNleftBitmap,mark,NULL);
   XtVaSetValues(istrlimit,XtNleftBitmap,mark,NULL);
   if (getCacheAlphaNumericStatus())
     XtVaSetValues(cachealphanum,XtNleftBitmap,mark,NULL);
   if (getMemoryAlphaNumericStatus())
     XtVaSetValues(memoryalphanum,XtNleftBitmap,mark,NULL);

   /*
   **  Create windows for widgets and map them.
   */
   XtRealizeWidget(topLevel);

   /*
   ** Initialize before to start the Main Loop
   */
   XawListChange(list_memory,listitem_memory,(int) MemoryMaxAddress,-1,TRUE);
   reshow_label(bitcache,StringBitCache,(int)getBitMemoryCache());
   reshow_label(bitmemory,StringBitMemory,(int)getBitMemory());
   reshow_memory();
   reshow_cache();
   reshow_reg();
   show_total(total);

   show_replace_mode(replace_mode);
   show_scrittura(scrittura);
   reshow_label(bitcache,StringBitCache,(int)getBitMemoryCache());
   show_politica(politica);

   XawListChange(list_cache,listitem_memory,(int) 16,-1,TRUE);
   XawListChange(list_cache,listitem_cache,(int) item_cache,-1,TRUE);

   {
     char RetValue[MediumBufferLength];
   
     if (getOptionFromName(GeneralPreferencesFile,WelcomeOpt,RetValue,sizeof(RetValue)) == True)
       if (getBoolOptionsValue(RetValue))
	 ErrorMessage(WelcomeCode,NULL,NULL,0);
   }
   /*ErrorMessage(UnderConstruction,NULL,NULL,0); * Welcome message */

   /*
   ** Version control
   */
   versioncheck(envp);

   /*
   ** Birthday control
   */
   if(birthday())
     timer=XtAppAddTimeOut(XtWidgetToApplicationContext(topLevel),
			   ThirtySeconds,BirthdayTimer, 0);

   if(strrchr(argv[0],'/') != NULL)
     printf("\n\nWelcome to %s %s: the X Memory Access Simulator!\n",
	    strrchr(argv[0],'/')+1,VersionString);
   else
     printf("\n\nWelcome to %s %s: the X Memory Access Simulator!\n",
	    argv[0],VersionString);
   /*
   ** COPYRIGHT MESSAGE
   */
   printf("Realized by Paolo Dovera 1997/98/99/00\n");
   printf("email: %s\n",AUTHORemail);
   printf("       %s\n",secondAUTHORemail);
   printf("WEB: %s\n",WEB);
   printf("THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!\n\n");

   /*
   ** Prepare the frames for the animations
   */
   make_frames(bitmap.frames);
   bitmap_timer=XtAppAddTimeOut(XtWidgetToApplicationContext(topLevel),Second,
				BitmapTimer,topLevel);

   /**
    **  Loop for events.
    **/
   XtAppMainLoop(app_context);
   return(EXIT_SUCCESS);
}
