/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h> /* isalpha() */

/*
** include of application modules and images
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/List.h>       /* Athena List Widget */
#include <X11/Xaw/AsciiText.h>  /* Athena AsciiText Widget */

/*
** include of application modules and images
*/
/*#include "Colouring.h"*/
#include "ErrorMessage.h"
#include "GUI_support.h"
#include "GUI.h" /* needed by NumItemReg */
#include "loader.h"
#include "policy.h"
#include "cache.h"
#include "memory.h"
#include "global.h"

#include "images/crono_base_1"
#include "images/crono_base_2"
#include "images/crono_base_3"
#include "images/crono_base_4"
#include "images/crono_base_5"
#include "images/crono_base_6"
#include "images/crono_base_7"
#include "images/crono_base_8"
#include "images/crono_base_9"
#include "images/crono_base_10"
#include "images/crono_base_11"
#include "images/crono_base_12"

/* BEGIN LOCAL DEFINE */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget list_reg;
extern Widget list_memory;
extern Widget list_cache;

extern String listitem_reg[NumItemReg];
extern String *listitem_cache;
extern String *listitem_memory;

extern unsigned int item_reg;
extern unsigned int item_cache;

extern Widget pshell_message,text_message;
extern Widget topLevel;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
/*
static void SetBackgroundColor(Widget w, String color);
static void SetForegroundColor(Widget w, String color);
*/
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

int IstructionViewer(Widget viewer,unsigned int PC)
     /*
     ** Show the istruction decoded
     */
{
  char label[MediumBufferLength],string[MediumBufferLength];
  unsigned int numericValue;

  if (unloader(PC,string) == UnloaderExactDecode) {
    sprintf(label,"Istruction to execute: %s",string);
    XtVaSetValues(viewer,XtVaTypedArg,XtNforeground,XtRString,"steelblue",
                  strlen("steelblue"+1),NULL);
  }
  else {
    sscanf(string,"%x",&numericValue);
    sprintf(label,"Unknown Istruction: 0x%x",numericValue);
    XtVaSetValues(viewer,XtVaTypedArg,XtNforeground,XtRString,"red",
                  strlen("red"+1),NULL);
  }
  XtVaSetValues(viewer,XtNlabel,label,NULL);
  return(True);
}

int reshow_label(Widget w, String partial_string, int value)
     /*
     ** Update the label of widget w with the concatenate of
     ** partial_string and the ascii of value
     */
{
  char message[MediumBufferLength];

  sprintf(message,"%s0x%x",partial_string,value);
  XtVaSetValues(w,XtNlabel,message,NULL);
  return(True);
}

void reshow_memory(void)
     /*
     ** Write into memory list
     */
{
  unsigned int i;
  int istruzione,MaxMemoryAddress;
  char istr[ShortBufferLength];

  MaxMemoryAddress=getMaxAddress();
  for (i=0; i<MaxMemoryAddress; i++) {
    Istr2String((unsigned int)(istruzione=memread(i)),istr);
    if (isalpha(*istr) && getMemoryAlphaNumericStatus())
      sprintf(listitem_memory[i],"Mem%.3x:%s",i,istr);
    else
      sprintf(listitem_memory[i],"Mem%.3x= 0x%.8x    ",i,istruzione);
  }
  XawListChange(list_memory,listitem_memory,MaxMemoryAddress,-1,TRUE);
}

void reshow_cache(void)
     /*
     ** Write into cache list
     */
{
  unsigned int i;
  int istruzione;
  char istr[ShortBufferLength];

  for (i=0; i<item_cache; i++) {
    Istr2String((unsigned int)(istruzione=data(i)),istr);
    if (isalpha(*istr) && getCacheAlphaNumericStatus())
      sprintf(listitem_cache[i],"Cha%.3x:%s",i,istr);
    else
      sprintf(listitem_cache[i],"Cha%.3x= 0x%.8x    ",i,istruzione);
  }
  XawListChange(list_cache,listitem_cache,(int) item_cache,-1,TRUE);
}

void reshow_reg(void)
     /*
     ** Write into registers list
     */
{
  unsigned int i;

  for (i=0; i<item_reg-2; i++)
    sprintf(listitem_reg[i],"Reg%x= 0x%.8x   ",i,ritornaregistro(i));
 sprintf(listitem_reg[item_reg-2],"  Hi= 0x%.8x   ",ritornaregistro(item_reg-2));
 sprintf(listitem_reg[item_reg-1],"  Lo= 0x%.8x   ",ritornaregistro(item_reg-1));
  XawListChange(list_reg,listitem_reg,(int) item_reg,-1,TRUE);
}

void show_total(Widget total)
     /*
     ** Update the state of the label for total, miss and hit
     */
{
  unsigned int totali,miss,hit;
  float hitPercent=0.0,missPercent=0.0;
  char etichetta[MediumBufferLength];

  statistic(&miss,&totali);
  hit=totali-miss;
  if (totali != 0) {
    hitPercent=((1.0*hit)/totali)*100;
    missPercent=((1.0*miss)/totali)*100;
  }
  sprintf(etichetta,"Total: %d [Miss:%d %2.2f%%] [Hit:%d %2.2f%%]",totali,miss,
	  missPercent,hit,hitPercent);
  XtVaSetValues(total,XtNlabel,etichetta,NULL);
}

void show_scrittura(Widget obj)
     /*
     ** Update the state of scrittura
     */
{
  XtVaSetValues(obj,XtNlabel,getStringWritemode(),NULL);
}

void show_politica(Widget obj)
     /*
     ** Update the state of politica
     */
{
  XtVaSetValues(obj,XtNlabel,getStringPolicy(),NULL);
}

void show_replace_mode(Widget obj)
     /*
     ** Update the state of replace_mode
     */
{
  XtVaSetValues(obj,XtNlabel,getStringReplacement(),NULL);
}

int TestPC(int *PC,String stringPC)
     /*
     ** Set PC and show it
     */
{
  int newPC;

  sscanf(stringPC,"%8x",&newPC);
  if ((newPC >= 0) && (newPC < getMaxAddress())) {
    *PC=newPC;
    return(True);
  }
  else
    return(False);
}

int SetLdm(String stringParse, unsigned int Maxaddress)
     /*
     ** Set a Ldm with the user value
     */
{
   unsigned int ldm;
   int value;

   if(sscanf(stringParse,"%x=%8x",&ldm,&value) != 2)
     return(SetLdmWithoutValue);
   if ((ldm >=0) && (ldm < Maxaddress)) {
     polwritewithoutstat(ldm,value); /* thus write in cache too */
     return(True);
   }
   else
     return(ExecOutPC);
}

int Calculate_relative_shell_position_and_set(Widget pshell)
     /*
     ** Calculate the position of the dialog and set it before popup it
     */
{
  Position x, y;
  Dimension width, height;

  XtVaGetValues(topLevel, XtNwidth, &width, XtNheight, &height, NULL);
  XtTranslateCoords(topLevel,            /* Widget */
		    (Position) width/2,  /*  x */
		    (Position) height/2, /*  y */
		    &x,&y);              /* coords on root window */
  XtVaSetValues(pshell,XtNx,x,XtNy,y,NULL);
  return(True);
}

int Shell_position_on_right(Widget pshell)
     /*
     ** Calculate the position of the dialog on the right of the main window
     ** and set it before popup it
     */
{
  Position x, y;
  Dimension width, height;

  XtVaGetValues(topLevel, XtNwidth, &width, XtNheight, &height, NULL);
  XtTranslateCoords(topLevel,           /* Widget */
		    (Position) width+10,/* /2,    x */
		    (Position) 0,       /*height/2,   y */
		    &x,&y);             /* coords on root window */
  XtVaSetValues(pshell,XtNx,x,XtNy,y,NULL);
  return(True);
}

int Shell_position_generic(Widget w, Widget pshell)
     /*
     ** Set pshell in middle of w
     */
{
  Position x, y;
  Dimension width, height;

  XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);
  XtTranslateCoords(w,                   /* Widget */
		    (Position) width/2,  /* x */
		    (Position) height/2, /* y */
		    &x,&y);              /* coords on root window */
  XtVaSetValues(pshell,XtNx,x,XtNy,y,NULL);
  return(True);
}

int make_frames(Pixmap *frames)
     /*
     ** Prepare all the frames to show
     */
{
  frames[0]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_12_bits,crono_base_12_width,
				  crono_base_12_height);
  frames[1]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_1_bits,crono_base_1_width,
				  crono_base_1_height);
  frames[2]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_2_bits,crono_base_2_width,
				  crono_base_2_height);
  frames[3]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_3_bits,crono_base_3_width,
				  crono_base_3_height);
  frames[4]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_4_bits,crono_base_4_width,
				  crono_base_4_height);
  frames[5]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_5_bits,crono_base_5_width,
				  crono_base_5_height);
  frames[6]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_6_bits,crono_base_6_width,
				  crono_base_6_height);
  frames[7]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_7_bits,crono_base_7_width,
				  crono_base_7_height);
  frames[8]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_8_bits,crono_base_8_width,
				  crono_base_8_height);
  frames[9]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_9_bits,crono_base_9_width,
				  crono_base_9_height);
  frames[10]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_10_bits,crono_base_10_width,
				  crono_base_10_height);
  frames[11]=XCreateBitmapFromData(XtDisplay(topLevel),
				  RootWindowOfScreen(XtScreen(topLevel)),
				  crono_base_11_bits,crono_base_11_width,
				  crono_base_11_height);
  return(True);
}

/*
** PAY ATTENTION!!!
** COLOR HANDLER UNDER THIS LINE
** pay attentio, unstable code
*/
/*
static void SetBackgroundColor(Widget w, String color)
{
  XtVaSetValues(w,XtVaTypedArg,XtNbackground,XtRString,color,
                  strlen(color)+1,NULL);
}

static void SetForegroundColor(Widget w, String color)
{
  XtVaSetValues(w,XtVaTypedArg,XtNforeground,XtRString,"red",
                  strlen("red"+1),NULL);
}

void SetBackground2white(Widget w)
{
  SetBackgroundColor(w,"white");
}
*/
