/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "opthandler.h"
#include "ErrorMessage.h"
#include "MemEditor.h"
#include "Edit.h"
#include "GUI_support.h"
#include "loader.h"
#include "policy.h"
#include "memory.h"
#include "global.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */

/* BEGIN LOCAL DEFINE */
#define TmpFileName "tmp"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern Widget total;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateMemEditor(void);
static void PopdownMemEditor(Widget w,XtPointer client_data,
			     XtPointer call_data);
static void MemEditorDone(Widget w,XtPointer client_data,XtPointer call_data);

/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_MemEditor,cancelME;
static Widget start_address_value,finish_address_value;
static int CreatedMemEditor=False;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateMemEditor(void)
     /*
     ** Create the widgets for the editor
     */
{
  Widget form_MemEditor,start_address,finish_address;
  Widget doneME;
  XtTranslations Translations; /* to parse the translation table */
  static XawTextBlock add_block;

  Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
  pshell_MemEditor = XtVaCreatePopupShell(
	"xmasMemEditor",
	transientShellWidgetClass,
	topLevel,NULL);
        form_MemEditor = XtVaCreateManagedWidget(
           "form_MemEditor",
	   formWidgetClass,
	   pshell_MemEditor,NULL);
	start_address = XtVaCreateManagedWidget(
	   "start_address",
	   labelWidgetClass,
	   form_MemEditor,NULL);
	start_address_value = XtVaCreateManagedWidget(
	   "start_address_value",
	   asciiTextWidgetClass,
	   form_MemEditor,
	   XtNeditType, XawtextEdit,
	   XtNfromHoriz,start_address,NULL);
	finish_address = XtVaCreateManagedWidget(
	   "finish_address",
	   labelWidgetClass,
	   form_MemEditor,
	   XtNfromVert,start_address,NULL);
	finish_address_value = XtVaCreateManagedWidget(
	   "finish_address_value",
	   asciiTextWidgetClass,
	   form_MemEditor,
	   XtNeditType, XawtextEdit,
	   XtNfromVert,start_address,
	   XtNfromHoriz,finish_address,NULL);
	doneME = XtVaCreateManagedWidget(
	   "doneME",
	   commandWidgetClass,
	   form_MemEditor,
	   XtNfromVert,finish_address,NULL);
	cancelME = XtVaCreateManagedWidget(
	   "cancelME",
	   commandWidgetClass,
	   form_MemEditor,
	   XtNfromVert,finish_address,
	   XtNfromHoriz,doneME,NULL);

  XtOverrideTranslations(start_address_value, Translations);
  XtOverrideTranslations(finish_address_value, Translations);

  XtAddCallback(cancelME,XtNcallback,PopdownMemEditor,(XtPointer)0);
  XtAddCallback(doneME,XtNcallback,MemEditorDone,(XtPointer)0);

  add_block.firstPos = 0;
  add_block.format = XawFmt8Bit;
  add_block.length = (int)strlen("0x");
  add_block.ptr = "0x";
  XawTextReplace(start_address_value,add_block.firstPos,add_block.length,
		       &add_block);
  XawTextReplace(finish_address_value,add_block.firstPos,
		       add_block.length,&add_block);
  XawTextSetInsertionPoint(start_address_value,add_block.length);
  XawTextSetInsertionPoint(finish_address_value,add_block.length);
  CreatedMemEditor=True; /* remeber that it is created */
}

/*
** Action
*/
void PopupMemEditor(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup the editor, if needed create it
     */
{
  if (!CreatedMemEditor)
    CreateMemEditor();
  Calculate_relative_shell_position_and_set(pshell_MemEditor);
  XtPopup(pshell_MemEditor,XtGrabNone);
}

/*
** Callback
*/
static void PopdownMemEditor(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the pshell_BP
     */
{
  XtPopdown(pshell_MemEditor);
}

/*
** Callback
*/
static void MemEditorDone(Widget w, XtPointer client_data, XtPointer call_data)
{
  FILE *fp;
  String start,end;
  char alfanumeric[ShortBufferLength];
  int start_numeric=-1,end_numeric=-1,rit,i;
  unsigned int ldm;
  char tmpPath[LongBufferLength];
  char pidString[SmallBufferLength];

  XtVaGetValues(start_address_value,XtNstring,&start,NULL);
  XtVaGetValues(finish_address_value,XtNstring,&end,NULL);
  if (sscanf(start,"0x%x",&start_numeric) != -1) /* defined */
    if (sscanf(end,"0x%x",&end_numeric) != -1)   /* defined */
      if ((start_numeric >= 0) && (start_numeric < getMaxAddress()))
	/* in memory */
	if ((end_numeric >= 0) && (end_numeric < getMaxAddress()))
	  /* in memory */
	  if (start_numeric <= end_numeric) { /* sequential values */
	    if (getOptionFromName(GeneralPreferencesFile,TmpDir,tmpPath,sizeof(tmpPath)) == True) { /* get the option value TmpDir from the file GeneralPreferencesFile */
	      strcat(tmpPath,TmpFileName);
	      sprintf(pidString,"%d",getpid());
	      strcat(tmpPath,pidString);
	      if ((fp=fopen(tmpPath,"w")) != NULL) {
		fprintf(fp,".text\n");
		for (ldm=start_numeric; ldm <= end_numeric; ldm++) {
		  rit=Istr2String((unsigned int)polread(ldm),alfanumeric);
		  for (i=0; i<strlen(alfanumeric) && isalpha(alfanumeric[i]); i++)
		    alfanumeric[i]=tolower(alfanumeric[i]);
		  if (rit == UnloaderExactDecode)
		    fprintf(fp,"%s\n",alfanumeric);
		}
		fclose(fp);
		EditorGetBufferFromFile(tmpPath,NULL,NULL,0x0);
		unlink(tmpPath);
		show_total(total);
		XtCallCallbacks(cancelME,XtNcallback,0);
	      }
	      else
		ErrorMessage(MemEditorTmpFile,tmpPath,NULL,0);
	    }
	    else
	      ErrorMessage(MemEditorPrefTmpDir,GeneralPreferencesFile,NULL,0);
	  }
	  else
	    ErrorMessage(EndLessThanStart,NULL,NULL,0);
	else
	  ErrorMessage(EndOutMemory,NULL,NULL,(int)getMaxAddress());
      else
	ErrorMessage(StartOutMemory,NULL,NULL,(int)getMaxAddress());
    else
      ErrorMessage(IncorrectEndValue,NULL,NULL,0);
  else
    ErrorMessage(IncorrectStartValue,NULL,NULL,0);
}


