/*      
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <linux/limits.h>
/*
** Awful code!
** to avoid strange compile error, what is the problem?
*/
int scandir(const char *dir, struct dirent ***namelist,
              int (*select)(const struct dirent *),
              int (*compar)(const struct dirent **, const struct dirent **));
int alphasort(const struct dirent **a, const struct dirent **b);

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>          /* Athena ShellWidget */
#include <X11/Xaw/AsciiText.h>  /* Athena AsciiTextWidget */
#include <X11/Xaw/Command.h>    /* Athena CommandWidget */
#include <X11/Xaw/Form.h>       /* Athena FormWidget */
#include <X11/Xaw/Label.h>      /* Athena LabelWidget */
#include <X11/Xaw/List.h>       /* Athena CommandWidget */
#include <X11/Xaw/Viewport.h>   /* Athena Viewport Widget */ 

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "OpenSave.h"
#include "YNShell.h"
#include "GUI_support.h"
#include "esp.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define MAX_COUNT_FILE 2048
#define File 1
#define Dir  2

#define UpdateInterval 4000

#define NumCharForLongFileName 6
#define MaxItemListLength 28
#define MaxNameItemListLength MaxItemListLength - NumCharForLongFileName

#define WidgetWidth 400
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateOpenSave(void);
static void ListChoosed(Widget w,XEvent *event,String *params,
			Cardinal *num_params);
static void OpenSaveDone(Widget w,XEvent *event,String *params,
			 Cardinal *num_params);

static void callback_list(Widget w, XtPointer client_data,
			  XtPointer call_data);
static void callback_done(Widget w, XtPointer client_data,
			  XtPointer call_data);
static void callback_cancel(Widget w, XtPointer client_data,
			    XtPointer call_data);
static void callback_delete(Widget w, XtPointer client_data,
			    XtPointer call_data);
void refresh(void);
void refresh_path(void);
static void FileDir(unsigned int indexList);
static void Delete(void);
static int generic_text_reset(Widget text,char *ptr);

static void UpdateListofFiles(XtPointer w, XtIntervalId *id);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
struct fileInfo {
  String realFileName;
  unsigned int kind;
};

static Widget done;
static Widget cancel;
static Widget actual_dir;
static Widget list;
static Widget text_file;
static Widget xmasOpenSave;
static Widget file_name;
static String listitems[MAX_COUNT_FILE];
static struct fileInfo listitemsInternal[MAX_COUNT_FILE];
static int CreatedOpenSave=False;
static void (*funct_to_run)(String);

static XtIntervalId UpdateFilesTimer;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void UpdateListofFiles(XtPointer w, XtIntervalId *id)
     /*
     ** Update the list of list every UpdateInterval usec
     */
{
  refresh(); /* refresh the list of files */
  UpdateFilesTimer=XtAppAddTimeOut(XtWidgetToApplicationContext(w),
				   UpdateInterval,UpdateListofFiles,w);
}

/*
** callback associated to list Widget
*/
static void callback_list(Widget w, XtPointer client_data, XtPointer call_data)
{
  XawListReturnStruct *list_struct = (XawListReturnStruct *) call_data;

  FileDir(list_struct->list_index);
}

/*
** callback associated to done Widget
*/
static void callback_done(Widget w, XtPointer client_data, XtPointer call_data)
{
  String nomefile;

  XtVaGetValues(text_file,XtNstring,&nomefile,NULL);
  if (strcmp(nomefile,"") != 0) { /* not empty name */
    XtCallCallbacks(cancel,XtNcallback,0);
    funct_to_run(nomefile);
  }
}

/*
** callback associated to cancel Widget
*/
static void callback_cancel(Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopdown(xmasOpenSave);
  XtRemoveTimeOut(UpdateFilesTimer);
}

/*
** callback associated to cancel Widget
*/
static void callback_delete(Widget w, XtPointer client_data, XtPointer call_data)
{
  String nomefile;

  XtVaGetValues(text_file,XtNstring,&nomefile,NULL);
  if (strcmp(nomefile,"") != 0) /* not empty name */
    PopupYN(Delete,"Are you sure to delete?",AbsoluteApplicationMiddle,NULL);
}

static void CreateOpenSave(void)
     /*
     ** Create the widget and the correct relations between them
     */
{
  static Widget form;
  static Widget view;
  static Widget delete;

  XtTranslations Translations; /* to parse the translation table */

  XtActionsRec AllActions[] = { /* action to add */
    {"ListChoosed",ListChoosed},
    {"OpenSaveDone",OpenSaveDone}
  };

  xmasOpenSave = XtVaCreatePopupShell(
	"xmasOpenSave",
	transientShellWidgetClass,
	topLevel,NULL);
  form = XtVaCreateManagedWidget(
	"form",
	formWidgetClass,
	xmasOpenSave,NULL);
  actual_dir = XtVaCreateManagedWidget(
	"actual_dir",
	labelWidgetClass,
	form,NULL);
  view = XtVaCreateManagedWidget(
	"view",
	viewportWidgetClass,
	form,
	XtNfromVert,actual_dir,
	XtNallowVert,True,
	XtNallowHoriz,True,
	XtNheight,250,
	XtNwidth,WidgetWidth,NULL);
  list = XtVaCreateManagedWidget(
	"list",
	listWidgetClass,
	view,
	XtNforceColumns,True,
	XtNdefaultColumns,2,NULL);
  file_name = XtVaCreateManagedWidget(
	"file_name",
	labelWidgetClass,
	form,
	XtNfromVert,view,NULL);
  text_file = XtVaCreateManagedWidget(
	"text_file",
	asciiTextWidgetClass,
	form,
	XtNfromVert,view,
	XtNfromHoriz,file_name,
	XtNeditType, XawtextEdit,
	XtNscrollVertical,XawtextScrollWhenNeeded,
	XtNwidth,315,NULL);
  done = XtVaCreateManagedWidget(
	"done",
	commandWidgetClass,
	form,
	XtNfromVert,file_name,NULL);
  cancel = XtVaCreateManagedWidget(
	"cancel",
	commandWidgetClass,
	form,
	XtNfromVert,file_name,
	XtNfromHoriz,done,NULL);
  delete = XtVaCreateManagedWidget(
	"delete",
	commandWidgetClass,
	form,
	XtNfromVert,file_name,
	XtNfromHoriz,cancel,
	XtNhorizDistance,32,NULL);

  XtAppAddActions(app_context, AllActions, XtNumber(AllActions));

  XtAddCallback(list,XtNcallback,callback_list,(XtPointer)0);
  XtAddCallback(done,XtNcallback,callback_done,(XtPointer)0);
  XtAddCallback(cancel,XtNcallback,callback_cancel,(XtPointer)0);
  XtAddCallback(delete,XtNcallback,callback_delete,(XtPointer)0);

  Translations = XtParseTranslationTable
     ("<KeyPress>Return: OpenSaveDone()\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
  XtOverrideTranslations(text_file,Translations);
  Translations = XtParseTranslationTable
    ("<Btn1Down>(2): Set() ListChoosed()\n\
      <Btn1Down>,<Btn1Up>: Set() Notify()");
  XtOverrideTranslations(list,Translations);

  XtVaSetValues(actual_dir,XtNwidth,350,NULL);
  CreatedOpenSave=True;
}

void PopupOpenSave(Widget w,String etichetta,void (*exec_funct)(String filename))
     /*
     ** Popup the OpenSave shell, if needed create it
     */
{
  if (!CreatedOpenSave)
    CreateOpenSave();
  Shell_position_generic(w,xmasOpenSave);
  if (exec_funct == NULL)
    ErrorMessage(OpenSaveFunctionError,NULL,NULL,0);
  else if (etichetta == NULL)
    ErrorMessage(OpenSaveStringError,NULL,NULL,0);
  else {
    XtPopup(xmasOpenSave,XtGrabNone);
    XtVaSetValues(file_name,XtNlabel,etichetta,NULL);
    funct_to_run=exec_funct;
    refresh();
    refresh_path(); /* refresh the actual directory */
    UpdateFilesTimer=XtAppAddTimeOut(XtWidgetToApplicationContext(topLevel),
				     UpdateInterval,UpdateListofFiles,
				     topLevel);
  }
}

void refresh(void)
     /*
     ** Refresh the list of files
     */
{
  struct dirent **namelist;
  struct stat kind_of_file;
  unsigned int length;
  int n,i=0,NameLength;
  char dir[LongBufferLength];

  n = scandir(".", &namelist, 0, alphasort);
  if (n < 0) {
    /*perror("scandir");*/
    getcwd(dir,sizeof(dir));
    ErrorMessage(OpenSaveUnableScan,dir,NULL,0);
  }
  else {
    i=0;
    while(i<n) {
      listitems[i]=realloc(listitems[i],MaxItemListLength);
      listitemsInternal[i].realFileName=realloc(listitemsInternal[i].realFileName,NAME_MAX+1);
      memset(listitems[i],' ',MaxItemListLength);
      listitems[i][MaxItemListLength-1]='\0';
      NameLength=strlen(namelist[i]->d_name);
      /* length is the minimum length of the string */
      if (MaxNameItemListLength < NameLength)
	length=MaxNameItemListLength;
      else
	length=NameLength;
      strncpy(listitems[i],namelist[i]->d_name,length);
      strcpy(listitemsInternal[i].realFileName,namelist[i]->d_name);
      /* if too long add three dot notation */
      if (NameLength > MaxNameItemListLength) {
	listitems[i][MaxNameItemListLength]='.';
	listitems[i][MaxNameItemListLength+1]='.';
	listitems[i][MaxNameItemListLength+2]='.';
	length+=3;
      }
      stat(namelist[i]->d_name,&kind_of_file);
      /* add / if it is a directory */
      if (S_ISDIR(kind_of_file.st_mode)) {
	listitems[i][length]='/';
	listitemsInternal[i].kind=Dir;
      }
      else
	listitemsInternal[i].kind=File;
      i++;
    }
    XawListChange(list,listitems,i,-1,True);
  }
}

void refresh_path(void)
     /*
     ** Refresh the directory showd by the actual_dir label
     */
{
   char  path[LongBufferLength];

   if (getcwd(path,sizeof(path)) != NULL)
     XtVaSetValues(actual_dir,XtNlabel,path,NULL);
}

/*
** Action
*/
static void ListChoosed(Widget w,XEvent *event,String *params,
			Cardinal *num_params)
     /*
     ** Action for double click on a list entry
     */
{
  XawListReturnStruct *item;

  item=XawListShowCurrent(w);
  if ((item->list_index != -1) && (strchr(item->string,'/') == NULL))
    /* double click on a file, not a directory */
    XtCallCallbacks(done,XtNcallback,0);
}

static void OpenSaveDone(Widget w,XEvent *event,String *params,
			 Cardinal *num_params)
     /*
     ** Call the callback associated to done command
     */
{
  XtCallCallbacks(done,XtNcallback,0);
}

static void FileDir(unsigned int indexList)
     /*
     ** Decide the kind of file with the returned value of chdir
     */
{
  String inode;

  inode=listitemsInternal[indexList].realFileName;
  if (listitemsInternal[indexList].kind == Dir) {
    chdir(inode);
    /*if (ret != -1) { * -1 cannot change directory */
    refresh();
    refresh_path();
    XtVaSetValues(text_file,XtNstring,"",NULL);
  }
  else
    XtVaSetValues(text_file,XtNstring,inode,NULL);
}

static void Delete(void)
     /*
     ** Delete a file and update the file name text widget
     */
{
  String nomefile;

  XtVaGetValues(text_file,XtNstring,&nomefile,NULL);
  if (strcmp(nomefile,"") != 0) { /* not empty name */
    if (unlink(nomefile) != -1) {
      refresh();
      generic_text_reset(text_file,"");
    }
    else
      ErrorMessage(OpenSaveDeleteError,nomefile,NULL,0);
  }
}

static int generic_text_reset(Widget text,char *ptr)
     /*
     ** Erase the widget text and place the string referred by ptr
     */
{
  XawTextBlock add_block;
  String oldtext;

  XtVaGetValues(text,XtNstring,&oldtext,NULL);
  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=strlen(ptr);
  add_block.ptr=ptr;
  XawTextDisableRedisplay(text);
  XawTextReplace(text,0,(XawTextPosition)strlen(oldtext),&add_block);
  XawTextEnableRedisplay(text);
  return(True);
}
