/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Label.h>     /* Athena Label Widget */
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "Preferences.h"
#include "general_support.h"
#include "GUI_support.h"
#include "opthandler.h"
#include "YNShell.h"
/*#include "esp.h"*/
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define AsciiTextWidth  500
#define AsciiTextHeight 300

#define GeneralChoice     0
#define PrintChoice       1
#define CacheChoice       2
#define MemoryChoice      3
#define RegistersChoice   4
#define EditorChoice      5
#define EmailChoice       6
#define BreakpointsChioce 7
#define ExecutorChoice    8
#define PathTracerChoice  9
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void PopdownPreferences(Widget w, XtPointer client_data, XtPointer call_data);
static void Save(Widget w, XtPointer client_data, XtPointer call_data);
static void DefaultSetting(Widget w, XtPointer client_data, XtPointer call_data);
static int clear_all_text(Widget asciitext,char *ptr);
static int ripetitive_add_tail_text(Widget asciitext,char *ptr);
static int UpdateGenericFile(String filename);

static void LoadCustomizablePreferences(String PreferencesFilename);
static void ShowPreferences(Widget w, XtPointer client_data, XtPointer call_data);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static int CreatedPreferences=False;
static Widget pshell_preferences;
static Widget text;
static Widget default_setting;
static char completefilename[2048];
static char *(*DefaultPrefFunction)(void);
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreatePreferences(void)
     /*
     ** Create the widget structure for the preferences shell
     */
{
  Widget form_preferences;
  Widget save,wclose;
  Widget buttonmodules,menumodules;
  Widget general,editor,executor,pathtracer;
  Widget breakpoints,email,print,registers,cache,memory;

  pshell_preferences = XtVaCreatePopupShell(
	"xmasPreferences",
	transientShellWidgetClass,
	topLevel,NULL);
  form_preferences = XtVaCreateManagedWidget(
           "form_preferences",
	   formWidgetClass,
	   pshell_preferences,NULL);
  buttonmodules = XtVaCreateManagedWidget(
	"buttonmodules",
	menuButtonWidgetClass,
	form_preferences,
	XtNmenuName,"menumodules",
	NULL);
   menumodules = XtCreatePopupShell(
	"menumodules",
	simpleMenuWidgetClass,
	buttonmodules,
	NULL,0);
        general = XtVaCreateManagedWidget(
		  "general",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	editor = XtVaCreateManagedWidget(
		  "editor",
		  smeBSBObjectClass,
		  menumodules,
		  XtNsensitive,False,
		  NULL);
	executor = XtVaCreateManagedWidget(
		  "executor",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	breakpoints = XtVaCreateManagedWidget(
		  "breakpoints",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	pathtracer = XtVaCreateManagedWidget(
		  "pathtracer",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	email = XtVaCreateManagedWidget(
		  "email",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	print = XtVaCreateManagedWidget(
		  "print",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	registers = XtVaCreateManagedWidget(
		  "registers",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	cache = XtVaCreateManagedWidget(
		  "cache",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
	memory = XtVaCreateManagedWidget(
		  "memory",
		  smeBSBObjectClass,
		  menumodules,
		  NULL);
  text = XtVaCreateManagedWidget(
	   "text",
	   asciiTextWidgetClass,
	   form_preferences,
	   XtNeditType, XawtextEdit,
	   XtNwidth,AsciiTextWidth,
	   XtNheight,AsciiTextHeight,
	   NULL);
  save = XtVaCreateManagedWidget(
	   "save",
	   commandWidgetClass,
	   form_preferences,
	   NULL);
  wclose = XtVaCreateManagedWidget(
	   "close",
	   commandWidgetClass,
	   form_preferences,
	   NULL);
  default_setting = XtVaCreateManagedWidget(
	   "default_setting",
	   commandWidgetClass,
	   form_preferences,
	   NULL);

  XtAddCallback(general,XtNcallback,ShowPreferences,(XtPointer)GeneralChoice);
  XtAddCallback(editor,XtNcallback,ShowPreferences,(XtPointer)EditorChoice);
  XtAddCallback(executor,XtNcallback,ShowPreferences,(XtPointer)ExecutorChoice);
  XtAddCallback(breakpoints,XtNcallback,ShowPreferences,(XtPointer)BreakpointsChioce);
  XtAddCallback(pathtracer,XtNcallback,ShowPreferences,(XtPointer)PathTracerChoice);
  XtAddCallback(email,XtNcallback,ShowPreferences,(XtPointer)EmailChoice);
  XtAddCallback(print,XtNcallback,ShowPreferences,(XtPointer)PrintChoice);
  XtAddCallback(registers,XtNcallback,ShowPreferences,(XtPointer)RegistersChoice);
  XtAddCallback(cache,XtNcallback,ShowPreferences,(XtPointer)CacheChoice);
  XtAddCallback(memory,XtNcallback,ShowPreferences,(XtPointer)MemoryChoice);

  XtAddCallback(wclose,XtNcallback,PopdownPreferences,(XtPointer)0);
  XtAddCallback(save,XtNcallback,Save,(XtPointer)0);
  /*XtAddCallback(save,XtNcallback,PopdownPreferences,(XtPointer)0);*/
  XtAddCallback(default_setting,XtNcallback,DefaultSetting,(XtPointer)0);

  CreatedPreferences=True;
}

/*
** Action
*/
void PopupPreferencesAction(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup the Preferences shell
     */
{
  /*
  ** param[0]: first parameter passed to PopupPreferencesAction
  **           in interfaccia.c with PopupPreferencesAction(preferences) call
  */
  PopupPreferences(params[0]);
}

void PopupPreferences(String file)
     /*
     ** Popup prefereces shell
     */
{
  if (!CreatedPreferences)
    CreatePreferences();
  DefaultPrefFunction=DefaultGeneralPreferences;
  Shell_position_generic(topLevel,pshell_preferences);
  XtPopup(pshell_preferences,XtGrabNone);
  LoadCustomizablePreferences(file);
}

/*
** Callback
*/
static void PopdownPreferences(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the preferences shell
     */
{
  XtPopdown(pshell_preferences);
}

/*
** Callback
*/
static void Save(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Save the preferences file
     */
{
  FILE *fp;
  String textfile;

  XtVaGetValues(text,XtNstring,&textfile,NULL);
  if ((fp=fopen(completefilename,"w")) != NULL) {
    fwrite(textfile,strlen(textfile),1,fp);
    fclose(fp);
    PopupY("Preferences saved!",OnMousePosition,w);
  } else
    ErrorMessage(PreferencesCannotSave,completefilename,NULL,0);
}

/*
** Callback
*/
static void DefaultSetting(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Reset the showed preferences with a default setting
     */
{
  char *DefaultPrefString;

  DefaultPrefString=(*DefaultPrefFunction)();
  clear_all_text(text,DefaultPrefString);    
  XawTextSetInsertionPoint(text,0);
}

static int UpdateGenericFile(String filename)
     /*
     ** Update the text widget with the selected file
     */
{
  char buf[HugeBufferLength]="\0";
  int fd,ret;

  if ((fd=open(filename,0)) != -1) {
    /* sizeof(buf)-1: the last char is always a \0 for terminate the string
                      read less char than its size, for sake of accuracy: -1
    */
    clear_all_text(text,"");
    XawTextDisableRedisplay(text); /* avoid a boring blinking */
    while ((ret=read(fd,buf,sizeof(buf)-1)) != 0) {
      ripetitive_add_tail_text(text,buf);
      memset(buf,0,sizeof(buf)); /* overwrite the previous read result */
    }
    close(fd);
    XawTextEnableRedisplay(text); /* enable the widget to redisplay itself */
    return(True);
  } else
      return(False);
}

static int clear_all_text(Widget asciitext,char *ptr)
     /*
     ** Clear the widget asciitext
     */
{
  XawTextBlock add_block;
  String oldtext;

  XtVaGetValues(asciitext,XtNstring,&oldtext,NULL);
  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=strlen(ptr);
  add_block.ptr=ptr;
  XawTextDisableRedisplay(asciitext);
  XawTextReplace(asciitext,0,(XawTextPosition)strlen(oldtext),&add_block);
  XawTextEnableRedisplay(asciitext);
  return(True);
}

static int ripetitive_add_tail_text(Widget asciitext,char *ptr)
     /*
     ** Add the string ptr to end of widget text
     ** NB: doesn't disable/enable widget redisplay
     */
{
  XawTextBlock add_block;
  String oldtext;

  XtVaGetValues(asciitext,XtNstring,&oldtext,NULL);
  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=strlen(ptr);
  add_block.ptr=ptr;
  /*XawTextDisableRedisplay(asciitext);*/
  XawTextReplace(asciitext,(XawTextPosition)strlen(oldtext),(XawTextPosition)strlen(oldtext),&add_block);
  /*XawTextEnableRedisplay(asciitext);*/
  return(True);
}

static void LoadCustomizablePreferences(String PreferencesFilename)
     /*
     ** Load the file relative to the module choosed, load the file pointed
     ** by client_data
     */
{
  char XmasDir[XmasDirLength];

  if (XmasDirStatus(XmasDir,sizeof(XmasDir)) == True) {
    strcat(XmasDir,"/");
    strcat(XmasDir,PreferencesFilename);
    strcpy(completefilename,XmasDir);
    if (UpdateGenericFile(XmasDir) == True)
      strcpy(completefilename,XmasDir);
    else {
      clear_all_text(text,"");
      DefaultSetting(NULL,NULL,NULL);
      ErrorMessage(PreferencesCannotLoadFile,PreferencesFilename,NULL,0);
    }
  }
  else
    ErrorMessage(PreferencesCannotEnterDirectory,NULL,NULL,0);
}

/*
** Callback
*/
static void ShowPreferences(Widget w, XtPointer client_data,
			    XtPointer call_data)
     /*
     ** choose which preferences to show
     */
{
  unsigned int choice = (unsigned int) client_data;

  switch (choice) {
  case GeneralChoice:
    DefaultPrefFunction=DefaultGeneralPreferences;
    LoadCustomizablePreferences(GeneralPreferencesFile);
    break;
  case PrintChoice:
    DefaultPrefFunction=DefaultPrintPreferences;
    LoadCustomizablePreferences(PrintPreferencesFile);
    break;
  case CacheChoice:
    DefaultPrefFunction=DefaultCachePreferences;
    LoadCustomizablePreferences(CachePreferencesFile);
    break;
  case MemoryChoice:
    DefaultPrefFunction=DefaultMemoryPreferences;
    LoadCustomizablePreferences(MemoryPreferencesFile);
    break;
  case RegistersChoice:
    DefaultPrefFunction=DefaultRegisterPreferences;
    LoadCustomizablePreferences(RegisterPreferencesFile);
    break;
  case EditorChoice:
    DefaultPrefFunction=DefaultEditorPreferences;
    LoadCustomizablePreferences(EditorPreferencesFile);
    break;
  case EmailChoice:
    DefaultPrefFunction=DefaultEmailPreferences;
    LoadCustomizablePreferences(EmailPreferencesFile);
    break;
  case BreakpointsChioce:
    DefaultPrefFunction=DefaultBreakpointsPreferences;
    LoadCustomizablePreferences(BreakpointsPreferencesFile);
    break;
  case ExecutorChoice:
    DefaultPrefFunction=DefaultExecutorPreferences;
    LoadCustomizablePreferences(ExecutorPreferencesFile);
    break;
  case PathTracerChoice:
    DefaultPrefFunction=DefaultPathTracerPreferences;
    LoadCustomizablePreferences(PathTracerPreferencesFile);
  }
}
