/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "opthandler.h"
#include "ErrorMessage.h"
#include "Print.h"
#include "Choose.h"
#include "general_support.h"
#include "Edit.h"
#include "config.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define tmpfile "tmpXmasBufferText"
#define tmpfilelatex "tmpXmasBufferLatex"

#define NUM_MENU_FILE 2
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreatePrintText(char *printer,unsigned int AutomaticLF_LFCR);
static void CreatePrintLaTeX(char *printer);
static int PrintTextFormat(String filename, char *printer,unsigned int AutomaticLF_LFCR);
static int PrintLaTeXFormat(String filename, char *printer,char *command);

static void BodyPrint(FILE *fp);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void Print(char *type,char *printer, unsigned int AutomaticLF_LFCR)
     /*
     ** Decide which kind of device to use
     */
{
  if (strcmp(type,TEXT_PRINTER)==0)
    CreatePrintText(printer,AutomaticLF_LFCR);
  if (strcmp(type,PS_PRINTER)==0)
    CreatePrintLaTeX(printer);
}

static void CreatePrintText(char *printer, unsigned int AutomaticLF_LFCR)
     /*
     ** Prepare the program to print with a text printer on printer
     ** name *printer
     */
{
  FILE *fp;
  char tmpPath[MediumBufferLength];
  char pidString[SmallBufferLength];

  if (getOptionFromName(GeneralPreferencesFile,TmpDir,tmpPath,sizeof(tmpPath)) == True){
    strcat(tmpPath,tmpfile);
    sprintf(pidString,"%d",getpid()); /* unique name */
    strcat(tmpPath,pidString);
    if ((fp=fopen(tmpPath,"w")) == NULL)
      ErrorMessage(PrintTmpFileError,tmpPath,NULL,0);
    else {
      BodyPrint(fp); /* File and Print Info + Program */
      fclose(fp);
      PrintTextFormat(tmpPath,printer,AutomaticLF_LFCR);
      if (unlink(tmpPath) == -1)
	ErrorMessage(PrintCommandUnlink,tmpPath,NULL,0);
    }
  }
  else
    ErrorMessage(PrintUnabletoGetTmpDir,GeneralPreferencesFile,NULL,0);
}

static int PrintTextFormat(String filename, char *printer,unsigned int AutomaticLF_LFCR)
     /*
     ** Print filename in text mode
     */
{
  char command[MediumBufferLength];
  char tmpComand[MediumBufferLength];
  char *Option;

  if (AutomaticLF_LFCR == True)
    Option=PrintTextCRLF;
  else
    Option=PrintTextLF;
  if (getOptionFromName(PrintPreferencesFile,Option,tmpComand,sizeof(tmpComand)) != True) { /* can't get an option value */
      ErrorMessage(PrintUnalbleGetCommand,GeneralPreferencesFile,Option,0);
    return(False);
  }
  sprintf(command,tmpComand,filename,printer);
  if (system(command) != 0) {
    ErrorMessage(PrintCommandText,command,NULL,0);
    return(False);
  }
  return(True);
}

static void CreatePrintLaTeX(char *printer)
     /*
     ** Prepare the program to print with LaTeX on printer name *printer
     */
{
  FILE *fp;
  char cwd[MediumBufferLength];
  char command[MediumBufferLength];
  char tmpPath[MediumBufferLength];
  char tmpFile[MediumBufferLength];
  char pidString[SmallBufferLength];

  if (getOptionFromName(GeneralPreferencesFile,TmpDir,tmpFile,sizeof(tmpFile)) == True){
    strncpy(tmpPath,tmpFile,sizeof(tmpPath));
    strcat(tmpFile,tmpfilelatex);
    sprintf(pidString,"%d",getpid()); /* unique name */
    strcat(tmpFile,pidString);
    if ((fp=fopen(tmpFile,"w")) == NULL)
      ErrorMessage(PrintTmpFileError,tmpFile,NULL,0);
    else {
      /* LaTeX Header */
      fprintf(fp,"\\documentclass [12pt] {article}\n\
\\begin{document}\n\
\\begin{verbatim}\n");
      /* File and Print Info + Program */
      BodyPrint(fp);
      /* LaTeX Tail */
      fprintf(fp,"\\end{verbatim}\n\\end{document}");
      fclose(fp);
      getcwd(cwd,sizeof(cwd)); /* store the cwd to come back */
      chdir(tmpPath); /* jump to temporary directory */
      PrintLaTeXFormat(tmpFile,printer,command);
      /* to delete all LaTex files */
      sprintf(command,"rm -f %s*",tmpFile);
      if (system(command) == -1)
	ErrorMessage(PrintCommandRM,command,NULL,0);
      chdir(cwd); /* jump back to old directory */
    }
  }
  else
    ErrorMessage(PrintUnabletoGetTmpDir,GeneralPreferencesFile,NULL,0);
}

static int PrintLaTeXFormat(String filename, char *printer, char *command)
     /*
     ** Print filename in LaTeX mode
     */
{
  char tmpCommand[MediumBufferLength];

  if (getOptionFromName(PrintPreferencesFile,PrintLaTeX,tmpCommand,sizeof(tmpCommand)) != True){
    ErrorMessage(PrintUnalbleGetCommand,GeneralPreferencesFile,PrintLaTeX,0);
    return(False);
  }
  sprintf(command,tmpCommand,filename,filename,filename,printer,filename);
  if (system(command) != 0) {
    ErrorMessage(PrintCommandLaTeX,command,NULL,0);
    return(False);
  }
  return(True);
} 

static void BodyPrint(FILE *fp)
     /*
     ** Write the file and print information and the program
     ** NB: FOR EVERY PRINT MODE
     */
{
  char *program;
  char filename[MediumBufferLength];
  char directory[LongBufferLength];
  char StartPC[SmallBufferLength];
  char Date[ShortBufferLength],Clock[ShortBufferLength];

  getFileDirectory(directory);
  getFileName(filename);
  getProgram(&program);
  getStartPC(StartPC);
  DateTime(Date,Clock);
  fprintf(fp,"%s %s print of:\n\
Directory: %s\n\
File name: %s\n\
Date: %s\n\
Time: %s\n\
Start PC: %s\n\
Source code:\n\n\
%s\n",ProgName,VersionString,directory,filename,Date,Clock,StartPC,program);
}

/*
latex %s\ndvips %s.dvi -o %s.ps\nlpr -P%s %s.ps
*/
