/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <ctype.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "Search.h"
#include "GUI_support.h"
#include "memory.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/keysymdef.h>
#include <X11/Shell.h>         /* Athena Shell Widget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command Widget */
#include <X11/Xaw/Label.h>     /* Athena Label Widget */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/List.h>      /* Athena List Widget */

/* BEGIN LOCAL DEFINE */
#define HexPrefix "0x"
#define FoundResult "found at 0x"
#define NotFoundResult "not founded"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern Widget list_memory;
extern String *listitem_memory;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateSearch(void);
static void Done(Widget w, XtPointer client_data,XtPointer call_data);
static void Search(Widget w, XtPointer client_data, XtPointer call_data);
static void ActionSearch(Widget w, XEvent *event, String *params, Cardinal *num_params);
static int AlfaNumSearch(String text);
static int NumSearch(unsigned int numeric_value);
static void UpperAllLetter(char *in,char *out);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_Search,text_search,label_result,search_result;
static Widget search;
static int CreatedSearch=False;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateSearch(void)
     /*
     ** Create the widgets for the Search shell
     */
{
  static Widget form_Search,done;
  static Widget label_search;

  XtTranslations Translations; /* to parse the translation table */

  XtActionsRec AllActions[] = { /* action to add */
    {"ActionSearch",ActionSearch}
  };

  pshell_Search = XtVaCreatePopupShell(
	   "xmasSearch",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   form_Search = XtVaCreateManagedWidget(
              "form_Search",
	      formWidgetClass,
	      pshell_Search,NULL);
	   label_search = XtVaCreateManagedWidget(
	      "label_search",
	      labelWidgetClass,
	      form_Search,NULL);
	   text_search = XtVaCreateManagedWidget(
              "text_search",
	      asciiTextWidgetClass,
	      form_Search,
	      XtNeditType, XawtextEdit,
	      XtNfromHoriz,label_search,NULL);
	   label_result = XtVaCreateManagedWidget(
	      "label_result",
	      labelWidgetClass,
	      form_Search,
	      XtNfromVert,label_search,NULL);
	   search_result = XtVaCreateManagedWidget(
	      "search_result",
	      labelWidgetClass,
	      form_Search,
	      XtNwidth,120,
	      XtNfromVert,label_search,
	      XtNfromHoriz,label_result,NULL);
	   search = XtVaCreateManagedWidget(
              "search",
	      commandWidgetClass,
	      form_Search,
	      XtNfromVert,label_result,NULL);
	   done = XtVaCreateManagedWidget(
	      "done",
	      commandWidgetClass,
	      form_Search,
	      XtNfromVert,label_result,
	      XtNfromHoriz,search,NULL);

   /* add actions */
   XtAppAddActions(app_context, AllActions, XtNumber(AllActions));
   /*
   ** Search Callback
   */
   XtAddCallback(search,XtNcallback,Search,(XtPointer)0);
   XtAddCallback(done,XtNcallback,Done,(XtPointer)0);

   Translations = XtParseTranslationTable
     ("<KeyPress>Return: ActionSearch()\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
   XtOverrideTranslations(text_search, Translations);

   CreatedSearch=True;
}

/*
** Callback
*/
void PopupSearch(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Popup the Search shell
     */
{
  if(!CreatedSearch)
    CreateSearch();
  Calculate_relative_shell_position_and_set(pshell_Search);
  XtPopup(pshell_Search,XtGrabNone); /* get the focus */
}

/*
** Callback
*/
static void Done(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the Search shell
     */
{
  XtPopdown(pshell_Search);
}

/*
** Action
*/

static void ActionSearch(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Call the Search callback
     */
{
  XtCallCallbacks(search,XtNcallback,0);
}

/*
** Callback
*/
static void Search(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Search the value in the memory and show the result in a label and
     ** highlight the relative location of memory if exist
     */
{
  String alfanumeric_value;
  unsigned int numeric_value;
  int address=-1;
  char result[32];

  XawListUnhighlight(list_memory);
  XtVaGetValues(text_search,XtNstring,&alfanumeric_value,NULL);
  if (sscanf(alfanumeric_value,"0x%8x",&numeric_value) == 1)
    /* 1 => hex input */
    address=NumSearch(numeric_value);
  else if (isalpha(*alfanumeric_value) != 0) {
    /* alfanumeric istruction: NOP-like*/
    UpperAllLetter(alfanumeric_value,alfanumeric_value);
    address=AlfaNumSearch(alfanumeric_value);
  }
  else
    ErrorMessage(SearchInputInvalid,NULL,NULL,0);
  if (address == -1) {
    XawListUnhighlight(list_memory);
    strcpy(result,NotFoundResult);
  }
  else {
    XawListHighlight(list_memory,(int)address);
    sprintf(result,"%s%x",FoundResult,address);
  }
  XtVaSetValues(search_result,XtNlabel,result,NULL);
}

static int AlfaNumSearch(String text)
     /*
     ** Search an alfanumeric input into memory and return the address or -1
     */
{
  unsigned int k;
  unsigned int MaxMemoryAddress;

  MaxMemoryAddress=getMaxAddress();
  for (k=0; k < MaxMemoryAddress; k++)
    if (strncmp(text,listitem_memory[k]+7,strlen(text)) == 0)
      return(k);
  return(-1);
}

static int NumSearch(unsigned int numeric_value)
     /*
     ** Search a numeric value into memory and return the address or -1
     */
{
  unsigned int address;
  unsigned int MaxMemoryAddress;

  MaxMemoryAddress=getMaxAddress();
  for (address=0; address < MaxMemoryAddress; address++)
    if (numeric_value == memread(address))
      return(address);
  return(-1);
}

static void UpperAllLetter(char *in,char *out)
     /*
     ** Capitalize all letters of in and output into out
     */
{
  char *tmp;
  int i;

  tmp=in;
  for (i=0; tmp+i<in; i++) {
    out[i]=toupper(in[i]);
  }
}
