/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */
#include <X11/Xaw/Scrollbar.h> /* Athena Scrollbar Widget */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */

/*
** include of application modules
*/
#include "global.h"
#include "Sizer.h"
#include "ErrorMessage.h"
#include "GUI_support.h"

/* BEGIN LOCAL DEFINE */
#define WidgetWidth 300
#define Decimal 10
#define Hex     16
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateSizer(void);
static void PopupSizer(int min, int max, int oldvalue, void (*function)(int), unsigned int Base);
static void PopdownSizer(Widget w, XtPointer client_data, XtPointer call_data);
static void Accept(Widget w, XtPointer client_data, XtPointer call_data);
static void UpdateShow(int value);
static void UpdateLabelwithString(Widget w, char *label);
static void UpdateLabelwithNumber(Widget w, int value);
static void UpdateLabelwithStringAndNumber(Widget w, char *label, int value);

static void scbarthumb(Widget w, XtPointer client_data, XtPointer call_data);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static int CreatedSizer=False;
static Widget pshell_Sizer;
static Widget show,scbar,minLabel,maxLabel,oldvalueLabel;
static Widget inserttext;

static int minvalue,maxvalue,actualvalue;

static unsigned int ShowBase=Hex;

static void (*exec_function)(int);
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateSizer(void)
     /*
     ** Create the widget structure for the Sizer shell
     */
{
  Widget form_Sizer,accept,close;
  Widget insertlabel;

  XtTranslations Translations; /* to parse the translation table */

  pshell_Sizer = XtVaCreatePopupShell(
	   "xmasSizer",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   form_Sizer = XtVaCreateManagedWidget(
              "form_Sizer",
	      formWidgetClass,
	      pshell_Sizer,NULL);
	   minLabel = XtVaCreateManagedWidget(
		   "minLabel",
		   labelWidgetClass,
		   form_Sizer,NULL);
	   scbar = XtVaCreateManagedWidget(
                   "sb",
                   scrollbarWidgetClass,
                   form_Sizer,
                   XtNorientation,XtorientHorizontal,
                   XtNwidth,WidgetWidth,
                   XtNlength,100,
		   XtNfromHoriz,minLabel,NULL);
	   maxLabel = XtVaCreateManagedWidget(
		   "maxLabel",
		   labelWidgetClass,
		   form_Sizer,
		   XtNfromHoriz,scbar,NULL);
	   show = XtVaCreateManagedWidget(
		   "show",
		   labelWidgetClass,
		   form_Sizer,
		   XtNfromVert,scbar,
		   XtNfromHoriz,minLabel,
		   XtNwidth,WidgetWidth,NULL);
	   oldvalueLabel = XtVaCreateManagedWidget(
		   "oldvalueLabel",
		   labelWidgetClass,
		   form_Sizer,
		   XtNfromVert,show,
		   XtNfromHoriz,minLabel,
		   XtNwidth,WidgetWidth,NULL);
	   insertlabel = XtVaCreateManagedWidget(
		   "insertlabel",
		   labelWidgetClass,
		   form_Sizer,
		   XtNfromHoriz,minLabel,
		   XtNfromVert,oldvalueLabel,NULL);
	   inserttext = XtVaCreateManagedWidget(
                   "inserttext",
		   asciiTextWidgetClass,
		   form_Sizer,
		   XtNeditType, XawtextEdit,
		   XtNfromHoriz,insertlabel,
		   XtNfromVert,oldvalueLabel,NULL);
	   accept = XtVaCreateManagedWidget(
                   "accept",
		   commandWidgetClass,
		   form_Sizer,
		   XtNvertDistance,30,
		   XtNfromVert,insertlabel,NULL);
	   close = XtVaCreateManagedWidget(
                   "close",
		   commandWidgetClass,
		   form_Sizer,
		   XtNvertDistance,30,
		   XtNfromHoriz,accept,
		   XtNfromVert,insertlabel,NULL);

  Translations = XtParseTranslationTable
     ("<KeyPress>Return: ActionSearch()\n\
       Ctrl<KeyPress>r: no-op(RingBell)\n\
       Ctrl<KeyPress>s: no-op(RingBell)");
  XtOverrideTranslations(inserttext, Translations);

  XtAddCallback(close,XtNcallback,PopdownSizer,(XtPointer)0);
  XtAddCallback(accept,XtNcallback,Accept,(XtPointer)0);
  XtAddCallback(scbar,XtNjumpProc, scbarthumb,"thumbProc");

  CreatedSizer=True;
}

static void PopupSizer(int min, int max, int oldvalue, void (*function)(int), unsigned int Base)
     /*
     ** Pop up the Sizer shell and initialize the status
     */
{
  if(!CreatedSizer)
    CreateSizer();
  minvalue=actualvalue=min;
  maxvalue=max;
  exec_function=function;
  ShowBase=Base;
  Calculate_relative_shell_position_and_set(pshell_Sizer);
  XtPopup(pshell_Sizer,XtGrabNone); /* get the focus */
  UpdateShow(minvalue);
  UpdateLabelwithNumber(minLabel,min);
  UpdateLabelwithNumber(maxLabel,max);
  UpdateLabelwithStringAndNumber(oldvalueLabel,"Old value: ",oldvalue);
  XtVaSetValues(inserttext,XtNstring,NULL,NULL);
  XawScrollbarSetThumb(scbar,0.0,0.0);
}

void PopupSizerHex(unsigned int min, unsigned int max, int oldvalue, void (*function)(int))
{
  PopupSizer(min,max,oldvalue,function,Hex);
}

void PopupSizerDecimal(unsigned int min, unsigned int max, int oldvalue, void (*function)(int))
{
  PopupSizer(min,max,oldvalue,function,Decimal);
}

/*
** Callback
*/
static void PopdownSizer(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the Sizer shell
     */
{
  XtPopdown(pshell_Sizer);
}

/*
** Callback
*/
static void Accept(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Return the thumb value to exec_function
     */
{
  String manualvalue;
  int uservalue;
  int tmpvalue,ret;

  uservalue=actualvalue;
  XtVaGetValues(inserttext,XtNstring,&manualvalue,NULL);
  if (*manualvalue != '\0') {
    ret=sscanf(manualvalue,"%d",&tmpvalue);
    if ((ret == 1) && (tmpvalue >= minvalue) && (tmpvalue <= maxvalue))
      uservalue=tmpvalue;
    else
      ErrorMessage(SizerManualValueOutofRange,NULL,NULL,0);
  }
  exec_function(uservalue);
  PopdownSizer(NULL,NULL,NULL);
}

static void UpdateShow(int value)
     /*
     ** Update the label for the actual thumb value
     */
{
  char label[32];

  switch (ShowBase) {
  case Decimal: sprintf(label,"New value: %d",value); break;
  case Hex: sprintf(label,"New value: 0x%x",value); break;
  default:
  }
  XtVaSetValues(show,XtNlabel,label,NULL);
}

static void UpdateLabelwithString(Widget w, char *label)
     /*
     ** Update a label widget with the label char pointer
     */
{
  XtVaSetValues(w,XtNlabel,label,NULL);
}

static void UpdateLabelwithNumber(Widget w, int value)
     /*
     ** Update the label widget with a value
     */
{
  char label[SmallBufferLength];

  switch (ShowBase) {
  case Decimal: sprintf(label,"%d",value); break;
  case Hex: sprintf(label,"0x%x",value); break;
  default:
  }
  UpdateLabelwithString(w,label);
}

static void UpdateLabelwithStringAndNumber(Widget w, char *label, int value)
     /*
     ** Update a label widget with the label char pointer and concatenate
     ** the numeric value
     ** label bound up to ShortBufferLength characters
     */
{
  char number[SmallBufferLength];
  char finallabel[ShortBufferLength];

  switch (ShowBase) {
  case Decimal: sprintf(number,"%d",value); break;
  case Hex: sprintf(number,"0x%x",value); break;
  default:
  }
  if ((strlen(label)+strlen(number)) < ShortBufferLength) {
    /*sprintf(finallabel,"%s%s",label,number);*/
    strcpy(finallabel,label);
    strcat(finallabel,number);
  }
  else {
    strncpy(finallabel,label,ShortBufferLength-strlen(number)-1);
    strcat(finallabel,number);
  }
  UpdateLabelwithString(w,finallabel);

}

/*
** Callback
*/
static void scbarthumb(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Calculate the actual value of thumb and update the relative
     ** label widget
     */
{
  float pos = *(float*) call_data;
  float delta;
  char stringvalue[32];
  int decimalvalue;

  delta=(maxvalue-minvalue)*pos;
  sprintf(stringvalue,"%f",delta);
  sscanf(stringvalue,"%d.%d",&actualvalue,&decimalvalue);
  actualvalue=minvalue+actualvalue;
  /*if (decimalvalue == 0)*/
    UpdateShow(actualvalue);
}
