/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "YNShell.h"
#include "GUI_support.h"

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>         /* Athena ShellWidget */
#include <X11/Xaw/Form.h>      /* Athena Form Widget */
#include <X11/Xaw/Command.h>   /* Athena Command */

/* BEGIN LOCAL DEFINE */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void CreateYN(void);
static void ShellPosition(unsigned int position, Widget shell, Widget wpos);
static void PopdownYN(Widget w,XtPointer client_data,XtPointer call_data);
static void Done(Widget w,XtPointer client_data,XtPointer call_data);
static void ActionYes(Widget w,XEvent *event,String *params,
		      Cardinal *num_params);
static void ActionNo(Widget w,XEvent *event,String *params,
		     Cardinal *num_params);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget pshell_YN,YN_label,YN_cancel,YN_done;
static int CreatedYN=False;
/* for execute the done function get from PopupYN */
static void (*exec_function)(void);
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

static void CreateYN(void)
     /*
     ** Create the widgets for the YNShell
     */
{
  static Widget form_YN,dummy;
  XtAccelerators Accelerators; /* to parse the accelerators table */

  XtActionsRec AllActions[] = { /* action to add */
    {"ActionYes",ActionYes},
    {"ActionNo",ActionNo}
  };

  pshell_YN = XtVaCreatePopupShell(
	   "xmasYN",
	   transientShellWidgetClass,
	   topLevel,NULL);
	   form_YN = XtVaCreateManagedWidget(
              "form_YN",
	      formWidgetClass,
	      pshell_YN,NULL);
	   dummy = XtVaCreateManagedWidget(
	      "YN_dummy",
	      commandWidgetClass,
	      form_YN,
	      XtNlabel,"   ",NULL);
	   YN_label = XtVaCreateManagedWidget(
	      "YN_label",
	      labelWidgetClass,
	      form_YN,NULL);
	   YN_done = XtVaCreateManagedWidget(
	      "YN_done",
	      commandWidgetClass,
	      form_YN,NULL);
	   YN_cancel = XtVaCreateManagedWidget(
	      "YN_cancel",
	      commandWidgetClass,
	      form_YN,NULL);

   /* add actions */
   XtAppAddActions(app_context, AllActions, XtNumber(AllActions));
   /*
   ** YNShell Callback
   */
   XtAddCallback(YN_done,XtNcallback,Done,(XtPointer)0);
   XtAddCallback(YN_cancel,XtNcallback,PopdownYN,(XtPointer)0);

   /* to install accelerators */
   Accelerators = XtParseAcceleratorTable /* Yes widget */
   ("#override\n\
       <KeyPress>y: ActionYes()\n\
       <KeyPress>Return: ActionYes()");
   XtVaSetValues(YN_done,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(form_YN,YN_done);
   Accelerators = XtParseAcceleratorTable /* No widget */
   ("#override\n\
       <KeyPress>n: ActionNo()");
   XtVaSetValues(YN_cancel,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(form_YN,YN_cancel);

   CreatedYN=True;
}

void PopupY(String label, unsigned int positionCode, Widget w)
     /*
     ** Popup the shell and set the label into YN_label
     ** do only a yes answer
     */
{
  if (label == NULL)
    ErrorMessage(YNLabelError,NULL,NULL,0);
  else {
    if(!CreatedYN)
      CreateYN();
    /*Calculate_relative_shell_position_and_set(pshell_YN);*/
    ShellPosition(positionCode,pshell_YN,w);
    XtPopup(pshell_YN,XtGrabNone); /* get the focus */
    XtVaSetValues(YN_label,XtNlabel,label,NULL); /* set the label */
    XtSetSensitive(YN_cancel,False);
    exec_function=NULL;
  }
}

void PopupYN(void (*funzione)(void),String label, unsigned int positionCode,
	     Widget w)
     /*
     ** Popup the shell, set the label into YN_label and get the function
     ** to execute in response to the press of done widget
     */
{
  if (funzione == NULL)
    ErrorMessage(YNFunctionError,NULL,NULL,0);
  else if (label == NULL)
    ErrorMessage(YNLabelError,NULL,NULL,0);
  else {
    if(!CreatedYN)
      CreateYN();
    ShellPosition(positionCode,pshell_YN,w);
    XtPopup(pshell_YN,XtGrabNone); /* get the focus */
    exec_function=funzione; /* function to execute for YN_done widget */
    XtVaSetValues(YN_label,XtNlabel,label,NULL); /* set the label */
    XtSetSensitive(YN_cancel,True);
  }
}

static void ShellPosition(unsigned int position, Widget shell, Widget wpos)
{
  switch (position) {
  case OnMousePosition: Shell_position_generic(wpos,pshell_YN); break;
  case AbsoluteApplicationMiddle: ;
  default: Calculate_relative_shell_position_and_set(pshell_YN);
  }
}

static void PopdownYN(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the YNShell
     */
{
  XtPopdown(pshell_YN);
}

static void Done(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the YNShell and execute the function associated
     */
{
  if (exec_function != NULL)
    exec_function(); /* execute the function for the YN_done job */
  XtCallCallbacks(YN_cancel,XtNcallback,0);
}

static void ActionYes(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** shortcut for Yes widget with keypressed y
     */
{
  XtCallCallbacks(w,XtNcallback,0);
}

static void ActionNo(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** shortcut for No widget with keypressed n
     */
{
  XtCallCallbacks(w,XtNcallback,0);
}

