/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

#ifndef CACHE_H
#define CACHE_H

#define WriteThrough 0 /* Copy Through */
#define WriteBack    1 /* Copy Back */

#define RANDOMode  3 /* random */
#define FIFOMode   2 /* First Input First Output */
#define LruMode    1 /* Least Recently Used */
#define LuMode     0 /* Least Used */

/*
** Control the state of the cache
*/
void inizializza_cache(void);
void invalidcache(void);
void flushcache(void);
void reset_cache_size(void);
unsigned int getBitMemoryCache(void);
int setBitCache(unsigned int newbitcache);
char *getStringWritemode(void);
unsigned int getModoScrittura(void);
unsigned int setModoScrittura(unsigned int newModoScrittura);
char *getStringReplacement(void);
unsigned int getReplaceMode(void);
unsigned int setReplaceMode(unsigned int newReplaceMode);

/*
** Replacement of an entry in cache
*/
unsigned int replace(unsigned int startentry, unsigned int endentry);
void Replacement(unsigned int AddressCache, unsigned int startentry,
		 unsigned int endentry);

/*
** Read/Write in cache
*/
void cachewrite(unsigned int AddressMem, unsigned int AddressCache,
		int MemValue);
int cacheread(unsigned int AddressMem,unsigned int AddressCache);
int cacheReadNoStat(unsigned int AddressMem, unsigned int AddressCache,
		    int *addtoMiss);
int cacheWriteNoStat(unsigned int AddressMem, unsigned int AddressCache,
		     int MemValue);
/*
** Clear the statistics of the cache
*/
void reset_statistic(void);
void statistic(unsigned int *miss, unsigned int *total);

/*
** Get/Set field of an entry in cache
*/
int data(unsigned int Entry);
int setdata(unsigned int Entry, int value);
unsigned int tag(unsigned int Entry);
int settag(unsigned int Entry, unsigned int value);
unsigned int valid(unsigned int Entry);
int setvalid(unsigned int Entry, unsigned int value);
unsigned int dirty(unsigned int Entry);
int setdirty(unsigned int Entry, unsigned int value);
unsigned int demand(unsigned int Entry);
int setdemand(unsigned int Entry, unsigned int value);
unsigned int getKeep(unsigned int Entry);
int setKeep(unsigned int Entry, unsigned int value);

/*
** To choose which function to use to do cache read/write
** example: to use orn ot the keep flag that need particular code!
*/
void ActiveCacheKeepMode(void);
void ActiveCacheNormalMode(void);
unsigned int getCacheAlphaNumericStatus(void);
void setCacheAlphaNumericStatus(unsigned int status);

/*
** Save the cache status
*/
int SaveCacheStatus(char *XmasDir);
#endif
