/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>

/*
** Intrinsics Definitions (only for include files)
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules
*/
#include "general_support.h"
#include "opthandler.h"
#include "ErrorMessage.h"
#include "errormessage.h"
#include "Edit.h"
#include "GUI_support.h"
#include "loader.h"
#include "policy.h"
#include "cache.h"
#include "memory.h"
#include "config.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define file_conf "xmas.conf"
#define XmasDirName ".Xmas"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

int versioncheck(char **envp)
     /*
     ** Check the version and author of the Xmas configuration file with the
     ** program value and update the file if needed
     */
{
  char XmasDir[XmasDirLength];
  char RetValue[MediumBufferLength];
  char ver[32];
  char *verfile,*verstring;
  int nverfile,nverstring;
  int i;

  XmasDirStatus(XmasDir,sizeof(XmasDir));
  strcat(XmasDir,"/");
  strcat(XmasDir,file_conf);
  if (getOptionFromName(file_conf,XmasConfVersion,RetValue,sizeof(RetValue)) == False) {
    writexmasconf(XmasDir);
    return(False);
  }
  sscanf(RetValue,"%16s",ver);
  verfile=ver;
  verstring=VersionString;
  for (i=0; i<4; i++) {
    if (verfile != NULL) {
      if (sscanf(verfile,"%d",&nverfile) != 1)
	nverfile=-1; /* the number doesn't exist */
      if ((verfile=strchr(verfile,'.')) != NULL)
	verfile++; /* skip the '.' */
    }
    if (verstring != NULL) {
      if (sscanf(verstring,"%d",&nverstring) != 1)
	nverstring=-1; /* the number doesn't exist */
      if ((verstring=strchr(verstring,'.')) != NULL)
	verstring++; /* skip the '.' */
    }
    if (nverfile < nverstring) { /* 0.4 --> 0.5 */
      ErrorMessage(UpdateFromOldVersion,VersionMessage,ver,0);
      writexmasconf(XmasDir);
      break;
    }
    if (nverfile > nverstring) { /* using 0.4 on 0.5 */
      ErrorMessage(UsingOldVersionOnRecentVersion,VersionString,ver,0);
      break;
    }
  }
  return(True);
}

int writexmasconf(char *path_file)
     /*
     ** Write the configuration file of Xmas: machine type, author and version
     */
{
  FILE *fp;
  char systemstring[MediumBufferLength];

  sprintf(systemstring,"uname -a > %s",path_file);
  system(systemstring);
  if ((fp=fopen(path_file,"a")) != NULL) {
    fprintf(fp,"%s\n%s\n",AuthorMessage,VersionMessage);
    fclose(fp);
    return(True);
  } 
  else
    return(False);
}

int birthday(void)
     /*
     ** Check if the day is my birthday
     */
{
  time_t t;
  char data_completa[ShortBufferLength],dotw[SmallBufferLength],month[SmallBufferLength],dom[SmallBufferLength];

  time(&t);
  sprintf(data_completa,"%s",ctime(&t));
  sscanf(data_completa,"%s %s %s",dotw,month,dom);
  if ((strcmp(month,"Nov")==0) && (strcmp(dom,"13")==0))
    return(True);
  return(False);
}

void DateTime(char *Date, char *Orario)
     /*
     ** Get the current date and time
     */
{
  time_t t;
  struct tm tt;
  char *p;
  char p1[SmallBufferLength],p2[SmallBufferLength],p3[SmallBufferLength],year[SmallBufferLength];

  time(&t);
  memcpy(&tt,gmtime(&t),sizeof(tt));
  p=ctime(&t);
  sscanf(p,"%s %s %s %s %s", p1, p2, p3, Orario, year);
  sprintf(Date, "%s-%.2d-%.2d", year, tt.tm_mon+1, tt.tm_mday);
  /* tt.tm_mon+1 because tt.tm_mon is always back of one at the actual month */
}

void SaveConfiguration(void)
     /*
     ** Check if there is the XmasDir and then call the saver functions
     */
{
  char XmasDir[XmasDirLength];

  if (XmasDirStatus(XmasDir,sizeof(XmasDir)) == True) {
    SaveEditStatus(XmasDir);
    SaveCacheStatus(XmasDir);
    SaveRegistersStatus(XmasDir);
    SavePoliticaStatus(XmasDir);
    SaveMemoryStatus(XmasDir);
    SaveBreakpointsStatus(XmasDir);
    SaveExecutorValues(XmasDir);
  }
}

int XmasDirStatus(char *dir, unsigned int length)
     /*
     ** Return True if the directory XmasDir exist or it is created,
     ** else return False to mean that is impossible to create the directory
     ** NB: dir is fullfill with the directory of Xmas if it exists
     **     ex: /home/users/.Xmas  ( not with final / )
     */
{
  char *home;
  char XmasDir[XmasDirLength];
  mode_t mode=0700; /* access mode: drwx------ */

  if ((home=getenv("HOME")) != NULL) {
    if (((strlen(home)+strlen(XmasDirName)+2) < length) && (length > 0)) {
      sprintf(XmasDir,"%s/%s",home,XmasDirName);
      if (access(XmasDir,F_OK) != 0)
	if (mkdir(XmasDir,mode) != 0)
	return(False);
      strncpy(dir,XmasDir,strlen(XmasDir)+1); /* +1 => copy \0 too */
      return(True);
    }
    else
      return(False);
  }
  return(False);
}


