/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <signal.h>

/*
** Intrinsics Definitions (only for include files)
*/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/*
** include of application modules and images
*/
#include "general_support.h"
#include "opthandler.h"
#include "pathexec.h"
#include "BrPts.h"
#include "Edit.h"
#include "loader.h"
#include "loaderP.h"
#include "policy.h"
#include "cache.h"
#include "memory.h" /* used by unloader */
#include "esp.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define String2Number 0
#define Number2String 1
#define NumberofOptions 12
#define NumberofMeta 1
#define MaxBreakLine 8

/*
** Decoder field instruction RISC-like, used by unloader and executor
*/
/*
** Speed up: +11%
*/
#define decode_rs_field(istr) ((istr & rs_mask) >> 21)
#define decode_rt_field(istr) ((istr & rt_mask) >> 16)
#define decode_rd_field(istr) ((istr & rd_mask) >> 11)
#define decode_funct_field(istr) (istr & funct_mask)
#define decode_small_zero_field(istr) ((istr & small_zero_mask) >> 6)
#define decode_imm_offset_field(istr) (istr & imm_mask)
#define decode_div_mult_zero_field(istr) ((istr & div_mult_zero_mask) >> 6)
#define decode_mfhi_mflo_zero_field(istr) ((istr & mfhi_mflo_zero_mask) >> 16)
#define decode_mthi_mtlo_zero_field(istr) ((istr & mthi_mtlo_zero_mask) >> 6)
#define decode_sa_field(istr) decode_small_zero_field(istr)
#define decode_j_field(istr) (istr & 0x3ffffff);
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOBAL VARIABLES */
extern unsigned int EditorAL; /* from menuOPTION.c */
extern int Bell;
/* END IMPORTED GLOBAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static unsigned int OutofTimeLimit=2; /* default TimeLimit bound */
static unsigned int MinOutofTimeLimit=1;
static unsigned int MaxOutofTimeLimit=4;

static unsigned int IstrLimit=1000; /* default IstrLimit bound */
static unsigned int MinIstrLimit=1000;
static unsigned int MaxIstrLimit=1e5;

static unsigned int MaxShiftAmount;/* max shift amount for left/right rotate */
static unsigned int MinShiftAmount=0;/*max shift amount for left/right rotate*/

static int execBp=False;
static unsigned int executor_mode=0; /*=Tl;*/
static int Bell=False; /* for VCPU hard limit time */

static char riga[IstrStringCar];   /* opcode string */
static int registri[MaxReg+2]={0}; /* VCPU registers + hi & lo */
/* 
** MaxReg+0: HI
** MaxReg+1: LO
*/

struct executor_mode_list {
  int (*test)(unsigned int);
  struct executor_mode_list *next;
};
struct executor_mode_list *mode_list=NULL;

struct BP { /* structure for breakpoint */
  int address;
  int reg;
  char testkind;
  int valuetest;
};
static struct BP breakline[MaxBreakLine];
static int MaxBPUsed=0; /* type int because the -Wconversion do not warning */

/* options array */
static char *orderedoptions[NumberofOptions]={".Direct",".Fifo",".Fully",
					 ".KDirect",".LruMode",".LuMode",
					 ".NWay",".Pc",".Random",".text",
					 ".WriteBack",".WriteThrough"};

/* metaistr array */
static char *metaistr[NumberofMeta]={"BP\0"};

/* istructions array */
static struct scelta_opcode orderedop[]={
  {"add",add,functadd,0},
  {"addi",addi,0,0},
  {"and",and,functand,0},
  {"andi",andi,0,0},
  {"beq",beq,0,0},
  {"bne",bne,0,0},
  {"div",divide,functdiv,0},
  {"halt",halt,0,0},
  {"lui",lui,0,0},
  {"lw",lw,0,0},
  {"j",jump,0,0},
  {"jr",jr,functjr,0},
  {"mfhi",mfhi,functmfhi,0},
  {"mflo",mflo,functmflo,0},
  {"mthi",mthi,functmthi,0},
  {"mtlo",mtlo,functmtlo,0},
  {"mult",mult,functmult,0},
  {"nop",nop,0,0},
  {"or",or,functor,0},
  {"ori",ori,0,0},
  {"sll",sll,functsll,0},
  {"sllv",sllv,functsllv,0},
  {"slt",slt,functslt,0},
  {"slti",slti,0,0},
  {"srl",srl,functsrl,0},
  {"srlv",srlv,functsrlv,0},
  {"sub",sub,functsub,0},
  {"sw",sw,0,0},
  {"xor",xor,functxor,0},
  {"xori",xori,0,0},
  {NULL,0,0,0},
};
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void inizializza_loader(void)
{
  LoadRegistersValue();
  /*
  ** set to -1 all breakpoints => invalidation
  ** TimeLimit enabled
  */
  reset_breakline();
  LoadBreakpointsValue();
  LoadExecutorValues();
  toggleTL();
  toggleIT();
  MaxShiftAmount=sizeof(int)*8; /* hardware dependent */
}

static int check_condition(unsigned int ConditionIndex)
     /*
     ** Check the condition of a breakpoint and return a boolean
     */
{
  int valuereg;

  if (breakline[ConditionIndex].reg != -1) {
    valuereg=registri[breakline[ConditionIndex].reg];
    switch (breakline[ConditionIndex].testkind) {
    case '<': return(valuereg < breakline[ConditionIndex].valuetest);
      break;
    case '>': return(valuereg > breakline[ConditionIndex].valuetest);
      break;
    case '=': return(valuereg == breakline[ConditionIndex].valuetest); 
      break;
    default: return(-1);
      break;
    }
  }
  else
    return(-1);
}

static int find_set(char *string)
     /*
     ** Ogni parola DEVE ESSERE strettamente crescente con la precedente
     ** NB: non mischiare un pezzo della riga r con il restante della riga r+1
     ** ESEMPIO: ...".WriteBack\0",".WroteThrough\0",...
     ** cercando .WriteThrough trova .WroteThrough !!!!!
     */
{
  int row=0;
  unsigned int stringLength,orderedoptionsLength,Length;

  /*MaxR=(int)strlen(string);
  while((column < MaxR) && (row < NumberofOptions))
    if (orderedoptions[row][column] == string[column])
      column++;
    else {
      row++; column=0;
    }
  if (column!=MaxR)
  return(UnKnownIstr);*/
  for (row=0; row < NumberofOptions; row++) {
    stringLength=strlen(string);
    orderedoptionsLength=strlen(orderedoptions[row]);
    if (stringLength < orderedoptionsLength)
      Length=orderedoptionsLength;
    else
      Length=stringLength;
    if (strncmp(string,orderedoptions[row],Length) == 0)
      return(row);
  }
  return(UnKnownIstr);
}

static int find_opcode(char *nomeistr,unsigned int *opcode, unsigned int *ff)
     /*
     ** see find_set
     */
{
  unsigned int row;
  unsigned int nomeistrLength,OpStringLength,Length;

  for (row=0; row < get_num_istruction(); row++) {
    nomeistrLength=strlen(nomeistr);
    OpStringLength=strlen(orderedop[row].OpString);
    if (nomeistrLength < OpStringLength)
      Length=OpStringLength;
    else
      Length=nomeistrLength;
    if (strncmp(nomeistr,orderedop[row].OpString,Length) == 0) {
      *opcode=orderedop[row].Valore;
      *ff=orderedop[row].FunctField;
      return((int)orderedop[row].Valore);
    }
  }
  return(UnKnownIstr);
}

/*
** BEGIN LOADER
*/

unsigned int loader(char *nome_file,unsigned int *PC, char *istr_string,
		    int *line_counter)
     /*
     ** Load the program and set the options
     */
{
  unsigned int i,valore=0,offset=0,originalPC;
  int text=False,cont=0;
  unsigned int ind;
  FILE *fp;
  unsigned int TrTipoPolitica,TrLoaderWay,TrReplaceMode,TrModoScrittura,TrPC;
  int *TrMem,*TrBP;
  unsigned int error=False;
  char *p,BPstring[10];

  if ((fp=fopen(nome_file,"r")) == NULL)
    return(LoaderFileError);
  /* begin open transaction for options */
  /* originalPC: use this if there is an error */
  TrMem=(int *)malloc(sizeof(int)*getMaxAddress());
  TrBP=(int *)malloc(sizeof(int)*MaxBreakLine);
  TrTipoPolitica=getTipoPolitica();
  TrLoaderWay=getWay();
  TrReplaceMode=getReplaceMode();
  TrModoScrittura=getModoScrittura();
  originalPC=TrPC=*PC;
  /* end open transaction */
  while ((text == False) && (fscanf(fp,"%s",riga)!=EOF) && (error == False)) { /* read and set the state of the cache and policy */
    switch (find_set(riga)) {
    case 0: TrTipoPolitica=Direct; break;
    case 1: TrTipoPolitica=Fully; break;
    case 2: TrReplaceMode=LruMode; break;
    case 3: TrReplaceMode=LuMode; break;
    case 4: TrTipoPolitica=Nway;
      if (fscanf(fp,"%d",(int *)&TrLoaderWay) != False) {
	if (!((TrLoaderWay >= 1) && (TrLoaderWay <= 4) &&
	      (TrLoaderWay<=(unsigned int)esp(2,getBitMemoryCache()))))
	  error=SetNWayError;
      }
      else
	error=LoaderIncompleteNWayError;
      break;
    case 5: if(fscanf(fp,"%x",(int *)&TrPC) != False) *PC=TrPC; /* set the PC to the option value */
      else error=LoaderPCError; 
      break;
    case 6: text=True; break;
    case 7: TrModoScrittura=WriteBack; break;
    case 8: TrModoScrittura=WriteThrough; break;
    }
  }
  if ((text == False) && (error == False)) /* without .text directive and without errors */
    error=LoaderFileIncomplete;
  while ((fgets(riga,(int)sizeof(riga),fp) != NULL) && (error == False)) {
    if (riga[0]=='\n')
      continue;
    if ((*PC) >= getMaxAddress()) {
      error=LoaderOutPC;
      break; /* stop the while, thus nobody overwrite error */
    }
    p=riga;
    sscanf(riga,"%s",BPstring);
    if (strcmp(BPstring,metaistr[NumberofMeta-1]) == 0) {
      if (offset < MaxBreakLine) {
	TrBP[offset]=(int)*PC; /* this is a breakpoint! */
	offset++; /* record the last BP used */
      }
      p+=strlen(BPstring)+1;
    }
    cont++;
    error=String2Istr(p,&valore);
    strcpy(istr_string,riga); /* copy for an error */
    TrMem[*PC]=valore;
    (*PC)++;
  }
  /* begin close transaction */
  if(error == False) {
    setTipoPolitica(TrTipoPolitica);
    setReplaceMode(TrReplaceMode);
    setModoScrittura(TrModoScrittura);
    setWay(TrLoaderWay);
    for (ind=TrPC; ind < (*PC); ind++)
      polwrite(ind,TrMem[ind]); /* write into memory */
    reset_breakline();
    for (i=0; i<offset; i++)
      breakline[i].address=TrBP[i]; /* update BP */
    UpdateBP(); /* update the shell for view breakpoints */
    MaxBPUsed=(int)offset;
    if (EditorAL==True) {
      char cwd[1024];

      getcwd(cwd,sizeof(cwd));
      EditorGetBufferFromFile(nome_file,nome_file,cwd,(*PC)-cont);
    }
    *PC=TrPC; /* reset the PC to begin */
    error=LoaderExactCode;
  }
  else
    *PC=originalPC;
  /* end close transaction */
  free(TrMem);
  free(TrBP);
  fclose(fp);
  *line_counter=cont;
  return(error);
}

unsigned int String2Istr(char *AlfaNumIstruction,
			 unsigned int *NumericIstruction)
     /*
     ** Convert an istruction from string format to number format
     */
{
  unsigned int error=False;
  unsigned int NumIstr=0,functfield=0;
  char opcode[10];

  readsubstring(&AlfaNumIstruction,opcode);
  switch(find_opcode(opcode,&NumIstr,&functfield)) {
  case add : error=encoder_add(&NumIstr,functfield,&AlfaNumIstruction); break;
  case lui: error=encoder_lui(&NumIstr,&AlfaNumIstruction); break;
  case slti:
  case xori:
  case ori:
  case andi:
  case addi:
  case lw: 
  case sw:
  case beq:
  case bne: error=encoder_addi_lw_sw_beq_bne(&NumIstr,&AlfaNumIstruction);break;
  case mflo: error=encoder_mflo_mfhi(&NumIstr,functfield,&AlfaNumIstruction);
    break;
  case nop: 
  case halt: error=encoder_nop_halt(&NumIstr); break;
  case jump: error=encoder_jump(&NumIstr,&AlfaNumIstruction); break;
  case mtlo: error=encoder_mtlo_mthi(&NumIstr,functfield,&AlfaNumIstruction);
    break;
  case UnKnownIstr: error=LoaderIstrError; break;
  default: error=LoaderUnknownError; break;
  }
  *NumericIstruction=NumIstr;
  return(error);
}

static int readsubstring(char **inputstring,char *outputstring)
     /*
     ** Read a substring to a space or an \n
     */
{
  sscanf(*inputstring,"%s",outputstring);
  *inputstring+=strlen(outputstring)+1;
  return(True);
}

static unsigned int encoder_jump(unsigned int *NumericIstruction,
				 char **AlfaNumIstrRef)
     /*
     ** encode j (jump) istruction
     */
{
  int target;

  if ((target=encode_imm_offset_field(AlfaNumIstrRef)) == INT_MAX)
    return(LoaderExceededImmField);
  *NumericIstruction = (*NumericIstruction << 26) | ((unsigned int)target & 0x3ffffff);
  return(False);
}

static unsigned int encoder_addi_lw_sw_beq_bne(unsigned int *NumericIstruction,
					       char **AlfaNumIstrRef)
     /*
     ** Encode the istructions addi,lw,sw,beq,bne
     */
{
  unsigned int rs,rt;
  int imm;

  if ((rs=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRS);
  if ((rt=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRT);
  if ((imm=encode_imm_offset_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
      return(LoaderWhithoutIMM);
  if ((rs >= MaxReg) || (rt >= MaxReg))
    return(LoaderExceededMaxReg);
  if ((imm < -32768) || (imm > 32767))
    return(LoaderExceededImmField);
  *NumericIstruction = (*NumericIstruction << 26) | (rs << 21) | (rt << 16) | ((unsigned int)imm & 0xffff);
  return(False);
}

static unsigned int encoder_nop_halt(unsigned int *NumericIstruction)
{
  /*
  ** Encode the nop or halt instruction
  */
  *NumericIstruction = *NumericIstruction << 26;
  return(False);
}

static unsigned int encoder_mtlo_mthi(unsigned int *NumericIstruction,
				      unsigned int ff, char **AlfaNumIstrRef)
     /*
     ** Encode the istructions mtlo,mthi
     */
{
  unsigned int rd;

  if ((rd=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRD);
  /* validity number */
  if (rd >= MaxReg)
    return(LoaderExceededMaxReg);
  *NumericIstruction=(*NumericIstruction << 26) | (rd << 21);
  switch (ff) {
  case functmthi: *NumericIstruction = *NumericIstruction | functmthi; break;
  case functmtlo: *NumericIstruction = *NumericIstruction | functmtlo; break;
  default: return(LoaderUnknownError);
  }
  return(False);
}

static unsigned int encoder_mflo_mfhi(unsigned int *NumericIstruction,
				      unsigned int ff, char **AlfaNumIstrRef)
     /*
     ** Encode the istructions mflo,mfhi
     */
{
  unsigned int rd;

  if ((rd=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRD);
  /* validity number */
  if (rd >= MaxReg)
    return(LoaderExceededMaxReg);
  *NumericIstruction = (*NumericIstruction << 26) | (rd << 11);
  switch (ff) {
  case functmfhi: *NumericIstruction = *NumericIstruction | functmfhi; break;
  case functmflo: *NumericIstruction = *NumericIstruction | functmflo; break;
  default: return(LoaderUnknownError);
  }
  return(False);
}

static unsigned int encoder_add(unsigned int *NumericIstruction,
				unsigned int ff, char **AlfaNumIstrRef)
     /*
     ** Encode the istructions div,mult,add,slt
     */ 
{
  unsigned int rs,rt;

  if ((rs=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRS);
  if (rs >= MaxReg)
    return(LoaderExceededMaxReg);
  *NumericIstruction = (*NumericIstruction << 26) | (rs << 21);
  switch (ff) {
  case functjr: *NumericIstruction = *NumericIstruction | functjr;
    return (False); break;
  }
  if ((rt=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRT);
  if (rt >= MaxReg)
    return(LoaderExceededMaxReg);
  *NumericIstruction = *NumericIstruction | (rt << 16);  
  switch (ff) {
  case functdiv:  *NumericIstruction = *NumericIstruction | functdiv; break;
  case functmult: *NumericIstruction = *NumericIstruction | functmult; break;
  case functadd:  return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functadd)); break;
  case functsub:  return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functsub)); break;
  case functand:  return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functand)); break;
  case functor:   return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functor)); break;
  case functslt:  return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functslt)); break;
  case functxor:  return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functxor)); break;
  case functsll:  return(encoder_add_sa_field(NumericIstruction,AlfaNumIstrRef,functsll)); break;
  case functsrl:  return(encoder_add_sa_field(NumericIstruction,AlfaNumIstrRef,functsrl)); break;
  case functsllv: return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functsllv)); break;
  case functsrlv: return(encoder_add_rd_field(NumericIstruction,AlfaNumIstrRef,functsrlv)); break;
  default: return(LoaderUnknownError);
  }
  return(False);
}

static unsigned int encoder_add_rd_field(unsigned int *NumericIstruction,
					 char **AlfaNumIstrRef,
					 unsigned int functfield)
     /*
     ** encode the rd field for add-like istruction with function field
     */
{
  unsigned int rs;

  rs=encode_register_field(AlfaNumIstrRef);
  if (rs == INT_MAX) /* do not exist */
    return(LoaderWhithoutRD);
  if (rs >= MaxReg) /* validity number */
    return(LoaderExceededMaxReg);
  *NumericIstruction = *NumericIstruction | (rs << 11) | functfield;
  return(False);
}

static unsigned int encoder_add_sa_field(unsigned int *NumericIstruction,
					 char **AlfaNumIstrRef,
					 unsigned int functfield)
     /*
     ** encode the shift amount field
     */
{
  unsigned int shiftamount;

  shiftamount=encode_imm_offset_field(AlfaNumIstrRef); /* overload!!! */
  if (shiftamount == INT_MAX) /* do not exist */
    return(LoaderWhithoutRD);
  if (shiftamount >= MaxShiftAmount) /* validity number */
    return(LoaderExceededMaxShiftAmount);
  if (shiftamount == MinShiftAmount)
    return(LoaderExceededMinShiftAmount);
  *NumericIstruction = *NumericIstruction | (shiftamount << 6) | functfield;
  return(False);
}

static unsigned int encoder_lui(unsigned int *NumericIstruction,
				char **AlfaNumIstrRef)
     /*
     ** Encode the istruction lui
     */
{
  unsigned int rt;
  int imm;

  if ((rt=encode_register_field(AlfaNumIstrRef)) == INT_MAX) /* do not exist */
    return(LoaderWhithoutRT);
  if ((imm=encode_imm_offset_field(AlfaNumIstrRef)) == INT_MAX)/*do not exist*/
    return(LoaderWhithoutIMM);
  if (rt >= MaxReg)
    return(LoaderExceededMaxReg);
  if ((imm < -65536) || (imm > 65535))
    return(LoaderExceededImmField);
  *NumericIstruction = (*NumericIstruction << 26) | (rt << 16) | ((unsigned int)imm & 0xffff);
  return(False);
}

static unsigned int encode_register_field(char **AlfaNumIstrRef)
     /*
     ** Read the number the register and return the number
     */
{
  char reg_rs[RegString]="\0";
  unsigned int reg_rs_hex;
  int returned_value;

  readsubstring(AlfaNumIstrRef,reg_rs);
  if (*reg_rs != '$') /* check the register identificator $, that must be on */
    return(INT_MAX);
  returned_value=sscanf(reg_rs+1,"%x",(int *)&reg_rs_hex);
  /* 
  ** Zero indicates that, while there was input available, no conversions 
  ** were  assigned. The value  EOF  is  returned if an input failure occurs
  ** before any conversion such as an end-of-file occurs
  */
  if (returned_value <= 0)
    reg_rs_hex=INT_MAX;
  return(reg_rs_hex); /* reg_rs+1 == &(reg_rs[1]) */
}

static int encode_imm_offset_field(char **AlfaNumIstrRef)
     /*
     ** Read the immediate/offset and return the number
     */
{
  char imm_offset_string[RegString];
  int imm_offset_hex;

  /*
  ** Read an hex value not a 16bit hex value, it may be greater than 16 bit
  ** the function caller must and (&) the value to bound the value
  */
  readsubstring(AlfaNumIstrRef,imm_offset_string);
  if (sscanf(imm_offset_string,"%x",&imm_offset_hex) <= 0)
    return(INT_MAX);
  return(imm_offset_hex);
}

/*
** END LOADER
*/

/*
** PAY ATTENTION !!!
** UNLOADER CODE UNDER THIS LINE
** DO NOT CUNFUSE
*/

int unloader(unsigned int ProgramCounter,char *AlfaNumIstruction)
     /*
     ** Write into string the istruction aimed by PC
     */
{
  int returned_value;
  unsigned int NumericIstruction;
  unsigned int Entry;

  Entry=getpolpos(ProgramCounter,False);
  if ((valid(Entry) == 1) && (tag(Entry) == ProgramCounter)) /* CACHE check */
    NumericIstruction=(unsigned int)data(Entry);
  else /* into memory */
    /*
    ** read the istruction directly from memory!
    */
    NumericIstruction=(unsigned int)memread(ProgramCounter);
  /* decode istr, Number2String */
  returned_value=Istr2String(NumericIstruction,AlfaNumIstruction);
  if ((dicotomic_find_breakline(ProgramCounter,0,MaxBPUsed) != -1) &&
      (getBP() == True)) /* only if active! */
    /* add the string "Break Point" if actived */
    strcat(AlfaNumIstruction," Break Point");
  return(returned_value);
}

int Istr2String(unsigned int NumericIstruction, char *AlfaNumIstruction)
     /*
     ** Convert an istruction from number format to string format
     */
{
  unsigned int opcode_field;
  int retValue;

  /* 0xFC000000 <==> 11111100...000 */
  opcode_field=(NumericIstruction & 0xFC000000) >> 26;
  /* write the numeric format */
  sprintf(AlfaNumIstruction,"%x",(int)NumericIstruction);
  switch (opcode_field)
    {
    case slti:
    case xori:
    case ori:
    case andi:
    case addi:
    case lw:
    case sw:
    case beq:
    case bne: retValue=decoder_bne(opcode_field,NumericIstruction,AlfaNumIstruction);
      break;
    case mflo: retValue=decoder_mflo_mfhi(NumericIstruction,AlfaNumIstruction);
      break;
    case mtlo: retValue=decoder_mtlo_mthi(NumericIstruction,AlfaNumIstruction);
      break;
    case halt: retValue=decoder_halt(AlfaNumIstruction); break;
    case add: retValue=decoder_add(NumericIstruction,AlfaNumIstruction); break;
    case nop: retValue=decoder_nop(AlfaNumIstruction); break;
    case lui: retValue=decoder_lui(NumericIstruction,AlfaNumIstruction); break;
    case jump: retValue=decoder_jump(NumericIstruction,AlfaNumIstruction); break;
    default:
      retValue=UnloaderIstrError;
      break;
    }
  return(retValue);
}

static int decoder_jump(unsigned int NumericIstruction, char *AlfaNumIstruction)
     /*
     ** decode j (jump) istruction
     */
{
  unsigned int jumplength;

  jumplength=decode_j_field(NumericIstruction);
  sprintf(AlfaNumIstruction,"J %x",jumplength);
  return(UnloaderExactDecode);
}

static int decoder_bne(unsigned int opcode_field,
		       unsigned int NumericIstruction,
		       char *AlfaNumIstruction)
     /*
     ** Decode the istructions bne-like
     ** addi,sw,lw,beq,bne
     ** and write the relative string
     */
{
  unsigned int rs,rt;
  unsigned int imm;

  rs=decode_rs_field(NumericIstruction);
  rt=decode_rt_field(NumericIstruction);
  imm=decode_imm_offset_field(NumericIstruction);
  if ((rs < MaxReg) && (rt < MaxReg)) {
    switch(opcode_field) /* choose the string format */
      {
      case addi: sprintf(AlfaNumIstruction,"ADDI $%x $%x %x",rs,rt,imm); break;
      case andi: sprintf(AlfaNumIstruction,"ANDI $%x $%x %x",rs,rt,imm); break;
      case ori:  sprintf(AlfaNumIstruction,"ORI $%x $%x %x" ,rs,rt,imm); break;
      case xori: sprintf(AlfaNumIstruction,"XORI $%x $%x %x",rs,rt,imm); break;
      case sw:   sprintf(AlfaNumIstruction,"SW $%x $%x %x"  ,rs,rt,imm); break;
      case lw:   sprintf(AlfaNumIstruction,"LW $%x $%x %x"  ,rs,rt,imm); break;
      case beq:  sprintf(AlfaNumIstruction,"BEQ $%x $%x %x" ,rs,rt,imm); break;
      case bne:  sprintf(AlfaNumIstruction,"BNE $%x $%x %x" ,rs,rt,imm); break;
      case slti: sprintf(AlfaNumIstruction,"SLTI $%x $%x %x",rs,rt,imm); break;
      default: return(UnloaderIstrError);
      }
  }
  else
    return(UnloaderExceededMaxReg);
  return(UnloaderExactDecode);
}

static int decoder_mtlo_mthi(unsigned int NumericIstruction,char *AlfaNumIstruction)
     /*
     ** Decode mtlo and mthi
     ** and write the relative string
     */
{
  unsigned int rs,functfield,zerofield;

  rs=decode_rs_field(NumericIstruction);
  functfield=decode_funct_field(NumericIstruction);
  zerofield=decode_mthi_mtlo_zero_field(NumericIstruction);
  if ((zerofield == 0) && (rs < MaxReg))
    switch(functfield)
      {
      case functmthi: sprintf(AlfaNumIstruction,"MTHI $%x",rs); break;
      case functmtlo: sprintf(AlfaNumIstruction,"MTLO $%x",rs); break;
      default: return(UnloaderIstrError);
      }
  else
    return(UnloaderExceededMaxReg);
  return(UnloaderExactDecode);
}


static int decoder_mflo_mfhi(unsigned int NumericIstruction,char *AlfaNumIstruction)
     /*
     ** Decode mflo and mfhi
     ** and write the relative string
     */
{
  unsigned int rd,functfield,zerofield,small_zerofield;

  rd=decode_rd_field(NumericIstruction);
  functfield=decode_funct_field(NumericIstruction);
  zerofield=decode_mfhi_mflo_zero_field(NumericIstruction);
  small_zerofield=decode_small_zero_field(NumericIstruction);
  if ((zerofield == 0) && (small_zerofield == 0) && (rd < MaxReg))
    switch(functfield)
      {
      case functmfhi: sprintf(AlfaNumIstruction,"MFHI $%x",rd); break;
      case functmflo: sprintf(AlfaNumIstruction,"MFLO $%x",rd); break;
      default: return(UnloaderIstrError);
      }
  else
    return(UnloaderExceededMaxReg);
  return(UnloaderExactDecode);
}

static int decoder_nop(char *AlfaNumIstruction)
     /*
     ** Decode nop
     ** and write the relative string
     */
{
  sprintf(AlfaNumIstruction,"NOP");
  return(UnloaderExactDecode);
}

static int decoder_halt(char *AlfaNumIstruction)
     /*
     ** Decode halt
     ** and write the relative string
     */
{
  sprintf(AlfaNumIstruction,"HALT");
  return(UnloaderExactDecode);
}

static int decoder_add(unsigned int NumericIstruction,char *AlfaNumIstruction)
     /*
     ** Decode the istruction add-like
     ** add,slt,div,mult
     ** and write the relative string
     */
{
  unsigned int rs,rt,rd,functfield,zerofield_div_mult,sa;

  rs=decode_rs_field(NumericIstruction);
  rt=decode_rt_field(NumericIstruction);
  rd=decode_rd_field(NumericIstruction);
  functfield=decode_funct_field(NumericIstruction);
  /*
  ** zero_div_mult is needed only by functdiv and functmult but the speed up
  ** increment is only 0.25% 
  */
  zerofield_div_mult=decode_div_mult_zero_field(NumericIstruction);
  if ((rs < MaxReg) && (rt < MaxReg))
    switch (functfield)
      {
      case functadd: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"ADD $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functsub: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"SUB $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functand: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"AND $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functor: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"OR $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functslt: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"SLT $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functdiv: if (zerofield_div_mult == 0)
	sprintf(AlfaNumIstruction,"DIV $%x $%x",rs,rt);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functmult: if (zerofield_div_mult == 0)
	sprintf(AlfaNumIstruction,"MULT $%x $%x",rs,rt);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functxor: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"XOR $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functsllv: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"SLLV $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functsrlv: if (rd < MaxReg)
	sprintf(AlfaNumIstruction,"SRLV $%x $%x $%x",rs,rt,rd);
      else
	return(UnloaderExceededMaxReg);
      break;
      case functsll: sa=decode_sa_field(NumericIstruction);
	if ((sa > MinShiftAmount) && (sa < MaxShiftAmount))
	sprintf(AlfaNumIstruction,"SLL $%x $%x %x",rs,rt,sa);
      else
	return(UnloaderWrongShiftAmount);
	break;
      case functsrl: sa=decode_sa_field(NumericIstruction);
	if ((sa > MinShiftAmount) && (sa < MaxShiftAmount))
	sprintf(AlfaNumIstruction,"SRL $%x $%x %x",rs,rt,sa);
      else
	return(UnloaderWrongShiftAmount);
      break;
      case functjr: sprintf(AlfaNumIstruction,"JR $%x",rs);
	break;
      default: return(UnloaderIstrError);
      }
  else
    return(UnloaderExceededMaxReg);
  return(UnloaderExactDecode);
}

static int decoder_lui(unsigned int NumericIstruction, char *AlfaNumIstruction)
     /*
     ** Decode the lui istruction
     */
{
  unsigned int rt;
  unsigned int imm;

  rt=decode_rt_field(NumericIstruction);
  imm=decode_imm_offset_field(NumericIstruction);
  if (rt < MaxReg)
    sprintf(AlfaNumIstruction,"LUI $%x %x",rt,imm);
  else
    return(UnloaderExceededMaxReg);
  return(UnloaderExactDecode);
}

/*
** END UNLOADER
*/

/*
** PAY ATTENTION !!!
** EXECUTOR CODE UNDER THIS LINE
** DO NOT CUNFUSE
*/

static void exec_add_test_head(int (*ctl)(unsigned int))
     /*
     ** Add with high priority a function to the execution list
     */
{
  struct executor_mode_list *tmp;

  (tmp=(struct executor_mode_list *)malloc(sizeof(struct executor_mode_list)))->test=ctl;
  tmp->next=mode_list;
  mode_list=tmp;
}

static void exec_add_test_tail(int (*ctl)(unsigned int))
     /*
     ** Add with low priority a function to the execution list
     */
{
  struct executor_mode_list *tmp;

  tmp=mode_list;
  if (tmp == NULL) {
    (mode_list=(struct executor_mode_list *)malloc(sizeof(struct executor_mode_list)))->test=ctl;
    mode_list->next=NULL;
  }
  else {
    while (tmp->next != NULL)
      tmp=tmp->next;
    (tmp->next=(struct executor_mode_list *)malloc(sizeof(struct executor_mode_list)))->test=ctl;
    tmp->next->next=NULL;
  }
}

static void exec_del_test(int (*ctl)(unsigned int))
     /*
     ** Delete a function from execution list
     */
{
  mode_list=ric_delete(mode_list,ctl);
}

static struct executor_mode_list *ric_delete(struct executor_mode_list *list,
					     int (*ctl)(unsigned int))
     /*
     ** Recursive delete of a function from execution list
     */
{
  struct executor_mode_list *tmp;

  if (list == NULL) /* basic [BASE]*/
    return(NULL);
  else {            /* step  [PASSO]*/
    tmp=ric_delete(list->next,ctl);
    if (list->test == ctl) {
      free(list);
      return(tmp);
    }
    else {
      list->next=tmp;
      return(list);
    }
  }
}
/*
static void exec_show_test(void)
{
  struct executor_mode_list *tmp;

  tmp=mode_list;
  while (tmp != NULL) {
    printf("%p %p\n",tmp->test,tmp);
    tmp=tmp->next;
  }
}
*/
static int ModeTest(unsigned int PC)
     /*
     ** Run the function from execution list starting from head (high priority)
     ** to tail (low priority)
     */
{
  struct executor_mode_list *tmp;
  int retValue;

  tmp=mode_list;
  while (tmp != NULL) {
    if ((retValue=tmp->test(PC)) != False) /* tmp->test(x) : execute the function */
      return(retValue);
    tmp=tmp->next;
  }
  return(False);
}

int execrun(unsigned int *PC)
     /*
     ** Decide executor mode for esegui
     */
{
  struct executor_mode_list *tmp;
  int returned;
  unsigned int MaxAddress;

  MaxAddress=getMaxAddress(); /* speed up: +3.6% */
  tmp=mode_list;
  Bell=False;
#ifdef __linux__
  signal(SIGALRM,gestore); 
#endif
  alarm(OutofTimeLimit);
  StartToolsForExecution(); /* initialize tools for execution */
  if ((executor_mode & Fr) == 0) { /* StepByStep mode */
    if (IsActiveExecutionPath() == True)
      peCheck(*PC);
    returned=esegui(PC,MaxAddress); 
  }
  else do {
    if ((returned=ModeTest(*PC)) != False)
      break;
  } while ((returned=esegui(PC,MaxAddress)) == ExecExactExecution);
  StopToolsForExecution(); /* stop tools after execution, free any resources */
  return(returned);
}

static void StartToolsForExecution(void)
     /*
     ** Initialize the activated tools to trace the execution
     */
{
  StartPathTracer();
}

static void StopToolsForExecution(void)
     /*
     ** Stop the tools to organize the recorded information
     */
{
  StopPathTracer();
}

static void gestore (int i)
     /*
     ** Set Bell to True when the TimeLimit time is elapsed
     */
{
  /* i is the signal number */
  Bell=ExecOutofTime;
}

static int bellCheck(unsigned int i)
     /*
     ** Check the TimeLimit (Bell)
     */
{
  return(Bell);
}

static int istrCheck(unsigned int i)
{
  static unsigned int istrTotal=0;

  if (istrTotal++ >= IstrLimit) {
    istrTotal=0;
    return(ExecIstrLimit);
  }
  return(False);
}

static int bpCheck(unsigned int PC)
     /*
     ** Check if PC has breakpoint
     */
{
  static int bypass=False; /* decide if jump next test */

  int position;

  if (bypass == True) {
    bypass=False;
    return(False);
  }
  if (((position=dicotomic_find_breakline(PC,0,MaxBPUsed)) != -1) && (check_condition((unsigned int)position) != False)) {
    bypass=True;
    return(ExecBreakPoint);
  }
  return(False);
}

static int peCheck(unsigned int PC)
     /*
     ** Check if path execution must record PC
     */
{
  if (IsActiveExecutionPath()==True)
    TracePC(PC);
  return(False);
}

unsigned int esegui(unsigned int *PC,unsigned int MaxAddress)
     /*
     ** Simulate the execution of the program into memory
     ** return the new PC
     ** PS: the test on PC is did for keep the PC in memory, because the
     **     IstructionViewer show a valid istruction
     */
{
  unsigned int NumericIstr,retValue;
  unsigned int opcode_field;
  unsigned tmpPC;

  tmpPC=*PC;
  /*NumericIstr=(unsigned int)polread(tmpPC);*/ /* get the istruction to execute */
  NumericIstr=(unsigned int)polread(*PC);
  opcode_field=NumericIstr>>26;
  switch (opcode_field) {
  case slti:
  case xori:
  case andi:
  case ori:
  case addi:
  case lw:
  case sw:
  case beq:
  case bne:  retValue=execute_bne(opcode_field,NumericIstr,PC); break;
  case mflo: retValue=execute_mflo_mfhi(NumericIstr,PC); break;
  case mtlo: retValue=execute_mtlo_mthi(NumericIstr,PC); break;
  case nop:  retValue=execute_nop(); (*PC)++; break;
  case halt: retValue=execute_halt(); break;
  case add:  retValue=execute_add(NumericIstr,PC); break;
  case lui:  retValue=execute_lui(NumericIstr,PC); break;
  case jump: retValue=execute_jump(NumericIstr,PC); break;
  default: return(ExecIstrError);
  }
  if ((*PC) >= MaxAddress) { /* check the PC value */
    *PC=tmpPC;
    return(ExecOutPC); /* PC out of memory */
  }
  return(retValue);
}

void setSbS(void)
     /*
     ** Set the Step by Step executor mode
     */
{
  executor_mode &= ~Fr;
}

void setFR(void)
     /*
     ** Set the Free Run executor mode
     */
{
  executor_mode |= Fr;
}

int toggleBP(void)
     /*
     ** Enable/disable the Breakpoint executor mode
     */
{
  switch (execBp) {
  case True: exec_del_test(bpCheck); execBp=False; break;
  case False: exec_add_test_head(bpCheck); execBp=True; break;
  }
  return(execBp);
}

int getBP(void)
{
  return(execBp);
}

int toggleTL(void)
     /*
     ** Enable/disable the Time Limit executor mode
     */
{
  static int execTl=False;

  switch (execTl) {
  case True: exec_del_test(bellCheck); execTl=False; break;
  case False: exec_add_test_head(bellCheck); execTl=True; break;
  }
  return(execTl);
}

int toggleIT(void)
     /*
     ** Enable/disable the Time Limit executor mode
     */
{
  static int execIT=False;

  switch (execIT) {
  case True: exec_del_test(istrCheck); execIT=False; break;
  case False: exec_add_test_head(istrCheck); execIT=True; break;
  }
  return(execIT);
}

int togglePE(void)
     /*
     ** Enable/disable the path execution mode
     */
{
  switch(IsActiveExecutionPath()) {
  case True: exec_del_test(peCheck); ActiveExecutionPath(False); break;
  case False: exec_add_test_tail(peCheck); ActiveExecutionPath(True); break;
  }
  return(IsActiveExecutionPath());
}

static unsigned int execute_jump(unsigned int istr, unsigned int *PC)
     /*
     ** execute j (jump) istruction
     */
{
  unsigned int imm;

  orderedop[JUMP].counter++;
  imm=decode_j_field(istr);
  *PC=imm;
  return(ExecExactExecution);
}

static unsigned int execute_bne(unsigned int opcode_field,unsigned int istr,
				unsigned int *PC)
     /*
     ** Execute the istructions bne-like
     ** addi,sw,lw,beq,bne
     */
{
  short int imm;
  unsigned int rs,rt,RelativeHop=1;

  rs=decode_rs_field(istr);
  rt=decode_rt_field(istr);
  imm=decode_imm_offset_field(istr);
  if ((rs >= MaxReg) && (rt >= MaxReg))
    return(ExecExceededMaxReg);
  switch(opcode_field) /* choose the correct istruction */
    {
    case andi:
      orderedop[ANDI].counter++;
      if (rs!=0) registri[rs]=registri[rt]&(int)imm;
      else return(ExecWriteonROR);
      break;
    case ori:
      orderedop[ORI].counter++;
      if (rs!=0) registri[rs]=registri[rt]|(int)imm;
      else return(ExecWriteonROR);
      break;
    case xori:
      orderedop[XORI].counter++;
      if (rs!=0) registri[rs]=registri[rt]^(int)imm;
      else return(ExecWriteonROR);
      break;
    case addi:
      orderedop[ADDI].counter++;
      if (rs!=0) registri[rs]=registri[rt]+(int)imm;
      else return(ExecWriteonROR);
      break;
    case sw:
      orderedop[SW].counter++;
      /* check for memory bound */
      if (imm+registri[rt] < getMaxAddress())
	polwrite((unsigned int)imm+registri[rt],registri[rs]);
      else
	return(ExecSWWrongAddres);
      break;
    case lw:
      orderedop[LW].counter++;
      /* check for memory bound */
      if (imm+registri[rt] < getMaxAddress()) {
	if (rs!=0) registri[rs]=polread((unsigned int)imm+registri[rt]);
	else return(ExecWriteonROR);
      }
      else
	return(ExecLWWrongAddres);
      break;
    case beq:
      orderedop[BEQ].counter++;
      if (registri[rs] == registri[rt])
	RelativeHop=imm;
      break;
    case bne:
      orderedop[BNE].counter++;
      if (registri[rs] != registri[rt])
	RelativeHop=imm;
      break;
    case slti:
      orderedop[SLTI].counter++;
      if (rs != 0) {
	if (registri[rt]<(unsigned int)imm) registri[rs]=1;
	else registri[rs]=0;
      }
      else return(ExecWriteonROR);
      break;
    default: return(ExecIstrError);
    }
  (*PC)+=RelativeHop;
  return(ExecExactExecution);
}

static unsigned int execute_mtlo_mthi(unsigned int istr, unsigned int *PC)
     /*
     ** Execute mtlo and mthi
     */
{
  unsigned int rs,functfield,zerofield;

  rs=decode_rs_field(istr);
  functfield=decode_funct_field(istr);
  zerofield=decode_mthi_mtlo_zero_field(istr);
  if (zerofield != 0)   /* check if the extracted field is zero */
    return(ExecZeroFieldError);
  if (rs >= MaxReg)    /* rd out of bound */
    return(ExecExceededMaxReg);
  if (rs == 0)         /* test the register into write (R0 is read only ) */
    return(ExecWriteonROR);
  switch(functfield) {
  case functmthi:
    orderedop[MTHI].counter++;
    registri[RegHi]=registri[rs];
    break;
  case functmtlo:
    orderedop[MTLO].counter++;
    registri[RegLo]=registri[rs];
    break;
  default: return(ExecIstrError);
  }
  (*PC)++;
  return(ExecExactExecution);
}

static unsigned int execute_mflo_mfhi(unsigned int istr, unsigned int *PC)
     /*
     ** Execute mflo and mfhi
     */
{
  unsigned int rd,functfield,zerofield,small_zerofield;

  rd=decode_rd_field(istr);
  functfield=decode_funct_field(istr);
  zerofield=decode_mfhi_mflo_zero_field(istr);
  small_zerofield=decode_small_zero_field(istr);
  if (zerofield != 0)       /* check if the extracted field is zero */
    return(ExecZeroFieldError);
  if (small_zerofield != 0) /* check if the extracted field is zero */
    return(ExecSmallZeroFieldError);
  if (rd >= MaxReg)    /* rd out of bound */
    return(ExecExceededMaxReg);
  if (rd == 0)         /* test the register into write (R0 is read only ) */
    return(ExecWriteonROR);
  switch(functfield) {
  case functmfhi:
    orderedop[MFHI].counter++;
    registri[rd]=registri[RegHi];
    break;
  case functmflo:
    orderedop[MFLO].counter++;
    registri[rd]=registri[RegLo];
    break;
  default: return(ExecIstrError);
  }
  (*PC)++;
  return(ExecExactExecution);
}

static unsigned int execute_nop(void)
     /*
     ** Execute nop
     */
{
  orderedop[NOP].counter++;
  /* do nothing */ return(ExecExactExecution);
}

static unsigned int execute_halt(void)
     /*
     ** Execute halt
     */
{
  orderedop[HALT].counter++;
  /* halt the VCPU */ return(ExecHalt);
}

static unsigned int execute_add(unsigned int istr,unsigned int *PC)
     /*
     ** Execute the istructions add-like
     ** add,slt,div,mult
     */
{
  unsigned int rs,rt,rd,functfield,zerofield_div_mult,sa;

  rs=decode_rs_field(istr);
  rt=decode_rt_field(istr);
  rd=decode_rd_field(istr);
  functfield=decode_funct_field(istr);
  zerofield_div_mult=decode_div_mult_zero_field(istr);
  if ((rs >= MaxReg) || (rt >= MaxReg) || (rd >= MaxReg))
    return(ExecExceededMaxReg);
  switch (functfield) {
  case functsllv:
    orderedop[SLLV].counter++;
    if ((registri[rd] > MinShiftAmount) && (registri[rd] < MaxShiftAmount)) {
      if (rs != 0) registri[rs]=registri[rt] << registri[rd];
      else return(ExecWriteonROR);
    }
    else return(ExecSAOutOfRange);
    break;
  case functsrlv:
    orderedop[SRLV].counter++;
    if ((registri[rd] > MinShiftAmount) && (registri[rd] < MaxShiftAmount)) {
      if (rs != 0) registri[rs]=registri[rt] >> registri[rd];
      else return(ExecWriteonROR);
    }
    else return(ExecSAOutOfRange);
    break;
  case functsrl:
    orderedop[SRL].counter++;
    sa=decode_sa_field(istr);
    if ((sa > MinShiftAmount) && (sa < MaxShiftAmount)) {
      if (rs != 0) registri[rs]=registri[rt] >> sa;
      else return(ExecWriteonROR);
    }
    else return(ExecSAOutOfRange);
    break;
  case functsll:
    orderedop[SLL].counter++;
    /*
    ** functsll is 0x0, this generate an ambigous case:
    ** to not execute a 0x000...000 value, sa must be greater than zero
    */
    if ((sa=decode_sa_field(istr)) != 0) {
      if (rs != 0) registri[rs]=registri[rt] << sa;
      else return(ExecWriteonROR);
    }
    else return(ExecIstrError);
    break;
  case functadd:
    orderedop[ADD].counter++;
    if (rs != 0) registri[rs]=registri[rd]+registri[rt];
    else return(ExecWriteonROR);
    break;
  case functsub:
    orderedop[SUB].counter++;
    if (rs != 0) registri[rs]=registri[rt]-registri[rd];
    else return(ExecWriteonROR);
    break;
  case functand:
    orderedop[AND].counter++;
    if (rs != 0) registri[rs]=registri[rd]&registri[rt];
    else return(ExecWriteonROR);
    break;
  case functor:
    orderedop[OR].counter++;
    if (rs != 0) registri[rs]=registri[rd]|registri[rt];
    else return(ExecWriteonROR);
    break;
  case functxor:
     orderedop[XOR].counter++;
    if (rs != 0) registri[rs]=registri[rd]^registri[rt];
    else return(ExecWriteonROR);
    break;
  case functslt:
    orderedop[SLT].counter++;
    if (rs != 0) {
      if (registri[rt]<registri[rd]) registri[rs]=1;
      else registri[rs]=0;
    }
    else return(ExecWriteonROR);
    break;
  case functdiv:
    orderedop[DIVIDE].counter++;
    if (zerofield_div_mult != 0)
      return(ExecZeroDivMultError);
    if (registri[rt] == 0) /* divide by zero error */
      return(ExecDivByZero);
    registri[RegHi]=registri[rs]/registri[rt];
    registri[RegLo]=registri[rs]%registri[rt];
    break;
  case functmult:
    orderedop[MULT].counter++;
    if (zerofield_div_mult != 0)
      return(ExecZeroDivMultError);
    registri[RegHi]=registri[rs]*registri[rt];
    break;
  case functjr:
    orderedop[JR].counter++;
    *PC=registri[rs];
    return(ExecExactExecution); /* <<----------- */
    break;
  default: return(ExecIstrError);
  }
  (*PC)++;
  return(ExecExactExecution);
}

static unsigned int execute_lui(unsigned int istr, unsigned int *PC)
     /*
     ** Execute the lui istruction
     */
{
  short int imm;
  unsigned rt;

  rt=decode_rt_field(istr);
  imm=decode_imm_offset_field(istr);
  orderedop[LUI].counter++;
  if (rt >= MaxReg)
    return(ExecExceededMaxReg);
  if (rt == 0)
    return(ExecWriteonROR);
  registri[rt]=imm << 16;
  (*PC)++;
  return(ExecExactExecution);
}

int ritornaregistro(unsigned int Indice)
     /*
     ** Return the register value asked
     */
{
  return(registri[Indice]);
}

void resetregistri(void)
     /*
     ** Reset all registers
     */
{
  int i;

  for (i=0; i<MaxReg+2; i++)
    registri[i]=0;
}

int setregistro(int numero_registro,int value)
     /*
     ** Set the value of a regiter to value
     */
{
  if ((numero_registro > 0) && (numero_registro <= MaxReg+2)) {
    registri[numero_registro]=value;
    return(True);
  }
  else
    return(SetRegisterError);
}
/*
** END EXECUTOR
*/

/*
** PAY ATTENTION !!!
** MISCELLENIA FUNCTIONS
** DO NOT CONFUSE
*/
int reset_breakline(void)
     /*
     ** set to -1 all breakpoints and reset all testkind
     */
{
  int bpPosition;

  for (bpPosition=0; bpPosition < MaxBreakLine; bpPosition++) {
    breakline[bpPosition].address=-1;
    breakline[bpPosition].reg=-1;
    breakline[bpPosition].testkind='?';
    breakline[bpPosition].valuetest=-1;
  }
  return(True);
}

static int dicotomic_find_breakline(unsigned int valore,int inizio,
				    int fine)
     /*
     ** the position of valore into breakline else return -1
     */
{
  int bpPosition=0;

  /*if (fine < inizio)
    return(-1);
  else {
    k=(inizio+fine)/2;
    if (breakline[k] == valore)
      return(k);
    else if (breakline[k] > valore)
      return(dicotomic_find_breakline(valore,inizio,k-1));
    else
      return(dicotomic_find_breakline(valore,k+1,fine));
  }*/
  while ((bpPosition < MaxBreakLine) && (breakline[bpPosition].address != -1)) {
    if (breakline[bpPosition].address == valore)
	return(bpPosition);
    bpPosition++;
  }
  return(-1);
}

int getMaxBreakLine(void)
     /*
     ** Return the number of available breakpoints
     */
{
  return(MaxBreakLine);
}

int getBreakLinePC(int index, int *value)
     /*
     ** Return the breakpoint PC at index
     */
{
  if ((index >= 0) && (index < MaxBreakLine)) {
    *value=breakline[index].address;
    return(True);
  }
  else
    return(False);
}

int getBreakLine(int index, int *value, int *reg, int *cost,char *testkind)
     /*
     ** Return all the breakpoint information at index
     */
{
  if ((index >= 0) && (index < MaxBreakLine)) {
    *value=breakline[index].address;
    *reg=breakline[index].reg;
    *testkind=breakline[index].testkind;
    *cost=breakline[index].valuetest;
    return(True);
  }
  else
    return(False);
}

int setBreakLine(int index, int value, int reg, int cost,char testkind)
     /*
     ** Write the number of line at break points number index
     ** plus the informations for the condition
     */
{
  if ((index >= 0) && (index < MaxBreakLine)) {
    breakline[index].address=value;
    breakline[index].reg=reg;
    breakline[index].testkind=testkind;
    breakline[index].valuetest=cost;
    return(True);
  }
  return(False);
}

int clearBreakLine(int index)
     /*
     ** Reset the breakpoint at index
     */
{
  if ((index >= 0) && (index < MaxBreakLine)) {
    breakline[index].address=-1;
    breakline[index].reg=-1;
    breakline[index].testkind='?';
    breakline[index].valuetest=-1;
    return(True);
  }
  return(False);
}

void compactBreakLine(unsigned int index)
{
  int i;

  for (i=index+1; i < MaxBreakLine; i++) {
    breakline[i-1].address=breakline[i].address;
    breakline[i-1].reg=breakline[i].reg;
    breakline[i-1].testkind=breakline[i].testkind;
    breakline[i-1].valuetest=breakline[i].valuetest;
  }
  clearBreakLine(MaxBreakLine-1);
}

unsigned int getOutofTimeLimit(void)
     /*
     ** Return the TimeLimit time bound
     */
{
  return(OutofTimeLimit);
}

unsigned int getMinOutofTimeLimit(void)
{
  return(MinOutofTimeLimit);
}

unsigned int getMaxOutofTimeLimit(void)
{
  return(MaxOutofTimeLimit);
}

int setOutofTimeLimit(unsigned int newvalue)
     /*
     ** Check the new TimeLimit bound
     */
{
  if ((newvalue >= getMinOutofTimeLimit()) &&
      (newvalue <= getMaxOutofTimeLimit())) {
    OutofTimeLimit=newvalue;
    return(True);
  }
  return(False);
}

unsigned int getIstrLimit(void)
{
  return(IstrLimit);
}

unsigned int getMinIstrLimit(void)
{
  return(MinIstrLimit);
}

unsigned int getMaxIstrLimit(void)
{
  return(MaxIstrLimit);
}

int setIstrLimit(unsigned int newvalue)
{
  if ((newvalue >= getMinIstrLimit()) &&
      (newvalue <= getMaxIstrLimit())) {
    IstrLimit=newvalue;
    return(True);
  }
  return(False);
}

/*
** END MISCELLENIA FUNCTION
*/

/*
** PAY ATTENTION !!!
** ISTRUCTION COUNTER CODE UNDER THIS LINE
** DO NOT CONFUSE
*/

int reset_istruction_count(void)
     /*
     ** Reset all the counter for every istruction
     */
{
  int i;

  for (i=0; i<get_num_istruction(); i++)
    orderedop[i].counter=0;
  return(True);
}

unsigned int get_istruction_count(unsigned int index)
     /*
     ** Return the counter
     */
{
  if (index < get_num_istruction())
    return(orderedop[index].counter);
  else
    return(IstrIndexOutBound);
}

int get_istruction_name(unsigned int index, char *name,
			unsigned int name_size)
     /*
     ** Return the ascii name of the istruction with position index
     */
{
  if ((index < get_num_istruction()) && (sizeof(orderedop[index].OpString) <= name_size)){
    strcpy(name,orderedop[index].OpString);
    return(True);
  }
  else
    return(False);
}

unsigned int get_num_istruction(void)
     /*
     ** Return the number of istructions understood by executor
     */
{
  unsigned int i=0;

  while (orderedop[i].OpString != NULL)
    i++;
  return(i);
}

/*
** END ISTRUCTION COUNTER
*/

static int LoadRegistersValue(void)
     /*
     ** Load the registers value if the option RegisterEntrySaveOpt is True (T)
     */
{
  FILE *fp;
  char RegPrf[FileNameLength];
  char RetValue[MediumBufferLength];
  char XmasDir[XmasDirLength];
  char row[MediumBufferLength];
  int i;
  int TmpRegs[MaxReg+2];
  unsigned int regpos,regvalue;

  if (getOptionFromName(RegisterPreferencesFile,RegisterEntrySaveOpt,RetValue,sizeof(RetValue)) == True)
    if (getBoolOptionsValue(RetValue)) {
      XmasDirStatus(XmasDir,sizeof(XmasDir));
      sprintf(RegPrf,"%s/%s",XmasDir,RegisterSaveDataFile);
      if ((fp=fopen(RegPrf,"r")) != NULL) {
	while (feof(fp) != True) { /* scan all file */
	  fgets(row,sizeof(row),fp);
	  if (strncmp(row,RegisterEntrySaveReg,strlen(RegisterEntrySaveReg)) 
	      == 0) {
	    sscanf(row,RegisterEntrySaveReg"%x=%x",&regpos,&regvalue);
	    TmpRegs[regpos]=regvalue;
	  }
	  if (strncmp(row,RegisterEntrySaveHi,strlen(RegisterEntrySaveHi))
	      == 0) {
	    sscanf(row,RegisterEntrySaveHi"=%x",&regvalue);
	    TmpRegs[MaxReg]=regvalue;
	  }
	  if (strncmp(row,RegisterEntrySaveLo,strlen(RegisterEntrySaveLo))
	      == 0) {
	    sscanf(row,RegisterEntrySaveLo"=%x",&regvalue);
	    TmpRegs[MaxReg+1]=regvalue;
	  }
	}
	for (i=0; i<MaxReg+2; i++)
	  setregistro(i,TmpRegs[i]);
	fclose(fp);
	return(True);
      }
    }
  return(False);
}

int SaveRegistersStatus(char *XmasDir)
     /*
     ** save the registers value
     */
{
  FILE *fp;
  char filename[FileNameLength];
  char RetValue[MediumBufferLength];
  unsigned int i;

  sprintf(filename,"%s/%s",XmasDir,RegisterSaveDataFile);
  if ((fp=fopen(filename,"w")) != NULL) {
    if (getOptionFromName(RegisterPreferencesFile,RegisterEntrySaveOpt,RetValue,sizeof(RetValue)) == True)
      if (getBoolOptionsValue(RetValue)) {
	for (i=0; i<MaxReg; i++)
	  fprintf(fp,"%s%x=0x%x\n",RegisterEntrySaveReg,i,registri[i]);
	fprintf(fp,"%s=0x%x\n",RegisterEntrySaveHi,registri[MaxReg]);
	fprintf(fp,"%s=0x%x\n",RegisterEntrySaveLo,registri[MaxReg+1]);
      }
    fclose(fp);
    return(True);
  }
  return(False);
}

static int LoadBreakpointsValue(void)
     /*
     ** Load the breakpoints status from preferences file
     */
{
  FILE *fp;
  char RetValue[MediumBufferLength];
  char BPPrf[FileNameLength];
  char XmasDir[XmasDirLength];
  unsigned int index,pc,reg,value;
  char test;

  if (getOptionFromName(BreakpointsPreferencesFile,BreakpointsSaveOpt,RetValue,sizeof(RetValue)) == True)
    if (getBoolOptionsValue(RetValue)) {
      XmasDirStatus(XmasDir,sizeof(XmasDir));
      sprintf(BPPrf,"%s/%s",XmasDir,BreakpointsSaveDataFile);
      if ((fp=fopen(BPPrf,"r")) != NULL) {
	while (feof(fp) != True) { /* scan all file */
	  if (fscanf(fp,BreakpointsSaveBP"%x=%x %x %c %x\n",&index,&pc,&reg,&test,&value)
	      == 5) {
	    /* 5 => the number of item to get */
	    breakline[index].address=pc;
	    breakline[index].reg=reg;
	    breakline[index].testkind=test;
	    breakline[index].valuetest=value;
	  }
	}
      }
    }
  return(False);
}

int SaveBreakpointsStatus(char *XmasDir)
     /*
     ** Save the breakpoints status to preferences file
     */
{
  FILE *fp;
  char filename[FileNameLength];
  char RetValue[MediumBufferLength];
  unsigned int i;

  sprintf(filename,"%s/%s",XmasDir,BreakpointsSaveDataFile);
  if ((fp=fopen(filename,"w")) != NULL) {
    if (getOptionFromName(BreakpointsPreferencesFile,BreakpointsSaveOpt,RetValue,sizeof(RetValue)) == True)
	if (getBoolOptionsValue(RetValue)) {
	for (i=0; i < MaxBreakLine; i++)
	  fprintf(fp,"%s%x=0x%x 0x%x %c 0x%x\n",BreakpointsSaveBP,i,breakline[i].address,breakline[i].reg,breakline[i].testkind,breakline[i].valuetest);
      }
    fclose(fp);
    return(True);
  }
  return(False);
}

static int LoadExecutorValues(void)
{
  FILE *fp;
  char RetValue[MediumBufferLength];
  char XmasDir[XmasDirLength];
  char ExecPrf[FileNameLength];
  char row[MediumBufferLength];
  unsigned int newTimeLimitValue,newIstrLimitValue;

  if (getOptionFromName(ExecutorPreferencesFile,ExecutorTimeLimitSaveOpt,RetValue,sizeof(RetValue)) == True)
    if (getBoolOptionsValue(RetValue)) {
      XmasDirStatus(XmasDir,sizeof(XmasDir));
      sprintf(ExecPrf,"%s/%s",XmasDir,ExecutorSaveDataFile);
      if ((fp=fopen(ExecPrf,"r")) != NULL) {
	while (feof(fp) != True) { /* scan all file */
	  fgets(row,MediumBufferLength,fp);
	  if (sscanf(row,ExecutorTimeLimitSave" %d\n",&newTimeLimitValue))
	    OutofTimeLimit=newTimeLimitValue;
	  if (sscanf(row,ExecutorIstrLimitSave" %d\n",&newIstrLimitValue))
	    IstrLimit=newIstrLimitValue;
	}
      }
    }
  return(False);
}
int SaveExecutorValues(char *XmasDir)
{
  FILE *fp;
  char filename[FileNameLength];
  char RetValue[MediumBufferLength];

  sprintf(filename,"%s/%s",XmasDir,ExecutorSaveDataFile);
  if ((fp=fopen(filename,"w")) != NULL) {
    if (getOptionFromName(ExecutorPreferencesFile,ExecutorTimeLimitSaveOpt,
			  RetValue,sizeof(RetValue)) == True)
	if (getBoolOptionsValue(RetValue))
	  fprintf(fp,ExecutorTimeLimitSave" %d\n",OutofTimeLimit);
    if (getOptionFromName(ExecutorPreferencesFile,ExecutorIstrLimitSaveOpt,
			  RetValue,sizeof(RetValue)) == True)
	if (getBoolOptionsValue(RetValue))
	  fprintf(fp,ExecutorIstrLimitSave" %d\n",IstrLimit);
  }
  return(False);
}
