/*	
** Xmas (X memory access simulator) is a little memory (+cache)
** simulator.  Copyright (C) 1997/98/99/00 <Paolo Dovera
** pd514314@silab.dsi.unimi.it> <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

#ifndef LOADERP_H
#define LOADERP_H

struct scelta_opcode
{
  char *OpString;
  unsigned int Valore;
  unsigned int FunctField;
  unsigned int counter;
};

struct scelta_opzioni
{
  char *OpString;
  unsigned int Valore;
};

/*
** Logical mask for extract the fields
*/
#define rs_mask 0x03e00000
#define rt_mask 0x001f0000
#define rd_mask 0x0000f800
#define funct_mask 0x0000003f
#define small_zero_mask 0x000007c0
#define imm_mask 0x0000ffff
#define div_mult_zero_mask 0x0000ffc0
#define mfhi_mflo_zero_mask 0x03ff0000
#define mthi_mtlo_zero_mask 0x000fffe0

/*
** PAY ATTENTION !!!
** LOADER CODE UNDER THIS LINE
** DO NOT CUNFUSE
*/
static int dicotomic_find_breakline(unsigned int valore,int inizio,int fine);
static int check_condition(unsigned int index);
static int find_opcode(char *nomeistr,unsigned int *opcode, unsigned int *ff);
static int find_set(char *string);

static int encode_imm_offset_field(char **AlfaNumIstrRef);
static unsigned int encode_register_field(char **AlfaNumIstrRef);
static unsigned int encoder_jump(unsigned int *NumericIstruction,
				 char **AlfaNumIstrRef);
static unsigned int encoder_add(unsigned int *NumericIstruction,
				unsigned int ff, char **AlfaNumIstrRef);
static unsigned int encoder_add_rd_field(unsigned int *NumericIstruction,
					 char **AlfaNumIstrRef,
					 unsigned int functfield);
static unsigned int encoder_add_sa_field(unsigned int *NumericIstruction,
					 char **AlfaNumIstrRef,
					 unsigned int functfield);
static unsigned int encoder_mflo_mfhi(unsigned int *NumericIstruction,
				      unsigned int ff, char **AlfaNumIstrRef);
static unsigned int encoder_mtlo_mthi(unsigned int *NumericIstruction,
				      unsigned int ff, char **AlfaNumIstrRef);
static unsigned int encoder_nop_halt(unsigned int *NumericIstruction);
static unsigned int encoder_addi_lw_sw_beq_bne(unsigned int *NumericIstruction,
					       char **AlfaNumIstrRef);
static unsigned int encoder_lui(unsigned int *NumericIstruction,
				char **AlfaNumIstrRef);
static int readsubstring(char **inputstring,char *outputstring);
/*
** END LOADER
*/

/*
** PAY ATTENTION !!!
** UNLOADER CODE UNDER THIS LINE
** DO NOT CUNFUSE
*/

/*
** CLASS decoder
*/
static int decoder_bne(unsigned int opcode_field,unsigned int istr,
		       char *string);
static int decoder_mflo_mfhi(unsigned int istr,char *string);
static int decoder_mtlo_mthi(unsigned int istr,char *string);
static int decoder_halt(char *string);
static int decoder_add(unsigned int istr,char *string);
static int decoder_nop(char *string);
static int decoder_lui(unsigned int istr,char *string);
static int decoder_jump(unsigned int istr, char *string);

/*
** END UNLOADER
*/

/*
** PAY ATTENTION !!!
** ESECUTORE CODE UNDER THIS LINE
** DO NOT CUNFUSE
*/
/*
** executor mode
*/
#define SbS 0x0 /* Step by Step (not used, I use ~Fr) */
#define Fr  0x1 /* Free Run mode */
#define Bp  0x2 /* active Breakpoint */
#define Tl  0x4 /* active TimeLimit */
#define Pe  0x8 /* active Path Execution (not used) */

static void exec_add_test_head(int (*test)(unsigned int));
static void exec_add_test_tail(int (*ctl)(unsigned int));
/*static void exec_show_test(void);*/
static void exec_del_test(int (*ctl)(unsigned int));
static struct executor_mode_list *ric_delete(struct executor_mode_list *list,
					     int (*ctl)(unsigned int));
static int ModeTest(unsigned int PC);

static void StartToolsForExecution(void);
static void StopToolsForExecution(void);

static int bellCheck(unsigned int i);
static int istrCheck(unsigned int i);
static int bpCheck(unsigned int PC);
static int peCheck(unsigned int PC);

static void gestore(int i);

unsigned int esegui(unsigned int *PC,unsigned int MaxAddress);
static unsigned int execute_bne(unsigned int opcode_field,unsigned int istr,
				unsigned int *PC);
static unsigned int execute_mflo_mfhi(unsigned int istr, unsigned int *PC);
static unsigned int execute_mtlo_mthi(unsigned int istr, unsigned int *PC);
static unsigned int execute_halt(void);
static unsigned int execute_add(unsigned int istr, unsigned int *PC);
static unsigned int execute_nop(void);
static unsigned int execute_lui(unsigned int istr, unsigned int *PC);
static unsigned int execute_jump(unsigned int istr, unsigned int *PC);

/*
** END EXECUTOR
*/

/*
** LOAD/SAVE
*/
static int LoadRegistersValue(void);
static int LoadBreakpointsValue(void);
static int LoadExecutorValues(void);
/*
** END LOAD/SAVE
*/

/*
** ISTRUCTIONS DEFINITION
*/

#define IstrStringCar 45
#define OpStr 15 /* opcode size of an istruction */
/*
** NB: do not go down RegString because Xmas can't read the immediate fields
**     more lenght of n char
**     10  => ten chars for decimal numbers (platform depend !?)
*/
#define RegString 10 /* char dimension of a registerd */
#define OpBit      6 /* available bit for opcode */
#define RegBit     5 /* available bit for a register */
#define ImmBit    16 /* available bit for immediate filed */

/* OpCode and FunctField of implemeted istructions */
#define add    0x0
#define ADD  0
#define addi   0x8
#define ADDI 1
#define and    0x0
#define AND  2
#define andi   0xc
#define ANDI 3
#define beq    0x4
#define BEQ  4
#define bne    0x5
#define BNE  5
#define divide 0x0
#define DIVIDE 6
#define halt   0x3f /* opcode: 111111 */
#define HALT 7
#define lui    0xf
#define LUI  8
#define lw     0x23
#define LW   9
#define jump   0x2
#define JUMP 10
#define jr     0x0
#define JR   11
#define mfhi   0x1 /* opcode != MIPS R2000 */
#define MFHI 12
#define mflo   0x1 /* opcode != MIPS R2000 */
#define MFLO 13
#define mthi   0x3d /* opcode: 111101 != MIPS R2000 */
#define MTHI 14
#define mtlo   0x3d /* opcode: 111101 != MIPS R2000 */
#define MTLO 15
#define mult   0x0
#define MULT 16
#define nop    0x3e /* opcode: 111110 */
#define NOP  17
#define or     0x0
#define OR   18
#define ori    0xd
#define ORI  19
#define sll    0x0
#define SLL  20
#define sllv   0x0
#define SLLV 21
#define slt    0x0
#define SLT  22
#define slti   0xb
#define SLTI 23
#define srl    0x0
#define SRL  24
#define srlv   0x0
#define SRLV 25
#define sub    0x0
#define SUB  26
#define sw     0x2b
#define SW   27
#define xor    0x0
#define XOR  28
#define xori   0xe
#define XORI 29

#define functmthi 0x11
#define functmtlo 0x13
#define functmfhi 0x10
#define functmflo 0x12
#define functslt  0x2a
#define functadd  0x20
#define functsub  0x22
#define functand  0x24
#define functor   0x25
#define functdiv  0x1a
#define functmult 0x18
#define functxor  0x26
#define functsll  0x0
#define functsllv 0x4
#define functsrl  0x2
#define functsrlv 0x6
#define functjr   0x8

/*
** dummy funct fields to write a more compact switch
*/
#define functdummy 0xffffffff
#define functdummy1 0xfffffffe

#endif
