/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** Main memory implemetation, word oriented
** Modulo per l' implementazione della memoria orientata alla word
** 1 word = 1 integer (Linux: 4 byte)
*/
/*
** generals include
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
** include of application modules
*/
#include "general_support.h"
#include "opthandler.h"
#include "memory.h"
#include "esp.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
#define MinBitSizeMemory 1
#define MaxBitSizeMemory 0xb
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES*/
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static int LoadMemoryValue(void);
static int LoadMemoryControls(void);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static int *Memory;               /* memory pointer */
static unsigned int MaxAddress=0; /* max memory address */
/*
** Must be 8 for a correct sizing of viewport widget into interfaccia.c
*/
static unsigned int BitMemory=8;  /* size of memory in bits */

static int ReshowMemoryAscii=True;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void memwrite(unsigned int Address,int MemValue)
     /*
     ** Address < MaxAddress
     ** write Value at address Address
     */
{
  if (Address < MaxAddress)    /* the address value is correct */
    Memory[Address]=MemValue;
  else {
    printf("ERROR: Exceeded memory address on write %u > MaxAddress = %d\n",
	   Address,MaxAddress-1);
    exit(EXIT_FAILURE);
  }
}

int memread(unsigned int Address)
     /*
     ** Address < MaxAddress
     ** read the value at address Address
     */
{
  if (Address < MaxAddress)    /* the address value is correct */
    return(Memory[Address]);
  else {
    printf("ERROR: Exceeded memory address on read %u > MaxAddress = %d\n",
	   Address,MaxAddress-1);
    exit(EXIT_FAILURE);
  }
}

void inizializza_memoria(void)
     /*
     ** Create the memory and initialize it
     */
{
  MaxAddress=(unsigned int)esp(2,BitMemory);
  if ((Memory=(int *)malloc(MaxAddress*sizeof(int))) == NULL) {
    printf("ERROR: No memory available to create memory structure\n");
    exit(EXIT_FAILURE);
  }
  invalidmemory();
  LoadMemoryControls();
  LoadMemoryValue();
}

void invalidmemory(void)
     /*
     ** Initialize the memory
     */
{
  (void *)memset(Memory,0,sizeof(Memory)*MaxAddress);
}

int reset_memory_size(unsigned int BitnewMemory)
     /*
     ** BitMemory > 0
     ** update the state of the memory for the new dimension
     */
{
  int *support;

  if ((support=(int *)malloc(esp(2,BitnewMemory)*sizeof(int))) == NULL)
    return(False);
  MaxAddress=(unsigned int)esp(2,BitnewMemory);
  free(Memory);
  Memory=support;
  invalidmemory();
  return(True);
}

unsigned int getMaxAddress(void)
     /*
     ** Return the maximum address of memory
     */
{
  return(MaxAddress);
}

unsigned int getBitMemory(void)
     /*
     ** Return the #bit of memory
     */
{
  return(BitMemory);
}

unsigned int setBitMemory(unsigned int newBitMemory)
     /*
     ** MinBitSize <= newBitMemory <= MaxBitSize
     ** set the number of bit memory
     */
{
  if ((newBitMemory >= MinBitSizeMemory) &&
      (newBitMemory <= MaxBitSizeMemory)) {
    BitMemory=newBitMemory;
    return(True);
  }
  else
    return(False);
}

unsigned int getMemoryAlphaNumericStatus(void)
     /*
     ** Return true if memory AlphaNumeric is active
     */
{
  return(ReshowMemoryAscii);
}

void setMemoryAlphaNumericStatus(unsigned int status)
     /*
     ** Change the alfanumeric display memory mode
     */
{
  ReshowMemoryAscii=status;
}

static int LoadMemoryControls(void)
     /*
     ** Load the memory controls from preferences file
     */
{
  FILE *fp;
  char MemoryPrf[FileNameLength];
  char RetValue[MediumBufferLength];
  char XmasDir[XmasDirLength];
  char row[LongBufferLength];
  unsigned int AlphaNumeric;

  if (getOptionFromName(MemoryPreferencesFile,MemoryEntrySaveOpt,RetValue,sizeof(RetValue)) == True) {
    if (getBoolOptionsValue(RetValue)) {
      XmasDirStatus(XmasDir,sizeof(XmasDir));
      sprintf(MemoryPrf,"%s/memory",XmasDir);
      if ((fp=fopen(MemoryPrf,"r")) != NULL) {
	while (feof(fp) != True) { /* scan all file */
	  fgets(row,sizeof(row),fp);
	  if (strncmp(row,MemoryAlphaNumericOptString,strlen(MemoryAlphaNumericOptString)) == 0)
	    if (sscanf(row+strlen(MemoryAlphaNumericOptString),"%x",&AlphaNumeric) == 1)
	      ReshowMemoryAscii=AlphaNumeric;
	}
	fclose(fp);
	return(True);
      }
    }
  }
  return(False);
}

static int LoadMemoryValue(void)
     /*
     ** Load the memor entry if the option MemoryEntrySaveOpt is True (T)
     */
{
  FILE *fp;
  char MemoryPrf[FileNameLength];
  char RetValue[MediumBufferLength];
  char XmasDir[XmasDirLength];
  char row[LongBufferLength];
  unsigned int i;
  int *TmpMemory;
  unsigned int memoryaddress,memoryvalue;

  if (getOptionFromName(MemoryPreferencesFile,MemoryEntrySaveOpt,RetValue,sizeof(RetValue)) == True) {
    if (getBoolOptionsValue(RetValue)) {
      XmasDirStatus(XmasDir,sizeof(XmasDir));
      sprintf(MemoryPrf,"%s/%s",XmasDir,MemorySaveDataFile);
      if ((fp=fopen(MemoryPrf,"r")) != NULL) {
	TmpMemory=(int *)malloc(getMaxAddress()*sizeof(int));
	memset(TmpMemory,0,getMaxAddress()*sizeof(int));
	while (feof(fp) != True) { /* scan all file */
	  fgets(row,sizeof(row),fp);
	  if (strncmp(row,MemoryEntrySaveLdm,strlen(MemoryEntrySaveLdm)) == 0) {
	    sscanf(row,MemoryEntrySaveLdm"%x=%x",&memoryaddress,&memoryvalue);
	    if (memoryaddress < MaxAddress)
	      /*
	      ** only memory address less than MaxAddress are valid,
	      ** others memory addresses are skipped
	      */
	      TmpMemory[memoryaddress]=memoryvalue;
	  }
	}
	for (i=0; i<MaxAddress; i++)
	  memwrite(i,TmpMemory[i]);
	free(TmpMemory);
	fclose(fp);
	return(True);
      }
    }
  }
  return(False);
}


int SaveMemoryStatus(char *XmasDir)
     /*
     ** save the memory value
     */
{
  FILE *fp;
  char filename[FileNameLength];
  char RetValue[MediumBufferLength];
  unsigned int i;

  sprintf(filename,"%s/%s",XmasDir,MemorySaveDataFile);
  if ((fp=fopen(filename,"w")) != NULL) {
    if (getOptionFromName(MemoryPreferencesFile,MemoryEntrySaveOpt,RetValue,sizeof(RetValue)) == True)
      if (getBoolOptionsValue(RetValue)) {
	fprintf(fp,"%s %d\n",MemoryAlphaNumericOptString,ReshowMemoryAscii);
	for (i=0; i<MaxAddress; i++)
	  fprintf(fp,"%s%x=0x%x\n",MemoryEntrySaveLdm,i,memread(i));
      }
    fclose(fp);
    return(True);
  }
  return(False);
}
