/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>

/*
** Public include file for widgets we actually use in this file.
*/
#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Shell.h>

/*
** include of application modules
*/
#include "ErrorMessage.h"
#include "menu.h"
#include "menuISTRCOUNTER.h"
#include "GUI_support.h"
#include "loader.h"
#include "global.h"

/* BEGIN LOCAL DEFINE */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void ClearAllCounter(Widget w, XtPointer client_data, XtPointer call_data);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget *entryMenuIstrCounter;
static Widget buttonistrcounter, menuistrcounter;
static Widget total_istr_count;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

/*
** Action
*/
void UpdateMenuCounter(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Display the counter for every istruction
     */
{
  char string_for_update[ShortBufferLength];
  char ascii_opcode[SmallBufferLength];
  unsigned int i,totale_istruzioni=0,num_istr;

  for (i=0; i < get_num_istruction(); i++) {
    num_istr=get_istruction_count(i);
    totale_istruzioni+=num_istr;
    get_istruction_name(i,ascii_opcode,sizeof(ascii_opcode));
    sprintf(string_for_update,"#%-5s: %d",ascii_opcode,num_istr);
    XtVaSetValues(entryMenuIstrCounter[i],XtNlabel,string_for_update,NULL);
  }
  sprintf(string_for_update,"total : %d",totale_istruzioni);
  XtVaSetValues(total_istr_count,XtNlabel,string_for_update,NULL);
}

/*
** Callback
*/
static void ClearAllCounter(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Clear every counter for the istructions
     */
{
  reset_istruction_count(); /* from loader.c */
}

void InitializemenuISTRCOUNTER(Widget form)
     /*
     ** Create the widget for istruction counter menu
     */
{
   int i;
   char name[ShortBufferLength];
   static Widget separator,clear_all_counter,separator1;
   unsigned int NumIstructions;
   XtAccelerators Accelerators; /* to parse the accelerators table */

   NumIstructions=get_num_istruction();
   /* widget memory dynamically created to show all counters */
   entryMenuIstrCounter=(Widget *)malloc(sizeof(Widget)*NumIstructions);
   /* Menu Istruction Counter */
   buttonistrcounter = XtVaCreateManagedWidget(
	"buttonistrcounter",
	menuButtonWidgetClass,
	form,
	XtNmenuName,"menuistrcounter",
	NULL);
   menuistrcounter = XtVaCreatePopupShell(
	"menuistrcounter",
	simpleMenuWidgetClass,
	buttonistrcounter,
	XtNwidth,150,NULL);

   for (i=0; i < NumIstructions; i++) {
     sprintf(name,"istrcounter%d",i+1);
     entryMenuIstrCounter[i] = XtVaCreateManagedWidget(name, smeBSBObjectClass,
				 menuistrcounter, NULL);
   }
   separator1 = XtVaCreateManagedWidget(
	       "separator1",
	       smeLineObjectClass,
	       menuistrcounter,NULL);
   total_istr_count = XtVaCreateManagedWidget(
        "total_istr_count",
	smeBSBObjectClass,
	menuistrcounter, NULL);
   separator = XtVaCreateManagedWidget(
	       "separator",
	       smeLineObjectClass,
	       menuistrcounter,NULL);
   clear_all_counter = XtVaCreateManagedWidget(
        "clear_all_counter",
	smeBSBObjectClass,
	menuistrcounter, NULL);

   /* callbacks */
   XtAddCallback(clear_all_counter,XtNcallback,ClearAllCounter,(XtPointer)0);

   /* to install accelerators */
   Accelerators = XtParseAcceleratorTable
   ("#override\n\
       <BtnDown>: UpdateMenuCounter() reset() PopupMenu()");
   XtVaSetValues(buttonistrcounter,XtNaccelerators,Accelerators,NULL);
   XtInstallAccelerators(buttonistrcounter,buttonistrcounter);
}







