/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>

/*
** Include files required for all Toolkit programs
*/
#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
** Public include file for widgets we actually use in this file.
*/
#include <X11/Xlib.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Shell.h>

/*
** include of application modules
*/
#include "Preferences.h"
#include "menu.h"
#include "Edit.h"
#include "global.h"

#include "images/xlogo11"

/* BEGIN LOCAL DEFINE */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/*
** from interfaccia.c for disable/enable the alt message
*/
extern int AlwaysMessageOnHalt;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
unsigned int EditorAL=False; /* for loader.c */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static void EditorAutoLoad(Widget w,XtPointer client_data,XtPointer call_data);
static void EditorAutoSave(Widget w,XtPointer client_data,XtPointer call_data);
static void DisableMessageOnHalt(Widget w,XtPointer client_data,
			       XtPointer call_data);
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
#define NUM_MENU_FILE 4
static Widget entryMenuOption[NUM_MENU_FILE];
static Widget buttonoption, menuoption;

static Pixmap xlogo;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

/*
** Callback
*/
static void EditorAutoLoad(Widget w, XtPointer client_data,XtPointer call_data)
     /*
     ** Popup the editor
     */
{
  Widget EAL = (Widget) client_data;

  switch (EditorAL) {
  case False: XtVaSetValues(EAL,XtNleftBitmap,xlogo,NULL); EditorAL=True;
    break;
  case True: XtVaSetValues(EAL,XtNleftBitmap,NULL,NULL); EditorAL=False;
    break;
  }
}

/*
** Callback
*/
static void EditorAutoSave(Widget w, XtPointer client_data,XtPointer call_data)
     /*
     ** Popup the editor
     */
{
  Widget EAS = (Widget) client_data;
  static unsigned int EditorAS=False;

  switch (EditorAS) {
  case False: XtVaSetValues(EAS,XtNleftBitmap,xlogo,NULL); EditorAS=True;
    EditorActiveEAS();
    break;
  case True: XtVaSetValues(EAS,XtNleftBitmap,NULL,NULL); EditorAS=False;
    EditorDisableEAS();
    break;
  }
}

/*
** Callback
*/
static void DisableMessageOnHalt(Widget w, XtPointer client_data,
				XtPointer call_data)
     /*
     ** enable/disable the message on every alt istruction
     ** through AlwaysMessageOnHalt
     */
{
  Widget Disabler = (Widget) client_data;

  switch (AlwaysMessageOnHalt) {
  case True:  XtVaSetValues(Disabler,XtNleftBitmap,xlogo,NULL);
    AlwaysMessageOnHalt=False;
    break;
  case False: XtVaSetValues(Disabler,XtNleftBitmap,NULL,NULL);
    AlwaysMessageOnHalt=True;
    break;
  }
}

/*
** Callback
*/
static void Preferences(Widget w, XtPointer client_data,
				XtPointer call_data)
{
  PopupPreferences("general.prf");
}

void InitializemenuOPTION(Widget form)
     /*
     ** Create the menu Options
     */
{
   int i;
   char name[ShortBufferLength];

   /* Menu Option */
   buttonoption = XtVaCreateManagedWidget(
	"buttonoption",
	menuButtonWidgetClass,
	form,
	XtNmenuName,"menuoption",
	NULL);
   menuoption = XtCreatePopupShell(
	"menuoption",
	simpleMenuWidgetClass,
	buttonoption,
	NULL,
        0);

   for (i=0; i < NUM_MENU_FILE; i++) {
     sprintf(name,"option%d",i+1);
     entryMenuOption[i] = XtCreateManagedWidget(name, smeBSBObjectClass,
						menuoption, NULL, 0);
   }
   XtAddCallback(entryMenuOption[0], XtNcallback, EditorAutoLoad,
		 entryMenuOption[0]);
   XtAddCallback(entryMenuOption[1], XtNcallback, EditorAutoSave,
		 entryMenuOption[1]);
   XtAddCallback(entryMenuOption[2], XtNcallback, DisableMessageOnHalt,
		 entryMenuOption[2]);
   XtAddCallback(entryMenuOption[3], XtNcallback, Preferences,
		 entryMenuOption[3]);
   xlogo=XCreateBitmapFromData(XtDisplay(buttonoption),
			      RootWindowOfScreen(XtScreen(buttonoption)),
			      xlogo11_bits, xlogo11_width, xlogo11_height);
}




