/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/times.h>

/*
** Intrinsics Definitions
*/
#include <X11/Intrinsic.h>     /* Intrinsics Definitions  */
#include <X11/StringDefs.h>    /* Standard Name-String definitions */

/*
** Public include file for widgets I actually use in this file.
*/
#include <X11/Shell.h>         /* Athena ShellWidget      */
#include <X11/Xaw/Form.h>      /* Athena Form Widget      */
#include <X11/Xaw/Command.h>   /* Athena CommandWidget    */
#include <X11/Xaw/AsciiText.h> /* Athena AsciiText Widget */
#include <X11/Xaw/Label.h>     /* Athena Label Widget     */

/*
** include of application modules
*/
#include "global.h"
#include "opthandler.h"
#include "general_support.h"
#include "OpenSave.h"
#include "ErrorMessage.h"
#include "pathexec.h"
#include "GUI_support.h"
#include "loader.h"
#include "policy.h"
#include "esp.h"

/* BEGIN LOCAL DEFINE */
/* hardcoded! how do I obtain a dynamic value? */
#define rowOutputLength ShortBufferLength
#define MediaMemory 2
#define MediaDisk   4

#define OutputFileNameOnly "XmasPathTracer.dat" /* no directory */
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
extern Widget topLevel;
extern XtAppContext app_context;
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES */
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC FUNCTION (PRIVATE FUNCTION) */
static int generic_text_reset(Widget text,char *ptr,int start,int end,
			      unsigned int length);
static void CreateExecutionPath(void);
static void PopdownEdit(Widget w, XtPointer client_data, XtPointer call_data);
static void DeleteList(Widget w, XtPointer client_data, XtPointer call_data);
static void UpdateStatus(void);
static void UpdateIstrCounted(void);
static void Save(Widget w, XtPointer client_data, XtPointer call_data);
static void Saver(String filename);
static int Add2File(unsigned int PC);
static int Add2List(unsigned int PC);
/*  static void showlistPC(void); */
/* END STATIC FUNCTION (PRIVATE FUNCTION) */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static Widget xmasExecutionPath;
static Widget show_text;
static Widget status;
static Widget istr_counted;

static unsigned int PATHEXECUTION=FALSE;
static unsigned int CreatedExecutionPath=FALSE;
static unsigned int exec_counter=0;

static unsigned int StorageMedia=MediaDisk;
static unsigned int NewTracing=False;

static unsigned int ChunckIndex=0;/* array index */
static unsigned int ChunckSize=1; /* size of dynamic array referred
				     by PEqueue */
static unsigned int *PEqueue;     /* array reference */

static char outputfilename[MediumBufferLength];
static FILE *outputfile;
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void InitializeExecutionPath(void)
{
  char tmpPath[MediumBufferLength];
  int FileName,DirNameLength;

  if (getOptionFromName(GeneralPreferencesFile,TmpDir,tmpPath,sizeof(tmpPath))
      == True) {
    FileName=strlen(OutputFileNameOnly);
    DirNameLength=strlen(tmpPath);
    if ((DirNameLength + FileName + 1) <= sizeof(outputfilename)) {
      strcpy(outputfilename,tmpPath);
      strcat(outputfilename,"XmasPathTracer.dat");
    }
  }
}

unsigned int IsShowedExecutionPath(void)
     /*
     ** Return if the PathExecution shell is visible
     */
{
  return(CreatedExecutionPath);
}

unsigned int IsActiveExecutionPath(void)
     /*
     ** Return the ExecutionPath mode
     */
{
  return(PATHEXECUTION);
}

void ActiveExecutionPath(int mode)
     /*
     ** Make active or unprime the ExecutionPath module
     */
{
  PATHEXECUTION=mode;
  UpdateStatus();
}

/*
** PLEASE PAY ATTENTION !!!
** GUI CODE BELOW THIS LINES
*/

static void CreateExecutionPath(void)
     /*
     ** reate the widgets for the ExecutionPath
     */
{
  static Widget form;
  static Widget done,delete,saveas;
  XtTranslations Translations; /* to parse the translation table */

  xmasExecutionPath = XtVaCreatePopupShell(
	"xmasPathTracer",
	transientShellWidgetClass,
	topLevel,NULL);
  form = XtVaCreateManagedWidget(
	"form",
	formWidgetClass,
	xmasExecutionPath,
	NULL);
  status = XtVaCreateManagedWidget(
	"status",
        labelWidgetClass,
	form,
	XtNwidth,400,
	NULL);
  show_text = XtVaCreateManagedWidget(
        "show_text",
	asciiTextWidgetClass,
	form,
	XtNdisplayCaret,False,
	XtNeditType,XawtextEdit,
	XtNwrap,XawtextWrapNever,
	XtNscrollVertical,XawtextScrollWhenNeeded,
	XtNscrollHorizontal,XawtextScrollWhenNeeded,
	XtNwidth,400,
	XtNheight,400,
	XtNfromVert,status,
	NULL);
  istr_counted = XtVaCreateManagedWidget(
	"istrcounted",
	labelWidgetClass,
	form,
	XtNfromVert,show_text,
	XtNwidth,400,
	NULL); 
  done = XtVaCreateManagedWidget(
	"done",
	commandWidgetClass,
	form,
	XtNfromVert,istr_counted,
	NULL);
  delete = XtVaCreateManagedWidget(
	"delete",
	commandWidgetClass,
	form,
	XtNfromVert,istr_counted,
	XtNfromHoriz,done,
	NULL);
  saveas = XtVaCreateManagedWidget(
	"saveas",
	commandWidgetClass,
	form,
	XtNfromVert,istr_counted,
	XtNfromHoriz,delete,
	NULL);

  XtAddCallback(done,XtNcallback,PopdownEdit,(XtPointer)0);
  XtAddCallback(delete,XtNcallback,DeleteList,(XtPointer)0);
  XtAddCallback(saveas,XtNcallback,Save,(XtPointer)0);

  Translations = XtParseTranslationTable
     ("<KeyPress>Return: no-op(RingBell)\n\
       Ctrl<KeyPress>: no-op(RingBell)");
  XtOverrideTranslations(show_text, Translations);

  CreatedExecutionPath=True;
}

void PopupExecutionPath(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /*
     ** Popup the ExecutionPath, if need create it
     */
{
  if (!CreatedExecutionPath)
    CreateExecutionPath();
  Calculate_relative_shell_position_and_set(xmasExecutionPath);
  XtPopup(xmasExecutionPath,XtGrabNone);
  UpdateShowText();
  UpdateStatus(); /* not override the status label dimension */
}

/*
** Callback
*/
static void PopdownEdit(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Simply popdown the path execution
     */
{
  XtPopdown(xmasExecutionPath);
}

static int generic_text_reset(Widget text, char *ptr, int start, int end, unsigned int length)
     /*
     ** Erase the widget text and place the string referred by ptr
     */
{
  XawTextBlock add_block;

  add_block.firstPos=0;
  add_block.format=XawFmt8Bit;
  add_block.length=length;
  add_block.ptr=ptr;
  if (end == -1) {
    String oldtext;

    XtVaGetValues(text,XtNstring,&oldtext,NULL);
    XawTextReplace(text,0,(XawTextPosition)strlen(oldtext),&add_block);
  }
  else
    XawTextReplace(text,start,end,&add_block);
  return(True);
}

void UpdateShowText(void)
     /*
     ** Update the unroll path execution widget
     */
{
  /*clock_t begin,end;*/
  FILE *fp;
  unsigned int pcvalue;
  int i;
  String text;
  char istr_string[32];
  char *outputString,*punt;

  outputString=malloc(ChunckSize*rowOutputLength*sizeof(char));
  *outputString='\0'; /* otherwise outputString is garbage
			 for zero long execution */
  punt=outputString;
  /*begin=times(NULL);*/
  switch (StorageMedia) {
  case MediaMemory: 
    for (i=0; i<ChunckIndex; i++) {
      Istr2String((unsigned int)polreadwithoutstat(PEqueue[i]),istr_string);
      sprintf(punt,"PC 0x%-3x<=> %s\n",PEqueue[i],istr_string);
      punt+=strlen(punt);
    }
    break;
  case MediaDisk:
    if (((fp=fopen(outputfilename,"r")) != NULL) && NewTracing) {
      while (fscanf(fp,"%x",&pcvalue) != EOF) {
	Istr2String((unsigned int)polreadwithoutstat(pcvalue),istr_string);
	sprintf(punt,"PC 0x%-3x<=> %s\n",pcvalue,istr_string);
	punt+=strlen(punt);
      }
      fclose(fp);
      unlink(outputfilename);
    }
    break;
  }
  XawTextDisableRedisplay(show_text);
  XtVaGetValues(show_text,XtNstring,&text,NULL);
  /* add in tail */
     /*generic_text_reset(show_text,outputString,strlen(text),strlen(text),punt-outputString);*/
  /* add in head */
  generic_text_reset(show_text,outputString,0,0,punt-outputString);
  XawTextEnableRedisplay(show_text);
  free(outputString);
  /*end=times(NULL);
    mdebug("elapsed time:",NULL,end-begin);*/
  exec_counter+=ChunckIndex;
  ChunckIndex=0;
  ChunckSize=1;
  PEqueue=(unsigned int *)realloc(PEqueue,ChunckSize*sizeof(unsigned int));
  UpdateIstrCounted();
}

/*
** Callback
*/
static void DeleteList(Widget w, XtPointer client_data, XtPointer call_data)
{
  generic_text_reset(show_text,"",0,-1,strlen(""));
  exec_counter=0;
  UpdateIstrCounted();
}

static void UpdateStatus(void)
     /*
     ** Update the status label
     */
{
  if (CreatedExecutionPath == True) {
    if (PATHEXECUTION == True)
      XtVaSetValues(status,XtNlabel,"Tracing Active",NULL);
    else
      XtVaSetValues(status,XtNlabel,"Tracing Disabled",NULL);
  }
}

static void UpdateIstrCounted(void)
     /*
     ** Update the number of PC widget
     */
{
  char row[32];

  sprintf(row,"istructions counted: %-5d",exec_counter);
  XtVaSetValues(istr_counted, XtNlabel, row,NULL);
}

static int Add2List(unsigned int PC)
     /*
     ** Add the PC to the list
     */
{
  if (PEqueue == NULL) {
    PEqueue=(unsigned int *)realloc(PEqueue,ChunckSize*sizeof(unsigned int));
    PEqueue[ChunckIndex++]=PC;
  }
  else {
    if (ChunckIndex < ChunckSize) {
      PEqueue[ChunckIndex++]=PC;
      return(True); /* speed up to +1.3% */
    }
    else {
      PEqueue=(unsigned int *)realloc(PEqueue,(ChunckSize*=2)*sizeof(unsigned int));
      PEqueue[ChunckIndex++]=PC;
    }
  }
  return(True);
}

/*  static void showlistPC(void) */
     /*
     ** Show all the PC listed from the last RUN widget event
     */
/*  { */
/*    int i=0; */

/*    printf("-----------------START------------------\n"); */
/*    for (i=0; i<ChunckIndex; i++) */
/*      printf("postion %d: value %d\n",i,PEqueue[i]); */
/*  } */

/*
** Callback
*/
static void Save(Widget w, XtPointer client_data, XtPointer call_data)
     /*
     ** Call the OpenSave shell to choose the file name
     */
{
  PopupOpenSave(topLevel,"Save as:",Saver); /* to pop up in the middle
					       of Xmas */
}

static void Saver(String filename)
     /*
     ** Do the hard work to save into a file
     */
{
  FILE *fp;
  String text;
  char Date[32],Clock[32];

  if ((fp=fopen(filename,"w")) == NULL)
    ErrorMessage(PathExecFileWrong,filename,NULL,0);
  else {
    XtVaGetValues(show_text,XtNstring,&text,NULL);
    DateTime(Date,Clock);
    fprintf(fp,"Date: %s\nTime: %s\n\n",Date,Clock);
    fwrite(text,strlen(text),1,fp);
    fclose(fp);
  }
}

void PEMessageChangeStatus(char *message,char *newstatus)
     /*
     ** Trace a message when some cache controls are changed
     */
{
  String text;
  char *label;

  if ((PATHEXECUTION  == True) && (CreatedExecutionPath == True)) {
    if (newstatus != NULL) {
      label=malloc(strlen(message)+strlen(newstatus)+3);
      strcpy(label,message);
      strcat(label,newstatus);
    }
    else {
      label=malloc(strlen(message)+3);
      strcpy(label,message);
    }
    strcat(label,"\n");

    XawTextDisableRedisplay(show_text);
    XtVaGetValues(show_text,XtNstring,&text,NULL);
    generic_text_reset(show_text,label,0,0,strlen(label));
    XawTextEnableRedisplay(show_text);

    free(label);
  }
}

static int Add2File(unsigned int PC)
     /*
     ** Store the PC in a file 
     */
{
  fprintf(outputfile,"%x ",PC);
  ChunckSize=++ChunckIndex;
  return(True);
}

int StartPathTracer(void)
     /*
     ** Set the environment before the tracer start
     */
{
  char RetValue[MediumBufferLength];

  unlink(outputfilename);

  getOptionFromName(PathTracerPreferencesFile,PathTracerStoreOpt,RetValue,sizeof(RetValue));
  if (TestOptionValueFirstChar(RetValue,'M') == True)
    StorageMedia=MediaMemory;
  else
    StorageMedia=MediaDisk;
  switch(StorageMedia) {
  case MediaDisk:
    outputfile=fopen(outputfilename,"w+");
    NewTracing=True;
    TracePC=Add2File;
    break;
  case MediaMemory:
    TracePC=Add2List;
    break;
  }
  return(True);
}

int StopPathTracer(void)
     /*
     ** Close the environment after a tracer run
     */
{
  switch(StorageMedia) {
  case MediaDisk:
    fclose(outputfile);
    break;
  }
  return(True);
}
