/*	
** Xmas (X memory access simulator) is a little memory (+cache) simulator.
** Copyright (C) 1997/98/99/00  <Paolo Dovera pd514314@silab.dsi.unimi.it>
**                              <Paolo Dovera pdovera@tiscalinet.it>

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

** Send any bug reports or comments to pd514314@silab.dsi.unimi.it
**                                     pdovera@tiscalinet.it
*/

/*
** generals include
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
** Intrinsics Definitions
*/

/*
** include of application modules and images
*/
#include "general_support.h"
#include "opthandler.h"
#include "policy.h"
#include "cache.h"
#include "esp.h"
#include "global.h"

/*
** Public include file for widgets I actually use in this file.
*/

/* BEGIN LOCAL DEFINE */
#define MinWay 2 /* min way allowed */
#define MaxWay 8 /* max way allowed */

/*
** define for PolicyMode Repeater
*/
#define DirectModeString  "Direct"
#define FullyModeString   "Fully"
#define NwayModeString    "Nway"
#define KDirectModeString "KDirect"
/* END LOCAL DEFINE */

/* BEGIN IMPORTED GLOABAL VARIABLES */
/* END IMPORTED GLOABAL VARIABLES */

/* BEGIN EXPORTABLE GLOBAL VARIABLES*/
/* END EXPORTABLE GLOBAL VARIABLES */

/* BEGIN STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */
static unsigned int Set;
static unsigned int EntryCache;
static unsigned int TipoPolitica=Nway; /* Direct v Nway v Fully */
static unsigned int Way=MinWay;        /* startup value */
/* END STATIC GLOBAL VARIABLES (PRIVATE VARIABLES) */

void inizializza_politica(void)
     /*
     ** Initialize the policy status
     */
{
  EntryCache=(unsigned int)esp(2,getBitMemoryCache());
  LoadPoliticaStatus(); /* load the policy controls, override defaults value */
  Set=EntryCache/Way;
}

int polread(unsigned int AddressMem)
     /*
     ** Return the data at AddressMem according with the policy
     */
{
  return(cacheread(AddressMem,getpolpos(AddressMem,True)));
}

void polwrite(unsigned int AddressMem, int MemValue)
     /*
     ** AddressMem defined, Value defined
     ** write the Value at address AddressMem, choose the entry in cache
     ** according with the policy
     */
{
  cachewrite(AddressMem,getpolpos(AddressMem,True),MemValue);
}

unsigned int getpolpos(unsigned int AddressMem, unsigned int WithReplacement)
     /*
     ** AddressMem defined
     ** return the position in cache in accord with the policy
     */
{
  unsigned int position;
  unsigned int begin,end;
  unsigned int FoundInCache=False;

  switch (TipoPolitica)
    {
    case KDirect:
    case Direct : return(AddressMem % EntryCache);
    case Fully  : begin=0; end=EntryCache;
		  break;
    case Nway   : begin=(AddressMem%Set)*Way; end=begin+Way;
		  break;
    default: printf("ERROR: TipoPolitica not initialized in getpolpos\n");
             exit(EXIT_FAILURE);
    }
  for(position=begin; position < end; position++)
    /* look up in cache only for Fully or Nway X */
    if (tag(position) == AddressMem && (valid(position) == True)) {
      FoundInCache=True;
      break; /* remain in cache */
    }
  if (FoundInCache == False)
    position=replace(begin,end);
  if (WithReplacement == True)
    Replacement(position,begin,end);
  return(position);
}

void polwritewithoutstat(unsigned int AddressMem, int Value)
     /*
     ** AddressMem definito, Value definito
     ** write Value at address AddressMem
     ** in cache choose the entry for the policy without update the statistics
     */
{
  (void)cacheWriteNoStat(AddressMem,getpolpos(AddressMem,True),Value);
}

int polreadwithoutstat(unsigned int AddressMem)
     /*
     ** AddressMem definito
     ** read the value at AddressMem
     ** in cache choose the entry for the policy without update the statistics
     */
{
  int dummy=0;

  return(cacheReadNoStat(AddressMem,getpolpos(AddressMem,False),&dummy));
}

unsigned int getTipoPolitica(void)
     /*
     ** Return the kind of policy
     */
{
  return(TipoPolitica);
}

char *getStringPolicy(void)
     /*
     ** Return the associated string
     */
{
  static char etichetta[MaxStringLength];

  switch (TipoPolitica) {
  case Direct: return(DirectModeString); break;
  case Fully: return(FullyModeString); break;
  case Nway: sprintf(etichetta,"%s %d",NwayModeString,Way);
    return(etichetta); break;
  case KDirect: return(KDirectModeString); break;
  default : return(NULL);
  }
}

unsigned int setTipoPolitica(unsigned int newTipoPolitica)
     /*
     ** newTipoPolitica defined
     ** set the kind of policy
     */
{
  if ((newTipoPolitica == Direct) || (newTipoPolitica == Fully) ||
      (newTipoPolitica == Nway) || (newTipoPolitica == KDirect)) {
    TipoPolitica=newTipoPolitica;
    return(True);
  }
  else
    return(False);
}
    
unsigned int getWay(void)
     /*
     ** Return the number of way
     */
{
  return(Way);
}

unsigned int setWay(unsigned int newWay)
     /*
     ** newWay >= MinWay & newWay <= MaxWay & TipoPolitica == NWay
     ** recalculate the policy status for the new number of way
     */
{
  if ((newWay >= MinWay) && (newWay <= MaxWay) && (TipoPolitica == Nway)) {
    Way=newWay;
    Set=EntryCache/Way;
    return(True);
  }
  else
    return(False);
}

int LoadPoliticaStatus(void)
     /*
     ** load the policy controls
     */
{
  FILE *fp;
  char bufferline[MediumBufferLength],XmasDir[XmasDirLength],filename[FileNameLength];
  unsigned int policy,way;

  XmasDirStatus(XmasDir,sizeof(XmasDir));
  sprintf(filename,"%s/%s",XmasDir,CacheSaveDataFile);
  if ((fp=fopen(filename,"r")) != NULL) {
    while (fgets(bufferline,sizeof(bufferline),fp) != NULL) {
      if (strncmp(bufferline,CachePolicyOptString,strlen(CachePolicyOptString)) == 0)
	if (sscanf(bufferline+strlen(CachePolicyOptString),"%x",&policy) == 1) {
	    setTipoPolitica(policy);
	    continue;
	}
      if (strncmp(bufferline,CacheWayOptString,strlen(CacheWayOptString)) == 0)
	if (sscanf(bufferline+strlen(CacheWayOptString),"%x",&way) == 1) {
	  setWay(way);
	  continue;
	}
    }
    fclose(fp);
    return(True);
  }
  return(False);
}

int SavePoliticaStatus(char *XmasDir)
     /*
     ** save the policy controls
     */
{
  FILE *fp;
  char filename[FileNameLength];

  sprintf(filename,"%s/%s",XmasDir,CacheSaveDataFile);
  if ((fp=fopen(filename,"a")) != NULL) { /* add to cache data file */
    fprintf(fp,"%s 0x%x\n",CachePolicyOptString,TipoPolitica);
    fprintf(fp,"%s 0x%x\n",CacheWayOptString,Way);
    fclose(fp);
    return(True);
  }
  return(False);
}
